/* Written by Gerson Kurz (gerson.kurz@t-online.de). Freeware. Enjoy. */

#include "precomp.h"
#include "FileWriteCache.h"

FileWriteCache::FileWriteCache()
{
    m_hFile = INVALID_HANDLE_VALUE;
    m_dwMaxCache = sizeof(m_szCache)-2;
    m_dwOffset = 0;
} 

BOOL FileWriteCache::IsOpen()
{
    return (m_hFile != INVALID_HANDLE_VALUE);
}

FileWriteCache::~FileWriteCache()
{
    Close();
}

BOOL FileWriteCache::Open( LPCSTR lpszFilename )
{
    m_hFile = CreateFile(lpszFilename ,GENERIC_WRITE,FILE_SHARE_READ,0,CREATE_ALWAYS,0,0);
    if( m_hFile != INVALID_HANDLE_VALUE )
        return TRUE;

    return FALSE;
}

void FileWriteCache::Write( LPCSTR lpszMessage )
{
    DWORD dwLength = strlen(lpszMessage), dwUnused;

    if( m_dwOffset + dwLength < m_dwMaxCache )
    {
        CopyMemory( &(m_szCache[m_dwOffset]), lpszMessage, dwLength );
        m_dwOffset += dwLength;
    }
    else
    {
        if( m_dwOffset )
        {
            WriteFile( m_hFile, m_szCache, m_dwOffset, &dwUnused, 0 );
            m_dwOffset = 0;
        }
        if( dwLength >= m_dwMaxCache )
        {
            WriteFile( m_hFile, lpszMessage, dwLength, &dwUnused, 0 );
        }
        else
        {
            CopyMemory( &(m_szCache[m_dwOffset]), lpszMessage, dwLength );
            m_dwOffset += dwLength;
        }
    }
}

void FileWriteCache::Flush()
{
    if( m_dwOffset )
    {
        DWORD dwUnused;
        WriteFile( m_hFile, m_szCache, m_dwOffset, &dwUnused, 0 );
        m_dwOffset = 0;
    }
}

void FileWriteCache::Close()
{
    if( m_hFile != INVALID_HANDLE_VALUE )
    {
        if( m_dwOffset )
        {
            DWORD dwUnused;
            WriteFile( m_hFile, m_szCache, m_dwOffset, &dwUnused, 0 );
        }
        CloseHandle( m_hFile );
        m_hFile = INVALID_HANDLE_VALUE;
    }
}
