#include "precomp.h"
#include "pcomspytools.H"
#include "InprocServerSpy.h"

CHAR szInprocDllName[256];

typedef HRESULT (STDAPICALLTYPE* LPFNDllCanUnloadNow)(void);
typedef HRESULT (STDAPICALLTYPE* LPFNDllGetClassObject)(REFCLSID rclsid, REFIID riid, LPVOID *ppvOut);
typedef HRESULT (STDAPICALLTYPE* LPFNDllRegisterServer)();
typedef HRESULT (STDAPICALLTYPE* LPFNDllUnregisterServer)();

static LPFNDllCanUnloadNow myDllCanUnloadNow;
static LPFNDllGetClassObject myDllGetClassObject;
static LPFNDllRegisterServer myDllRegisterServer;
static LPFNDllUnregisterServer myDllUnregisterServer;
static HMODULE hLibrary;

class SpyUnknown : public IUnknown
    {
    public:
        SpyUnknown(IUnknown* pUnknown);
		STDMETHODIMP QueryInterface (REFIID iid, void **ppv);
		STDMETHODIMP_(ULONG) AddRef(void);
		STDMETHODIMP_(ULONG) Release(void);

		LONG m_lRefCount;
        IUnknown* m_pUnknown;
    };

SpyUnknown::SpyUnknown(IUnknown* pUnknown)
{
    m_lRefCount = 1; // assume owner pUnknown has already been registered !!!
    m_pUnknown = pUnknown;
}

STDMETHODIMP SpyUnknown::QueryInterface (REFIID riid, void **ppv)
{
    HRESULT hResult = m_pUnknown->QueryInterface(riid, ppv);
    WriteComTraceA( "IUnknown::QueryInterface(%s) returns %s\n", LookupInterfaceA(riid), LookupHresultA(hResult) );
    return hResult;
}

STDMETHODIMP_(ULONG) SpyUnknown::AddRef(void)
{
    m_pUnknown->AddRef();
	return InterlockedIncrement(&m_lRefCount);
}

STDMETHODIMP_(ULONG) SpyUnknown::Release(void)
{
    m_pUnknown->Release();
	if( InterlockedDecrement(&m_lRefCount) == 0 )
    {
		delete this;
	}
	return 0;
}

class SpyClassFactory : public IClassFactory
    {
    public:
		SpyClassFactory(IClassFactory* pClassFactory);
		virtual ~SpyClassFactory();

		STDMETHODIMP QueryInterface (REFIID iid, void **ppv);
		STDMETHODIMP_(ULONG) AddRef(void);
		STDMETHODIMP_(ULONG) Release(void);
        HRESULT STDMETHODCALLTYPE CreateInstance( IUnknown *pUnkOuter, REFIID riid, void **ppvObject);
        HRESULT STDMETHODCALLTYPE LockServer( BOOL fLock );

        IClassFactory* m_pClassFactory;
		LONG m_lRefCount;
    };

SpyClassFactory::SpyClassFactory(IClassFactory* pClassFactory)
{
    m_pClassFactory = pClassFactory;
    m_lRefCount = 1;
}
SpyClassFactory::~SpyClassFactory()
{
    m_pClassFactory->Release();
}

HRESULT STDMETHODCALLTYPE SpyClassFactory::QueryInterface( REFIID riid, void** ppvObj )
{
    WriteComTraceA( "IClassFactory::QueryInterface(%s) called\n", LookupInterfaceA(riid) );

    *ppvObj = NULL;
    if( (IID_IUnknown == riid) || (IID_IClassFactory == riid) )
    {
        *ppvObj = (LPUNKNOWN)this;
        WriteComTraceA( "IClassFactory::QueryInterface(%s) returns S_OK with spy object\n", LookupInterfaceA(riid) );
        return S_OK;
    }
    HRESULT hResult = m_pClassFactory->QueryInterface(riid,ppvObj);
    WriteComTraceA( "IClassFactory::QueryInterface(%s) returns %s\n", LookupInterfaceA(riid), LookupHresultA(hResult) );
    return hResult;
}

ULONG STDMETHODCALLTYPE SpyClassFactory::AddRef()
{
    m_pClassFactory->AddRef();
    return InterlockedIncrement(&m_lRefCount);
}

ULONG STDMETHODCALLTYPE SpyClassFactory::Release()
{
    m_pClassFactory->Release();
    if( InterlockedDecrement(&m_lRefCount) == 0 )
    {
        delete this;
    }
    return m_lRefCount;
}

HRESULT STDMETHODCALLTYPE SpyClassFactory::CreateInstance( IUnknown *pUnkOuter, REFIID riid, void **ppvObject)
{
    WriteComTraceA( "IClassFactory::CreateInstance(%s) called\n", LookupInterfaceA(riid) );
    HRESULT hResult = m_pClassFactory->CreateInstance( pUnkOuter, riid, ppvObject );
    if( SUCCEEDED(hResult) && IsEqualGUID( riid, IID_IUnknown ) )
    {
        IUnknown* pUnk = *(IUnknown**) ppvObject;
        SpyUnknown* pSpy = new SpyUnknown( pUnk );
        if( pSpy )
        {
            DumpAllInterfaces(pUnk,"newly created instance");

            *(IUnknown**) ppvObject = pSpy;
        }
    }
    WriteComTraceA( "IClassFactory::CreateInstance(%s) returns %s\n", LookupInterfaceA(riid), LookupHresultA(hResult) );
    return hResult;
}

HRESULT STDMETHODCALLTYPE SpyClassFactory::LockServer( BOOL fLock )
{
    HRESULT hResult = m_pClassFactory->LockServer( fLock );
    WriteComTraceA( "IClassFactory::LockServer(%ld) returns %s\n", fLock, LookupHresultA(hResult) );
    return hResult;
}

void MakeSureIsLoaded()
{
    hLibrary = LoadLibrary( szInprocDllName );
    if( hLibrary )
    {
        myDllCanUnloadNow = (LPFNDllCanUnloadNow) GetProcAddress( hLibrary, "DllCanUnloadNow" );
        myDllGetClassObject = (LPFNDllGetClassObject) GetProcAddress( hLibrary, "DllGetClassObject" );
        myDllRegisterServer = (LPFNDllRegisterServer) GetProcAddress( hLibrary, "DllRegisterServer" );
        myDllUnregisterServer = (LPFNDllUnregisterServer) GetProcAddress(  hLibrary, "DllUnregisterServer" );
    }
}

STDAPI QOleAutomationCanUnloadNow(void)
{
    MakeSureIsLoaded();

    if( myDllCanUnloadNow )
    {
        BOOL bCanUnloadNow = myDllCanUnloadNow();
        if( bCanUnloadNow )
        {
            FreeLibrary( hLibrary );
            hLibrary = 0;
        }
        return bCanUnloadNow;
    }
    return FALSE;
}

STDAPI QOleAutomationGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppvOut)
{
    MakeSureIsLoaded();

    if( myDllGetClassObject )
    {
        HRESULT hResult = myDllGetClassObject(rclsid, riid, ppvOut);
        if( SUCCEEDED(hResult) && IsEqualGUID( riid, IID_IClassFactory ) )
        {
            SpyClassFactory* pSpy = new SpyClassFactory( *(IClassFactory**) ppvOut );
            if( pSpy )
            {
                *(IClassFactory**) ppvOut = pSpy;
            }
        }
        return hResult;
    }
    return E_NOTIMPL;
}

STDAPI QOleAutomationRegisterServer()
{
    MakeSureIsLoaded();
    if( myDllRegisterServer )
    {
        return myDllRegisterServer();
    }
    return S_OK;
}

STDAPI QOleAutomationUnregisterServer()
{
    MakeSureIsLoaded();
    if( myDllUnregisterServer )
    {
        return myDllUnregisterServer();
    }
    return S_OK;
}
