#ifndef PCOMSPYTOOLS_H
#define PCOMSPYTOOLS_H

#ifdef __cplusplus
extern "C" {
#endif

/* 
    This function dumps all interfaces exported by an IUnknown object to the tracefile.
    The string lpszTagline is written to the tracefile to make it easier to identify the object.  
*/
void WINAPI DumpAllInterfaces(IUnknown* p, LPCSTR lpszTagline);

/* 
    This function returns a textual representation of a known IID, CLSID or LIBID. 
*/
LPCSTR WINAPI LookupInterfaceA(REFIID riid);

/* 
    This function returns a textual representation of a known HRESULT. This includes the OLE errorcodes not 
    returned from FormatMessage. If it doesn't find an OLE code, it looks up system error codes. 
*/
LPCSTR WINAPI LookupHresultA(HRESULT hResult);

/*
    You can use this function to write your own text to the com tracefile. 
*/
void WINAPI WriteComTraceA( LPCSTR lpszFormat, ... );

/*
    Write back all buffered trace information
*/
void WINAPI FlushComTracefile();

/*
    This function looks up a MAPI tag
*/
LPCSTR WINAPI GetMapiTagDescriptionA( DWORD dwTag );

/*
    This function returns a MAPI tag associated with a name (e.g. PR_BODY)
*/
ULONG WINAPI LookupMapiTagByNameA( LPCSTR lpszTag );


/*
    This is a helper interface that represents a copy of an IStream object as a BYTE array. 
    (Of course, the whole IStream must fit into memory). Note that you CANNOT use the associated IStream functions,
    attempting to do so will return E_OUTOFMEMORY. The IStream is only valid while the source stream is copied; which is
    hidden from your application.  
*/
class IStreamAsBytes : public IStream   
    {
    public:
        virtual LPBYTE STDMETHODCALLTYPE GetData() = 0;
        virtual DWORD STDMETHODCALLTYPE GetSize() = 0;
    };

/*
    This function will take any incoming IStream object, create a new IStreamAsBytes for it, read its contents and store them in one
    large byte array. You must call IStreamAsBytes::Release() to free the associated memory
*/
BOOL WINAPI CreateStreamAsBytes( IStream* Source, IStreamAsBytes** Target );

#ifdef __cplusplus
} // extern "C" 
#endif

#endif /* PCOMSPYTOOLS_H */


