#include "precomp.h"
#include "pcomspytools.h"
#include "ReadMemoryStream.h"

ReadMemoryStream::ReadMemoryStream()
{
    Last = First = NULL;
    m_lpbAllocated = 0;
    m_lRefCount = 0;
}

ReadMemoryStream::~ReadMemoryStream()
{
    DeleteBlocks();
}

void ReadMemoryStream::DeleteBlocks()
{
    while( First )
    {
        BYTE_BLOCK* Next = First->Next;
        delete First;
        First = Next;
    }
    Last = NULL;
    delete m_lpbAllocated;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::QueryInterface( REFIID riid, void** ppvObj )
{
    *ppvObj = NULL;
    if( (IID_IUnknown == riid) || (IID_ISequentialStream == riid) || (IID_IStream == riid) )
    {
        *ppvObj = (LPUNKNOWN)this;
        return S_OK;
    }
    return E_NOINTERFACE;
}

ULONG STDMETHODCALLTYPE ReadMemoryStream::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

ULONG STDMETHODCALLTYPE ReadMemoryStream::Release()
{
    if( InterlockedDecrement(&m_lRefCount) == 0 )
    {
        delete this;
    }
    return m_lRefCount;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::Read( void *pv, ULONG cb, ULONG *pcbRead )
{
    *pcbRead = 0;
    return E_NOTIMPL;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::Write( const void *pv, ULONG cb, ULONG *pcbWritten )
{
    if( m_lpbAllocated )
        return E_OUTOFMEMORY;

    if( cb )
    {
        BYTE* bData = new BYTE[sizeof(BYTE_BLOCK) + cb];
        if( !bData )
            return E_OUTOFMEMORY;
        BYTE_BLOCK* p = (BYTE_BLOCK*) bData;
        p->Size = cb;
        CopyMemory( p->bData, pv, cb );

        if( Last )
        {
            p->Prev = Last;
            p->Next = 0;
            Last->Next = p;
            Last = p;
        }
        else
        {
            p->Next = p->Prev = 0;
            First = Last = p;
        }
    }
    *pcbWritten = cb;
    return S_OK;
}

DWORD STDMETHODCALLTYPE ReadMemoryStream::GetSize()
{
    return m_dwTotalSize;
}

LPBYTE STDMETHODCALLTYPE ReadMemoryStream::GetData()
{
    if( m_lpbAllocated )
        return m_lpbAllocated;

    if( !First )
        return NULL;

    DWORD dwTotalSize = 0;

    BYTE_BLOCK* p = First;
    while( p )
    {  
        dwTotalSize += p->Size;
        p = p->Next;
    }
    delete m_lpbAllocated;
    m_dwTotalSize = dwTotalSize;
    m_lpbAllocated = new BYTE[dwTotalSize+2];
    if( m_lpbAllocated )
    {
        LPBYTE WritePos = m_lpbAllocated;
        while( First )
        {  
            CopyMemory( WritePos, First->bData, First->Size );
            
            WritePos += First->Size;
            BYTE_BLOCK* Next = First->Next;
            delete First;
            First = Next;
        }
        m_lpbAllocated[dwTotalSize] = 0;
    }
    Last = NULL;
    return m_lpbAllocated;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::Seek( LARGE_INTEGER dlibMove, DWORD dwOrigin, ULARGE_INTEGER *plibNewPosition )
{
    plibNewPosition->QuadPart = 0;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::SetSize( ULARGE_INTEGER libNewSize )
{
    return S_OK;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::CopyTo( IStream *pstm, ULARGE_INTEGER cb, ULARGE_INTEGER *pcbRead, ULARGE_INTEGER *pcbWritten )
{
    return E_NOTIMPL;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::Commit( DWORD grfCommitFlags )
{
    return S_OK;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::Revert()
{
    return S_OK;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::LockRegion( ULARGE_INTEGER libOffset,ULARGE_INTEGER cb,DWORD dwLockType)
{
    return S_OK;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::UnlockRegion( ULARGE_INTEGER libOffset,ULARGE_INTEGER cb,DWORD dwLockType)
{
    return S_OK;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::Stat( STATSTG *pstatstg, DWORD grfStatFlag)
{
    return E_NOTIMPL;
}

HRESULT STDMETHODCALLTYPE ReadMemoryStream::Clone( IStream **ppstm )
{
    return E_NOTIMPL;
}

EXTERN_C BOOL WINAPI CreateStreamAsBytes( IStream* Source, IStreamAsBytes** Target )
{
    ReadMemoryStream* p = new ReadMemoryStream;
    if( !p )
        return FALSE;

    p->AddRef();

    STATSTG stat;
    ULARGE_INTEGER cbRead, cbWritten;

    HRESULT hResult = Source->Stat( &stat, STATFLAG_NONAME );

    if( !SUCCEEDED(hResult) )
        cbRead.QuadPart = 1024 * 1024 * 1024;

    hResult = Source->CopyTo(p,stat.cbSize,&cbRead,&cbWritten);
    if( SUCCEEDED(hResult) )
    {
        *Target = p;
        return TRUE;
    }
    return FALSE;
}
