#ifndef ReadMemoryStream_H
#define ReadMemoryStream_H

typedef struct s_byte_block
{
    s_byte_block* Next;
    s_byte_block* Prev;
    DWORD Size;
    BYTE bData[1]; // actual data follows
} BYTE_BLOCK;


class ReadMemoryStream : public IStreamAsBytes
    {
    public:
        ReadMemoryStream();
        virtual ~ReadMemoryStream();

        // IUnknown methods
        virtual HRESULT STDMETHODCALLTYPE QueryInterface( REFIID riid, void** ppvObject );
        virtual ULONG STDMETHODCALLTYPE AddRef();
        virtual ULONG STDMETHODCALLTYPE Release();
    	
        // ISequentialStream methods
        virtual HRESULT STDMETHODCALLTYPE Read( void *pv, ULONG cb, ULONG *pcbRead );
        virtual HRESULT STDMETHODCALLTYPE Write( const void *pv, ULONG cb, ULONG *pcbWritten );
        virtual HRESULT STDMETHODCALLTYPE Seek( LARGE_INTEGER dlibMove, DWORD dwOrigin, ULARGE_INTEGER *plibNewPosition );
        
        // IStream methods
        virtual HRESULT STDMETHODCALLTYPE SetSize( ULARGE_INTEGER libNewSize );
        virtual HRESULT STDMETHODCALLTYPE CopyTo( IStream *pstm, ULARGE_INTEGER cb, ULARGE_INTEGER *pcbRead, ULARGE_INTEGER *pcbWritten );
        virtual HRESULT STDMETHODCALLTYPE Commit( DWORD grfCommitFlags );
        virtual HRESULT STDMETHODCALLTYPE Revert();
        virtual HRESULT STDMETHODCALLTYPE LockRegion( ULARGE_INTEGER libOffset,ULARGE_INTEGER cb,DWORD dwLockType);
        virtual HRESULT STDMETHODCALLTYPE UnlockRegion( ULARGE_INTEGER libOffset,ULARGE_INTEGER cb,DWORD dwLockType);
        virtual HRESULT STDMETHODCALLTYPE Stat( STATSTG *pstatstg, DWORD grfStatFlag);
        virtual HRESULT STDMETHODCALLTYPE Clone( IStream **ppstm );

        virtual LPBYTE STDMETHODCALLTYPE GetData();
        virtual DWORD STDMETHODCALLTYPE GetSize();

    protected:
        void DeleteBlocks();
        BYTE_BLOCK* First;
        BYTE_BLOCK* Last;
        LPBYTE m_lpbAllocated;
        DWORD m_dwTotalSize;
        LONG m_lRefCount;
    };

#endif // ReadMemoryStream_H
