#include "precomp.h"
#include "pcomspytools.h"
#include "FileWriteCache.h"
#include "InprocServerSpy.h"

FileWriteCache TraceFile;
HANDLE hTraceMutex; // ANSI C promises this to be zero

static CHAR szTraceFileName[256];

void CloseTraceFile()
{
    if( TraceFile.IsOpen() )
    {
        TraceFile.Close();
        CloseHandle( hTraceMutex );
    }
}

void OpenTraceFile()
{
    if( !TraceFile.IsOpen() )
    {
        hTraceMutex = CreateMutex(NULL,FALSE,NULL);
        TraceFile.Open(szTraceFileName);
    }
}

EXTERN_C void WINAPI FlushComTracefile()
{
    if( TraceFile.IsOpen() )
    {
        TraceFile.Flush();
    }
}

EXTERN_C void WINAPI WriteComTraceA( LPCSTR lpszFormat, ... )
{
    CHAR szFormatBuffer[1024];
    va_list args;

    va_start( args, lpszFormat );
    _vsnprintf(szFormatBuffer, sizeof(szFormatBuffer)-1, lpszFormat, args );
    szFormatBuffer[sizeof(szFormatBuffer)-1] = 0;

    OutputDebugString(szFormatBuffer);

    if( hTraceMutex && TraceFile.IsOpen())
    {
        DWORD dwResult = WaitForSingleObject( hTraceMutex, INFINITE );
        if( (dwResult == WAIT_ABANDONED) || (dwResult == WAIT_OBJECT_0) )
        {
            __try
            {
                TraceFile.Write(szFormatBuffer); 
            }
            __except( EXCEPTION_EXECUTE_HANDLER )
            {
            }
            ReleaseMutex( hTraceMutex );
        }
    }
}

EXTERN_C BOOL WINAPI DllMain( HANDLE hInstance, DWORD dwReason, LPVOID )
{
	if( dwReason == DLL_PROCESS_ATTACH )
	{
		DisableThreadLibraryCalls( (HMODULE)hInstance ); 

        // Read configuration from Registry
        HKEY hkRegistry;
        DWORD dwDisposition;

        if( RegCreateKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\p-nand-q.com\\pcomspytools", 0, "", REG_OPTION_NON_VOLATILE,
            KEY_READ|KEY_WRITE, NULL, &hkRegistry, &dwDisposition ) == NO_ERROR )
        {
            DWORD dwData = sizeof(szTraceFileName), dwType;
            LONG lResult;
            
            lResult = RegQueryValueEx(hkRegistry, "TraceFileName", 0, &dwType, (LPBYTE) szTraceFileName, &dwData);
            if( (lResult != S_OK) || (dwType != REG_SZ) )
            {
                strcpy( szTraceFileName, "C:\\PCOMSPY.LOG" );
                RegSetValueEx(hkRegistry, "TraceFileName", 0, REG_SZ, (LPBYTE) szTraceFileName, sizeof(szTraceFileName));
            }
            
            dwData = sizeof(szInprocDllName);
            lResult = RegQueryValueEx(hkRegistry, "SpyOnInprocDllName", 0, &dwType, (LPBYTE) szInprocDllName, &dwData);
            if( (lResult != S_OK) || (dwType != REG_SZ) )
            {
                strcpy( szTraceFileName, "MSCOMM32.OCX" );
                RegSetValueEx(hkRegistry, "SpyOnInprocDllName", 0, REG_SZ, (LPBYTE) szInprocDllName, sizeof(szInprocDllName));
            }
        }
        OpenTraceFile();
	}
    else if( dwReason == DLL_PROCESS_DETACH )
    {
        CloseTraceFile();
    }
	return TRUE;
}
