#include "precomp.h"
#include "mapicode.h"

typedef struct
{
    HRESULT hResult;
    LPCSTR lpszDescription;
} ERRORCODE;

static ERRORCODE allcodes[] = {
{S_OK,"S_OK"},
{S_FALSE,"S_FALSE"},
{SEC_E_OK,"SEC_E_OK"},
{ERROR_FILE_NOT_FOUND,"ERROR_FILE_NOT_FOUND"},
{ERROR_PATH_NOT_FOUND,"ERROR_PATH_NOT_FOUND"},
{ERROR_TOO_MANY_OPEN_FILES,"ERROR_TOO_MANY_OPEN_FILES"},
{ERROR_ACCESS_DENIED,"ERROR_ACCESS_DENIED"},
{ERROR_INVALID_HANDLE,"ERROR_INVALID_HANDLE"},
{ERROR_ARENA_TRASHED,"ERROR_ARENA_TRASHED"},
{ERROR_INVALID_BLOCK,"ERROR_INVALID_BLOCK"},
{ERROR_BAD_ENVIRONMENT,"ERROR_BAD_ENVIRONMENT"},
{ERROR_BAD_FORMAT,"ERROR_BAD_FORMAT"},
{ERROR_INVALID_ACCESS,"ERROR_INVALID_ACCESS"},
{ERROR_INVALID_DATA,"ERROR_INVALID_DATA"},
{ERROR_OUTOFMEMORY,"ERROR_OUTOFMEMORY"},
{ERROR_INVALID_DRIVE,"ERROR_INVALID_DRIVE"},
{ERROR_CURRENT_DIRECTORY,"ERROR_CURRENT_DIRECTORY"},
{ERROR_NOT_SAME_DEVICE,"ERROR_NOT_SAME_DEVICE"},
{ERROR_NO_MORE_FILES,"ERROR_NO_MORE_FILES"},
{ERROR_WRITE_PROTECT,"ERROR_WRITE_PROTECT"},
{ERROR_BAD_UNIT,"ERROR_BAD_UNIT"},
{ERROR_NOT_READY,"ERROR_NOT_READY"},
{ERROR_BAD_COMMAND,"ERROR_BAD_COMMAND"},
{ERROR_CRC,"ERROR_CRC"},
{ERROR_BAD_LENGTH,"ERROR_BAD_LENGTH"},
{ERROR_SEEK,"ERROR_SEEK"},
{ERROR_NOT_DOS_DISK,"ERROR_NOT_DOS_DISK"},
{ERROR_SECTOR_NOT_FOUND,"ERROR_SECTOR_NOT_FOUND"},
{ERROR_OUT_OF_PAPER,"ERROR_OUT_OF_PAPER"},
{ERROR_WRITE_FAULT,"ERROR_WRITE_FAULT"},
{ERROR_READ_FAULT,"ERROR_READ_FAULT"},
{ERROR_GEN_FAILURE,"ERROR_GEN_FAILURE"},
{ERROR_SHARING_VIOLATION,"ERROR_SHARING_VIOLATION"},
{ERROR_LOCK_VIOLATION,"ERROR_LOCK_VIOLATION"},
{ERROR_WRONG_DISK,"ERROR_WRONG_DISK"},
{ERROR_SHARING_BUFFER_EXCEEDED,"ERROR_SHARING_BUFFER_EXCEEDED"},
{ERROR_HANDLE_EOF,"ERROR_HANDLE_EOF"},
{ERROR_HANDLE_DISK_FULL,"ERROR_HANDLE_DISK_FULL"},
{ERROR_NOT_SUPPORTED,"ERROR_NOT_SUPPORTED"},
{ERROR_REM_NOT_LIST,"ERROR_REM_NOT_LIST"},
{ERROR_DUP_NAME,"ERROR_DUP_NAME"},
{ERROR_BAD_NETPATH,"ERROR_BAD_NETPATH"},
{ERROR_NETWORK_BUSY,"ERROR_NETWORK_BUSY"},
{ERROR_TOO_MANY_CMDS,"ERROR_TOO_MANY_CMDS"},
{ERROR_ADAP_HDW_ERR,"ERROR_ADAP_HDW_ERR"},
{ERROR_BAD_NET_RESP,"ERROR_BAD_NET_RESP"},
{ERROR_UNEXP_NET_ERR,"ERROR_UNEXP_NET_ERR"},
{ERROR_BAD_REM_ADAP,"ERROR_BAD_REM_ADAP"},
{ERROR_PRINTQ_FULL,"ERROR_PRINTQ_FULL"},
{ERROR_NO_SPOOL_SPACE,"ERROR_NO_SPOOL_SPACE"},
{ERROR_PRINT_CANCELLED,"ERROR_PRINT_CANCELLED"},
{ERROR_NETNAME_DELETED,"ERROR_NETNAME_DELETED"},
{ERROR_NETWORK_ACCESS_DENIED,"ERROR_NETWORK_ACCESS_DENIED"},
{ERROR_BAD_DEV_TYPE,"ERROR_BAD_DEV_TYPE"},
{ERROR_BAD_NET_NAME,"ERROR_BAD_NET_NAME"},
{ERROR_TOO_MANY_NAMES,"ERROR_TOO_MANY_NAMES"},
{ERROR_TOO_MANY_SESS,"ERROR_TOO_MANY_SESS"},
{ERROR_SHARING_PAUSED,"ERROR_SHARING_PAUSED"},
{ERROR_REQ_NOT_ACCEP,"ERROR_REQ_NOT_ACCEP"},
{ERROR_REDIR_PAUSED,"ERROR_REDIR_PAUSED"},
{ERROR_FILE_EXISTS,"ERROR_FILE_EXISTS"},
{ERROR_CANNOT_MAKE,"ERROR_CANNOT_MAKE"},
{ERROR_FAIL_I24,"ERROR_FAIL_I24"},
{ERROR_OUT_OF_STRUCTURES,"ERROR_OUT_OF_STRUCTURES"},
{ERROR_ALREADY_ASSIGNED,"ERROR_ALREADY_ASSIGNED"},
{ERROR_INVALID_PASSWORD,"ERROR_INVALID_PASSWORD"},
{ERROR_NET_WRITE_FAULT,"ERROR_NET_WRITE_FAULT"},
{ERROR_NO_PROC_SLOTS,"ERROR_NO_PROC_SLOTS"},
{ERROR_TOO_MANY_SEMAPHORES,"ERROR_TOO_MANY_SEMAPHORES"},
{ERROR_EXCL_SEM_ALREADY_OWNED,"ERROR_EXCL_SEM_ALREADY_OWNED"},
{ERROR_SEM_IS_SET,"ERROR_SEM_IS_SET"},
{ERROR_TOO_MANY_SEM_REQUESTS,"ERROR_TOO_MANY_SEM_REQUESTS"},
{ERROR_INVALID_AT_INTERRUPT_TIME,"ERROR_INVALID_AT_INTERRUPT_TIME"},
{ERROR_SEM_OWNER_DIED,"ERROR_SEM_OWNER_DIED"},
{ERROR_SEM_USER_LIMIT,"ERROR_SEM_USER_LIMIT"},
{ERROR_DISK_CHANGE,"ERROR_DISK_CHANGE"},
{ERROR_DRIVE_LOCKED,"ERROR_DRIVE_LOCKED"},
{ERROR_BROKEN_PIPE,"ERROR_BROKEN_PIPE"},
{ERROR_OPEN_FAILED,"ERROR_OPEN_FAILED"},
{ERROR_BUFFER_OVERFLOW,"ERROR_BUFFER_OVERFLOW"},
{ERROR_DISK_FULL,"ERROR_DISK_FULL"},
{ERROR_NO_MORE_SEARCH_HANDLES,"ERROR_NO_MORE_SEARCH_HANDLES"},
{ERROR_INVALID_TARGET_HANDLE,"ERROR_INVALID_TARGET_HANDLE"},
{ERROR_INVALID_CATEGORY,"ERROR_INVALID_CATEGORY"},
{ERROR_INVALID_VERIFY_SWITCH,"ERROR_INVALID_VERIFY_SWITCH"},
{ERROR_BAD_DRIVER_LEVEL,"ERROR_BAD_DRIVER_LEVEL"},
{ERROR_CALL_NOT_IMPLEMENTED,"ERROR_CALL_NOT_IMPLEMENTED"},
{ERROR_SEM_TIMEOUT,"ERROR_SEM_TIMEOUT"},
{ERROR_INVALID_LEVEL,"ERROR_INVALID_LEVEL"},
{ERROR_NO_VOLUME_LABEL,"ERROR_NO_VOLUME_LABEL"},
{ERROR_MOD_NOT_FOUND,"ERROR_MOD_NOT_FOUND"},
{ERROR_PROC_NOT_FOUND,"ERROR_PROC_NOT_FOUND"},
{ERROR_WAIT_NO_CHILDREN,"ERROR_WAIT_NO_CHILDREN"},
{ERROR_CHILD_NOT_COMPLETE,"ERROR_CHILD_NOT_COMPLETE"},
{ERROR_DIRECT_ACCESS_HANDLE,"ERROR_DIRECT_ACCESS_HANDLE"},
{ERROR_NEGATIVE_SEEK,"ERROR_NEGATIVE_SEEK"},
{ERROR_SEEK_ON_DEVICE,"ERROR_SEEK_ON_DEVICE"},
{ERROR_IS_JOIN_TARGET,"ERROR_IS_JOIN_TARGET"},
{ERROR_IS_JOINED,"ERROR_IS_JOINED"},
{ERROR_IS_SUBSTED,"ERROR_IS_SUBSTED"},
{ERROR_NOT_JOINED,"ERROR_NOT_JOINED"},
{ERROR_NOT_SUBSTED,"ERROR_NOT_SUBSTED"},
{ERROR_JOIN_TO_JOIN,"ERROR_JOIN_TO_JOIN"},
{ERROR_SUBST_TO_SUBST,"ERROR_SUBST_TO_SUBST"},
{ERROR_JOIN_TO_SUBST,"ERROR_JOIN_TO_SUBST"},
{ERROR_SUBST_TO_JOIN,"ERROR_SUBST_TO_JOIN"},
{ERROR_BUSY_DRIVE,"ERROR_BUSY_DRIVE"},
{ERROR_SAME_DRIVE,"ERROR_SAME_DRIVE"},
{ERROR_DIR_NOT_ROOT,"ERROR_DIR_NOT_ROOT"},
{ERROR_DIR_NOT_EMPTY,"ERROR_DIR_NOT_EMPTY"},
{ERROR_IS_SUBST_PATH,"ERROR_IS_SUBST_PATH"},
{ERROR_IS_JOIN_PATH,"ERROR_IS_JOIN_PATH"},
{ERROR_PATH_BUSY,"ERROR_PATH_BUSY"},
{ERROR_IS_SUBST_TARGET,"ERROR_IS_SUBST_TARGET"},
{ERROR_SYSTEM_TRACE,"ERROR_SYSTEM_TRACE"},
{ERROR_INVALID_EVENT_COUNT,"ERROR_INVALID_EVENT_COUNT"},
{ERROR_TOO_MANY_MUXWAITERS,"ERROR_TOO_MANY_MUXWAITERS"},
{ERROR_INVALID_LIST_FORMAT,"ERROR_INVALID_LIST_FORMAT"},
{ERROR_LABEL_TOO_LONG,"ERROR_LABEL_TOO_LONG"},
{ERROR_TOO_MANY_TCBS,"ERROR_TOO_MANY_TCBS"},
{ERROR_SIGNAL_REFUSED,"ERROR_SIGNAL_REFUSED"},
{ERROR_DISCARDED,"ERROR_DISCARDED"},
{ERROR_NOT_LOCKED,"ERROR_NOT_LOCKED"},
{ERROR_BAD_THREADID_ADDR,"ERROR_BAD_THREADID_ADDR"},
{ERROR_BAD_ARGUMENTS,"ERROR_BAD_ARGUMENTS"},
{ERROR_BAD_PATHNAME,"ERROR_BAD_PATHNAME"},
{ERROR_SIGNAL_PENDING,"ERROR_SIGNAL_PENDING"},
{ERROR_MAX_THRDS_REACHED,"ERROR_MAX_THRDS_REACHED"},
{ERROR_LOCK_FAILED,"ERROR_LOCK_FAILED"},
{ERROR_CANCEL_VIOLATION,"ERROR_CANCEL_VIOLATION"},
{ERROR_ATOMIC_LOCKS_NOT_SUPPORTED,"ERROR_ATOMIC_LOCKS_NOT_SUPPORTED"},
{ERROR_INVALID_SEGMENT_NUMBER,"ERROR_INVALID_SEGMENT_NUMBER"},
{ERROR_INVALID_ORDINAL,"ERROR_INVALID_ORDINAL"},
{ERROR_ALREADY_EXISTS,"ERROR_ALREADY_EXISTS"},
{ERROR_INVALID_FLAG_NUMBER,"ERROR_INVALID_FLAG_NUMBER"},
{ERROR_SEM_NOT_FOUND,"ERROR_SEM_NOT_FOUND"},
{ERROR_INVALID_STARTING_CODESEG,"ERROR_INVALID_STARTING_CODESEG"},
{ERROR_INVALID_STACKSEG,"ERROR_INVALID_STACKSEG"},
{ERROR_INVALID_MODULETYPE,"ERROR_INVALID_MODULETYPE"},
{ERROR_INVALID_EXE_SIGNATURE,"ERROR_INVALID_EXE_SIGNATURE"},
{ERROR_EXE_MARKED_INVALID,"ERROR_EXE_MARKED_INVALID"},
{ERROR_BAD_EXE_FORMAT,"ERROR_BAD_EXE_FORMAT"},
{ERROR_ITERATED_DATA_EXCEEDS_64k,"ERROR_ITERATED_DATA_EXCEEDS_64k"},
{ERROR_INVALID_MINALLOCSIZE,"ERROR_INVALID_MINALLOCSIZE"},
{ERROR_DYNLINK_FROM_INVALID_RING,"ERROR_DYNLINK_FROM_INVALID_RING"},
{ERROR_IOPL_NOT_ENABLED,"ERROR_IOPL_NOT_ENABLED"},
{ERROR_INVALID_SEGDPL,"ERROR_INVALID_SEGDPL"},
{ERROR_AUTODATASEG_EXCEEDS_64k,"ERROR_AUTODATASEG_EXCEEDS_64k"},
{ERROR_RING2SEG_MUST_BE_MOVABLE,"ERROR_RING2SEG_MUST_BE_MOVABLE"},
{ERROR_RELOC_CHAIN_XEEDS_SEGLIM,"ERROR_RELOC_CHAIN_XEEDS_SEGLIM"},
{ERROR_INFLOOP_IN_RELOC_CHAIN,"ERROR_INFLOOP_IN_RELOC_CHAIN"},
{ERROR_ENVVAR_NOT_FOUND,"ERROR_ENVVAR_NOT_FOUND"},
{ERROR_NO_SIGNAL_SENT,"ERROR_NO_SIGNAL_SENT"},
{ERROR_FILENAME_EXCED_RANGE,"ERROR_FILENAME_EXCED_RANGE"},
{ERROR_RING2_STACK_IN_USE,"ERROR_RING2_STACK_IN_USE"},
{ERROR_META_EXPANSION_TOO_LONG,"ERROR_META_EXPANSION_TOO_LONG"},
{ERROR_INVALID_SIGNAL_NUMBER,"ERROR_INVALID_SIGNAL_NUMBER"},
{ERROR_THREAD_1_INACTIVE,"ERROR_THREAD_1_INACTIVE"},
{ERROR_LOCKED,"ERROR_LOCKED"},
{ERROR_TOO_MANY_MODULES,"ERROR_TOO_MANY_MODULES"},
{ERROR_NESTING_NOT_ALLOWED,"ERROR_NESTING_NOT_ALLOWED"},
{ERROR_EXE_MACHINE_TYPE_MISMATCH,"ERROR_EXE_MACHINE_TYPE_MISMATCH"},
{ERROR_BAD_PIPE,"ERROR_BAD_PIPE"},
{ERROR_PIPE_BUSY,"ERROR_PIPE_BUSY"},
{ERROR_NO_DATA,"ERROR_NO_DATA"},
{ERROR_PIPE_NOT_CONNECTED,"ERROR_PIPE_NOT_CONNECTED"},
{ERROR_VC_DISCONNECTED,"ERROR_VC_DISCONNECTED"},
{ERROR_INVALID_EA_NAME,"ERROR_INVALID_EA_NAME"},
{ERROR_EA_LIST_INCONSISTENT,"ERROR_EA_LIST_INCONSISTENT"},
{ERROR_NO_MORE_ITEMS,"ERROR_NO_MORE_ITEMS"},
{ERROR_CANNOT_COPY,"ERROR_CANNOT_COPY"},
{ERROR_DIRECTORY,"ERROR_DIRECTORY"},
{ERROR_EAS_DIDNT_FIT,"ERROR_EAS_DIDNT_FIT"},
{ERROR_EA_FILE_CORRUPT,"ERROR_EA_FILE_CORRUPT"},
{ERROR_EA_TABLE_FULL,"ERROR_EA_TABLE_FULL"},
{ERROR_INVALID_EA_HANDLE,"ERROR_INVALID_EA_HANDLE"},
{ERROR_EAS_NOT_SUPPORTED,"ERROR_EAS_NOT_SUPPORTED"},
{ERROR_NOT_OWNER,"ERROR_NOT_OWNER"},
{ERROR_TOO_MANY_POSTS,"ERROR_TOO_MANY_POSTS"},
{ERROR_PARTIAL_COPY,"ERROR_PARTIAL_COPY"},
{ERROR_OPLOCK_NOT_GRANTED,"ERROR_OPLOCK_NOT_GRANTED"},
{ERROR_INVALID_OPLOCK_PROTOCOL,"ERROR_INVALID_OPLOCK_PROTOCOL"},
{ERROR_DISK_TOO_FRAGMENTED,"ERROR_DISK_TOO_FRAGMENTED"},
{ERROR_DELETE_PENDING,"ERROR_DELETE_PENDING"},
{ERROR_MR_MID_NOT_FOUND,"ERROR_MR_MID_NOT_FOUND"},
{ERROR_INVALID_ADDRESS,"ERROR_INVALID_ADDRESS"},
{ERROR_ARITHMETIC_OVERFLOW,"ERROR_ARITHMETIC_OVERFLOW"},
{ERROR_PIPE_CONNECTED,"ERROR_PIPE_CONNECTED"},
{ERROR_PIPE_LISTENING,"ERROR_PIPE_LISTENING"},
{ERROR_EA_ACCESS_DENIED,"ERROR_EA_ACCESS_DENIED"},
{ERROR_OPERATION_ABORTED,"ERROR_OPERATION_ABORTED"},
{ERROR_IO_INCOMPLETE,"ERROR_IO_INCOMPLETE"},
{ERROR_NOACCESS,"ERROR_NOACCESS"},
{ERROR_SWAPERROR,"ERROR_SWAPERROR"},
{ERROR_STACK_OVERFLOW,"ERROR_STACK_OVERFLOW"},
{ERROR_INVALID_MESSAGE,"ERROR_INVALID_MESSAGE"},
{ERROR_CAN_NOT_COMPLETE,"ERROR_CAN_NOT_COMPLETE"},
{ERROR_INVALID_FLAGS,"ERROR_INVALID_FLAGS"},
{ERROR_UNRECOGNIZED_VOLUME,"ERROR_UNRECOGNIZED_VOLUME"},
{ERROR_FILE_INVALID,"ERROR_FILE_INVALID"},
{ERROR_FULLSCREEN_MODE,"ERROR_FULLSCREEN_MODE"},
{ERROR_NO_TOKEN,"ERROR_NO_TOKEN"},
{ERROR_BADDB,"ERROR_BADDB"},
{ERROR_BADKEY,"ERROR_BADKEY"},
{ERROR_CANTOPEN,"ERROR_CANTOPEN"},
{ERROR_CANTREAD,"ERROR_CANTREAD"},
{ERROR_CANTWRITE,"ERROR_CANTWRITE"},
{ERROR_REGISTRY_RECOVERED,"ERROR_REGISTRY_RECOVERED"},
{ERROR_REGISTRY_CORRUPT,"ERROR_REGISTRY_CORRUPT"},
{ERROR_REGISTRY_IO_FAILED,"ERROR_REGISTRY_IO_FAILED"},
{ERROR_NOT_REGISTRY_FILE,"ERROR_NOT_REGISTRY_FILE"},
{ERROR_KEY_DELETED,"ERROR_KEY_DELETED"},
{ERROR_NO_LOG_SPACE,"ERROR_NO_LOG_SPACE"},
{ERROR_KEY_HAS_CHILDREN,"ERROR_KEY_HAS_CHILDREN"},
{ERROR_CHILD_MUST_BE_VOLATILE,"ERROR_CHILD_MUST_BE_VOLATILE"},
{ERROR_NOTIFY_ENUM_DIR,"ERROR_NOTIFY_ENUM_DIR"},
{ERROR_DEPENDENT_SERVICES_RUNNING,"ERROR_DEPENDENT_SERVICES_RUNNING"},
{ERROR_INVALID_SERVICE_CONTROL,"ERROR_INVALID_SERVICE_CONTROL"},
{ERROR_SERVICE_REQUEST_TIMEOUT,"ERROR_SERVICE_REQUEST_TIMEOUT"},
{ERROR_SERVICE_NO_THREAD,"ERROR_SERVICE_NO_THREAD"},
{ERROR_SERVICE_DATABASE_LOCKED,"ERROR_SERVICE_DATABASE_LOCKED"},
{ERROR_SERVICE_ALREADY_RUNNING,"ERROR_SERVICE_ALREADY_RUNNING"},
{ERROR_INVALID_SERVICE_ACCOUNT,"ERROR_INVALID_SERVICE_ACCOUNT"},
{ERROR_SERVICE_DISABLED,"ERROR_SERVICE_DISABLED"},
{ERROR_CIRCULAR_DEPENDENCY,"ERROR_CIRCULAR_DEPENDENCY"},
{ERROR_SERVICE_DOES_NOT_EXIST,"ERROR_SERVICE_DOES_NOT_EXIST"},
{ERROR_SERVICE_CANNOT_ACCEPT_CTRL,"ERROR_SERVICE_CANNOT_ACCEPT_CTRL"},
{ERROR_SERVICE_NOT_ACTIVE,"ERROR_SERVICE_NOT_ACTIVE"},
{ERROR_FAILED_SERVICE_CONTROLLER_CONNECT,"ERROR_FAILED_SERVICE_CONTROLLER_CONNECT"},
{ERROR_EXCEPTION_IN_SERVICE,"ERROR_EXCEPTION_IN_SERVICE"},
{ERROR_DATABASE_DOES_NOT_EXIST,"ERROR_DATABASE_DOES_NOT_EXIST"},
{ERROR_SERVICE_SPECIFIC_ERROR,"ERROR_SERVICE_SPECIFIC_ERROR"},
{ERROR_PROCESS_ABORTED,"ERROR_PROCESS_ABORTED"},
{ERROR_SERVICE_DEPENDENCY_FAIL,"ERROR_SERVICE_DEPENDENCY_FAIL"},
{ERROR_SERVICE_LOGON_FAILED,"ERROR_SERVICE_LOGON_FAILED"},
{ERROR_SERVICE_START_HANG,"ERROR_SERVICE_START_HANG"},
{ERROR_INVALID_SERVICE_LOCK,"ERROR_INVALID_SERVICE_LOCK"},
{ERROR_SERVICE_MARKED_FOR_DELETE,"ERROR_SERVICE_MARKED_FOR_DELETE"},
{ERROR_SERVICE_EXISTS,"ERROR_SERVICE_EXISTS"},
{ERROR_ALREADY_RUNNING_LKG,"ERROR_ALREADY_RUNNING_LKG"},
{ERROR_SERVICE_DEPENDENCY_DELETED,"ERROR_SERVICE_DEPENDENCY_DELETED"},
{ERROR_BOOT_ALREADY_ACCEPTED,"ERROR_BOOT_ALREADY_ACCEPTED"},
{ERROR_SERVICE_NEVER_STARTED,"ERROR_SERVICE_NEVER_STARTED"},
{ERROR_DUPLICATE_SERVICE_NAME,"ERROR_DUPLICATE_SERVICE_NAME"},
{ERROR_DIFFERENT_SERVICE_ACCOUNT,"ERROR_DIFFERENT_SERVICE_ACCOUNT"},
{ERROR_CANNOT_DETECT_DRIVER_FAILURE,"ERROR_CANNOT_DETECT_DRIVER_FAILURE"},
{ERROR_CANNOT_DETECT_PROCESS_ABORT,"ERROR_CANNOT_DETECT_PROCESS_ABORT"},
{ERROR_NO_RECOVERY_PROGRAM,"ERROR_NO_RECOVERY_PROGRAM"},
{ERROR_SERVICE_NOT_IN_EXE,"ERROR_SERVICE_NOT_IN_EXE"},
{ERROR_NOT_SAFEBOOT_SERVICE,"ERROR_NOT_SAFEBOOT_SERVICE"},
{ERROR_END_OF_MEDIA,"ERROR_END_OF_MEDIA"},
{ERROR_FILEMARK_DETECTED,"ERROR_FILEMARK_DETECTED"},
{ERROR_BEGINNING_OF_MEDIA,"ERROR_BEGINNING_OF_MEDIA"},
{ERROR_SETMARK_DETECTED,"ERROR_SETMARK_DETECTED"},
{ERROR_NO_DATA_DETECTED,"ERROR_NO_DATA_DETECTED"},
{ERROR_PARTITION_FAILURE,"ERROR_PARTITION_FAILURE"},
{ERROR_INVALID_BLOCK_LENGTH,"ERROR_INVALID_BLOCK_LENGTH"},
{ERROR_DEVICE_NOT_PARTITIONED,"ERROR_DEVICE_NOT_PARTITIONED"},
{ERROR_UNABLE_TO_LOCK_MEDIA,"ERROR_UNABLE_TO_LOCK_MEDIA"},
{ERROR_UNABLE_TO_UNLOAD_MEDIA,"ERROR_UNABLE_TO_UNLOAD_MEDIA"},
{ERROR_MEDIA_CHANGED,"ERROR_MEDIA_CHANGED"},
{ERROR_BUS_RESET,"ERROR_BUS_RESET"},
{ERROR_NO_MEDIA_IN_DRIVE,"ERROR_NO_MEDIA_IN_DRIVE"},
{ERROR_NO_UNICODE_TRANSLATION,"ERROR_NO_UNICODE_TRANSLATION"},
{ERROR_DLL_INIT_FAILED,"ERROR_DLL_INIT_FAILED"},
{ERROR_SHUTDOWN_IN_PROGRESS,"ERROR_SHUTDOWN_IN_PROGRESS"},
{ERROR_NO_SHUTDOWN_IN_PROGRESS,"ERROR_NO_SHUTDOWN_IN_PROGRESS"},
{ERROR_IO_DEVICE,"ERROR_IO_DEVICE"},
{ERROR_SERIAL_NO_DEVICE,"ERROR_SERIAL_NO_DEVICE"},
{ERROR_IRQ_BUSY,"ERROR_IRQ_BUSY"},
{ERROR_MORE_WRITES,"ERROR_MORE_WRITES"},
{ERROR_COUNTER_TIMEOUT,"ERROR_COUNTER_TIMEOUT"},
{ERROR_FLOPPY_ID_MARK_NOT_FOUND,"ERROR_FLOPPY_ID_MARK_NOT_FOUND"},
{ERROR_FLOPPY_WRONG_CYLINDER,"ERROR_FLOPPY_WRONG_CYLINDER"},
{ERROR_FLOPPY_UNKNOWN_ERROR,"ERROR_FLOPPY_UNKNOWN_ERROR"},
{ERROR_FLOPPY_BAD_REGISTERS,"ERROR_FLOPPY_BAD_REGISTERS"},
{ERROR_DISK_RECALIBRATE_FAILED,"ERROR_DISK_RECALIBRATE_FAILED"},
{ERROR_DISK_OPERATION_FAILED,"ERROR_DISK_OPERATION_FAILED"},
{ERROR_DISK_RESET_FAILED,"ERROR_DISK_RESET_FAILED"},
{ERROR_EOM_OVERFLOW,"ERROR_EOM_OVERFLOW"},
{ERROR_NOT_ENOUGH_SERVER_MEMORY,"ERROR_NOT_ENOUGH_SERVER_MEMORY"},
{ERROR_POSSIBLE_DEADLOCK,"ERROR_POSSIBLE_DEADLOCK"},
{ERROR_MAPPED_ALIGNMENT,"ERROR_MAPPED_ALIGNMENT"},
{ERROR_SET_POWER_STATE_VETOED,"ERROR_SET_POWER_STATE_VETOED"},
{ERROR_SET_POWER_STATE_FAILED,"ERROR_SET_POWER_STATE_FAILED"},
{ERROR_TOO_MANY_LINKS,"ERROR_TOO_MANY_LINKS"},
{ERROR_OLD_WIN_VERSION,"ERROR_OLD_WIN_VERSION"},
{ERROR_APP_WRONG_OS,"ERROR_APP_WRONG_OS"},
{ERROR_SINGLE_INSTANCE_APP,"ERROR_SINGLE_INSTANCE_APP"},
{ERROR_RMODE_APP,"ERROR_RMODE_APP"},
{ERROR_INVALID_DLL,"ERROR_INVALID_DLL"},
{ERROR_NO_ASSOCIATION,"ERROR_NO_ASSOCIATION"},
{ERROR_DDE_FAIL,"ERROR_DDE_FAIL"},
{ERROR_DLL_NOT_FOUND,"ERROR_DLL_NOT_FOUND"},
{ERROR_NO_MORE_USER_HANDLES,"ERROR_NO_MORE_USER_HANDLES"},
{ERROR_MESSAGE_SYNC_ONLY,"ERROR_MESSAGE_SYNC_ONLY"},
{ERROR_SOURCE_ELEMENT_EMPTY,"ERROR_SOURCE_ELEMENT_EMPTY"},
{ERROR_DESTINATION_ELEMENT_FULL,"ERROR_DESTINATION_ELEMENT_FULL"},
{ERROR_ILLEGAL_ELEMENT_ADDRESS,"ERROR_ILLEGAL_ELEMENT_ADDRESS"},
{ERROR_MAGAZINE_NOT_PRESENT,"ERROR_MAGAZINE_NOT_PRESENT"},
{ERROR_DEVICE_REQUIRES_CLEANING,"ERROR_DEVICE_REQUIRES_CLEANING"},
{ERROR_DEVICE_DOOR_OPEN,"ERROR_DEVICE_DOOR_OPEN"},
{ERROR_DEVICE_NOT_CONNECTED,"ERROR_DEVICE_NOT_CONNECTED"},
{ERROR_NOT_FOUND,"ERROR_NOT_FOUND"},
{ERROR_NO_MATCH,"ERROR_NO_MATCH"},
{ERROR_SET_NOT_FOUND,"ERROR_SET_NOT_FOUND"},
{ERROR_POINT_NOT_FOUND,"ERROR_POINT_NOT_FOUND"},
{ERROR_NO_TRACKING_SERVICE,"ERROR_NO_TRACKING_SERVICE"},
{ERROR_NO_VOLUME_ID,"ERROR_NO_VOLUME_ID"},
{ERROR_UNABLE_TO_REMOVE_REPLACED,"ERROR_UNABLE_TO_REMOVE_REPLACED"},
{ERROR_UNABLE_TO_MOVE_REPLACEMENT,"ERROR_UNABLE_TO_MOVE_REPLACEMENT"},
{ERROR_UNABLE_TO_MOVE_REPLACEMENT_2,"ERROR_UNABLE_TO_MOVE_REPLACEMENT_2"},
{ERROR_JOURNAL_DELETE_IN_PROGRESS,"ERROR_JOURNAL_DELETE_IN_PROGRESS"},
{ERROR_JOURNAL_NOT_ACTIVE,"ERROR_JOURNAL_NOT_ACTIVE"},
{ERROR_POTENTIAL_FILE_FOUND,"ERROR_POTENTIAL_FILE_FOUND"},
{ERROR_JOURNAL_ENTRY_DELETED,"ERROR_JOURNAL_ENTRY_DELETED"},
{ERROR_BAD_DEVICE,"ERROR_BAD_DEVICE"},
{ERROR_CONNECTION_UNAVAIL,"ERROR_CONNECTION_UNAVAIL"},
{ERROR_DEVICE_ALREADY_REMEMBERED,"ERROR_DEVICE_ALREADY_REMEMBERED"},
{ERROR_NO_NET_OR_BAD_PATH,"ERROR_NO_NET_OR_BAD_PATH"},
{ERROR_BAD_PROVIDER,"ERROR_BAD_PROVIDER"},
{ERROR_CANNOT_OPEN_PROFILE,"ERROR_CANNOT_OPEN_PROFILE"},
{ERROR_BAD_PROFILE,"ERROR_BAD_PROFILE"},
{ERROR_NOT_CONTAINER,"ERROR_NOT_CONTAINER"},
{ERROR_EXTENDED_ERROR,"ERROR_EXTENDED_ERROR"},
{ERROR_INVALID_GROUPNAME,"ERROR_INVALID_GROUPNAME"},
{ERROR_INVALID_COMPUTERNAME,"ERROR_INVALID_COMPUTERNAME"},
{ERROR_INVALID_EVENTNAME,"ERROR_INVALID_EVENTNAME"},
{ERROR_INVALID_DOMAINNAME,"ERROR_INVALID_DOMAINNAME"},
{ERROR_INVALID_SERVICENAME,"ERROR_INVALID_SERVICENAME"},
{ERROR_INVALID_NETNAME,"ERROR_INVALID_NETNAME"},
{ERROR_INVALID_SHARENAME,"ERROR_INVALID_SHARENAME"},
{ERROR_INVALID_PASSWORDNAME,"ERROR_INVALID_PASSWORDNAME"},
{ERROR_INVALID_MESSAGENAME,"ERROR_INVALID_MESSAGENAME"},
{ERROR_INVALID_MESSAGEDEST,"ERROR_INVALID_MESSAGEDEST"},
{ERROR_SESSION_CREDENTIAL_CONFLICT,"ERROR_SESSION_CREDENTIAL_CONFLICT"},
{ERROR_REMOTE_SESSION_LIMIT_EXCEEDED,"ERROR_REMOTE_SESSION_LIMIT_EXCEEDED"},
{ERROR_DUP_DOMAINNAME,"ERROR_DUP_DOMAINNAME"},
{ERROR_NO_NETWORK,"ERROR_NO_NETWORK"},
{ERROR_CANCELLED,"ERROR_CANCELLED"},
{ERROR_USER_MAPPED_FILE,"ERROR_USER_MAPPED_FILE"},
{ERROR_CONNECTION_REFUSED,"ERROR_CONNECTION_REFUSED"},
{ERROR_GRACEFUL_DISCONNECT,"ERROR_GRACEFUL_DISCONNECT"},
{ERROR_ADDRESS_ALREADY_ASSOCIATED,"ERROR_ADDRESS_ALREADY_ASSOCIATED"},
{ERROR_ADDRESS_NOT_ASSOCIATED,"ERROR_ADDRESS_NOT_ASSOCIATED"},
{ERROR_CONNECTION_INVALID,"ERROR_CONNECTION_INVALID"},
{ERROR_CONNECTION_ACTIVE,"ERROR_CONNECTION_ACTIVE"},
{ERROR_NETWORK_UNREACHABLE,"ERROR_NETWORK_UNREACHABLE"},
{ERROR_HOST_UNREACHABLE,"ERROR_HOST_UNREACHABLE"},
{ERROR_PROTOCOL_UNREACHABLE,"ERROR_PROTOCOL_UNREACHABLE"},
{ERROR_PORT_UNREACHABLE,"ERROR_PORT_UNREACHABLE"},
{ERROR_REQUEST_ABORTED,"ERROR_REQUEST_ABORTED"},
{ERROR_CONNECTION_ABORTED,"ERROR_CONNECTION_ABORTED"},
{ERROR_RETRY,"ERROR_RETRY"},
{ERROR_CONNECTION_COUNT_LIMIT,"ERROR_CONNECTION_COUNT_LIMIT"},
{ERROR_LOGIN_TIME_RESTRICTION,"ERROR_LOGIN_TIME_RESTRICTION"},
{ERROR_LOGIN_WKSTA_RESTRICTION,"ERROR_LOGIN_WKSTA_RESTRICTION"},
{ERROR_INCORRECT_ADDRESS,"ERROR_INCORRECT_ADDRESS"},
{ERROR_ALREADY_REGISTERED,"ERROR_ALREADY_REGISTERED"},
{ERROR_SERVICE_NOT_FOUND,"ERROR_SERVICE_NOT_FOUND"},
{ERROR_NOT_AUTHENTICATED,"ERROR_NOT_AUTHENTICATED"},
{ERROR_NOT_LOGGED_ON,"ERROR_NOT_LOGGED_ON"},
{ERROR_ALREADY_INITIALIZED,"ERROR_ALREADY_INITIALIZED"},
{ERROR_NO_SUCH_SITE,"ERROR_NO_SUCH_SITE"},
{ERROR_DOMAIN_CONTROLLER_EXISTS,"ERROR_DOMAIN_CONTROLLER_EXISTS"},
{ERROR_ONLY_IF_CONNECTED,"ERROR_ONLY_IF_CONNECTED"},
{ERROR_OVERRIDE_NOCHANGES,"ERROR_OVERRIDE_NOCHANGES"},
{ERROR_BAD_USER_PROFILE,"ERROR_BAD_USER_PROFILE"},
{ERROR_NOT_SUPPORTED_ON_SBS,"ERROR_NOT_SUPPORTED_ON_SBS"},
{ERROR_SERVER_SHUTDOWN_IN_PROGRESS,"ERROR_SERVER_SHUTDOWN_IN_PROGRESS"},
{ERROR_HOST_DOWN,"ERROR_HOST_DOWN"},
{ERROR_NON_ACCOUNT_SID,"ERROR_NON_ACCOUNT_SID"},
{ERROR_NON_DOMAIN_SID,"ERROR_NON_DOMAIN_SID"},
{ERROR_APPHELP_BLOCK,"ERROR_APPHELP_BLOCK"},
{ERROR_ACCESS_DISABLED_BY_POLICY,"ERROR_ACCESS_DISABLED_BY_POLICY"},
{ERROR_REG_NAT_CONSUMPTION,"ERROR_REG_NAT_CONSUMPTION"},
{ERROR_CSCSHARE_OFFLINE,"ERROR_CSCSHARE_OFFLINE"},
{ERROR_PKINIT_FAILURE,"ERROR_PKINIT_FAILURE"},
{ERROR_SMARTCARD_SUBSYSTEM_FAILURE,"ERROR_SMARTCARD_SUBSYSTEM_FAILURE"},
{ERROR_DOWNGRADE_DETECTED,"ERROR_DOWNGRADE_DETECTED"},
{SEC_E_SMARTCARD_CERT_REVOKED,"SEC_E_SMARTCARD_CERT_REVOKED"},
{SEC_E_ISSUING_CA_UNTRUSTED,"SEC_E_ISSUING_CA_UNTRUSTED"},
{SEC_E_REVOCATION_OFFLINE_C,"SEC_E_REVOCATION_OFFLINE_C"},
{SEC_E_PKINIT_CLIENT_FAILURE,"SEC_E_PKINIT_CLIENT_FAILURE"},
{SEC_E_SMARTCARD_CERT_EXPIRED,"SEC_E_SMARTCARD_CERT_EXPIRED"},
{ERROR_MACHINE_LOCKED,"ERROR_MACHINE_LOCKED"},
{ERROR_CALLBACK_SUPPLIED_INVALID_DATA,"ERROR_CALLBACK_SUPPLIED_INVALID_DATA"},
{ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED,"ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED"},
{ERROR_DRIVER_BLOCKED,"ERROR_DRIVER_BLOCKED"},
{ERROR_INVALID_IMPORT_OF_NON_DLL,"ERROR_INVALID_IMPORT_OF_NON_DLL"},
{ERROR_NOT_ALL_ASSIGNED,"ERROR_NOT_ALL_ASSIGNED"},
{ERROR_SOME_NOT_MAPPED,"ERROR_SOME_NOT_MAPPED"},
{ERROR_NO_QUOTAS_FOR_ACCOUNT,"ERROR_NO_QUOTAS_FOR_ACCOUNT"},
{ERROR_LOCAL_USER_SESSION_KEY,"ERROR_LOCAL_USER_SESSION_KEY"},
{ERROR_NULL_LM_PASSWORD,"ERROR_NULL_LM_PASSWORD"},
{ERROR_UNKNOWN_REVISION,"ERROR_UNKNOWN_REVISION"},
{ERROR_REVISION_MISMATCH,"ERROR_REVISION_MISMATCH"},
{ERROR_INVALID_OWNER,"ERROR_INVALID_OWNER"},
{ERROR_INVALID_PRIMARY_GROUP,"ERROR_INVALID_PRIMARY_GROUP"},
{ERROR_NO_IMPERSONATION_TOKEN,"ERROR_NO_IMPERSONATION_TOKEN"},
{ERROR_CANT_DISABLE_MANDATORY,"ERROR_CANT_DISABLE_MANDATORY"},
{ERROR_NO_LOGON_SERVERS,"ERROR_NO_LOGON_SERVERS"},
{ERROR_NO_SUCH_LOGON_SESSION,"ERROR_NO_SUCH_LOGON_SESSION"},
{ERROR_NO_SUCH_PRIVILEGE,"ERROR_NO_SUCH_PRIVILEGE"},
{ERROR_PRIVILEGE_NOT_HELD,"ERROR_PRIVILEGE_NOT_HELD"},
{ERROR_INVALID_ACCOUNT_NAME,"ERROR_INVALID_ACCOUNT_NAME"},
{ERROR_USER_EXISTS,"ERROR_USER_EXISTS"},
{ERROR_NO_SUCH_USER,"ERROR_NO_SUCH_USER"},
{ERROR_GROUP_EXISTS,"ERROR_GROUP_EXISTS"},
{ERROR_NO_SUCH_GROUP,"ERROR_NO_SUCH_GROUP"},
{ERROR_MEMBER_IN_GROUP,"ERROR_MEMBER_IN_GROUP"},
{ERROR_MEMBER_NOT_IN_GROUP,"ERROR_MEMBER_NOT_IN_GROUP"},
{ERROR_LAST_ADMIN,"ERROR_LAST_ADMIN"},
{ERROR_WRONG_PASSWORD,"ERROR_WRONG_PASSWORD"},
{ERROR_ILL_FORMED_PASSWORD,"ERROR_ILL_FORMED_PASSWORD"},
{ERROR_PASSWORD_RESTRICTION,"ERROR_PASSWORD_RESTRICTION"},
{ERROR_LOGON_FAILURE,"ERROR_LOGON_FAILURE"},
{ERROR_ACCOUNT_RESTRICTION,"ERROR_ACCOUNT_RESTRICTION"},
{ERROR_INVALID_LOGON_HOURS,"ERROR_INVALID_LOGON_HOURS"},
{ERROR_INVALID_WORKSTATION,"ERROR_INVALID_WORKSTATION"},
{ERROR_PASSWORD_EXPIRED,"ERROR_PASSWORD_EXPIRED"},
{ERROR_ACCOUNT_DISABLED,"ERROR_ACCOUNT_DISABLED"},
{ERROR_NONE_MAPPED,"ERROR_NONE_MAPPED"},
{ERROR_TOO_MANY_LUIDS_REQUESTED,"ERROR_TOO_MANY_LUIDS_REQUESTED"},
{ERROR_LUIDS_EXHAUSTED,"ERROR_LUIDS_EXHAUSTED"},
{ERROR_INVALID_SUB_AUTHORITY,"ERROR_INVALID_SUB_AUTHORITY"},
{ERROR_INVALID_ACL,"ERROR_INVALID_ACL"},
{ERROR_INVALID_SID,"ERROR_INVALID_SID"},
{ERROR_INVALID_SECURITY_DESCR,"ERROR_INVALID_SECURITY_DESCR"},
{ERROR_BAD_INHERITANCE_ACL,"ERROR_BAD_INHERITANCE_ACL"},
{ERROR_SERVER_DISABLED,"ERROR_SERVER_DISABLED"},
{ERROR_SERVER_NOT_DISABLED,"ERROR_SERVER_NOT_DISABLED"},
{ERROR_INVALID_ID_AUTHORITY,"ERROR_INVALID_ID_AUTHORITY"},
{ERROR_ALLOTTED_SPACE_EXCEEDED,"ERROR_ALLOTTED_SPACE_EXCEEDED"},
{ERROR_INVALID_GROUP_ATTRIBUTES,"ERROR_INVALID_GROUP_ATTRIBUTES"},
{ERROR_BAD_IMPERSONATION_LEVEL,"ERROR_BAD_IMPERSONATION_LEVEL"},
{ERROR_CANT_OPEN_ANONYMOUS,"ERROR_CANT_OPEN_ANONYMOUS"},
{ERROR_BAD_VALIDATION_CLASS,"ERROR_BAD_VALIDATION_CLASS"},
{ERROR_BAD_TOKEN_TYPE,"ERROR_BAD_TOKEN_TYPE"},
{ERROR_NO_SECURITY_ON_OBJECT,"ERROR_NO_SECURITY_ON_OBJECT"},
{ERROR_CANT_ACCESS_DOMAIN_INFO,"ERROR_CANT_ACCESS_DOMAIN_INFO"},
{ERROR_INVALID_SERVER_STATE,"ERROR_INVALID_SERVER_STATE"},
{ERROR_INVALID_DOMAIN_STATE,"ERROR_INVALID_DOMAIN_STATE"},
{ERROR_INVALID_DOMAIN_ROLE,"ERROR_INVALID_DOMAIN_ROLE"},
{ERROR_NO_SUCH_DOMAIN,"ERROR_NO_SUCH_DOMAIN"},
{ERROR_DOMAIN_EXISTS,"ERROR_DOMAIN_EXISTS"},
{ERROR_DOMAIN_LIMIT_EXCEEDED,"ERROR_DOMAIN_LIMIT_EXCEEDED"},
{ERROR_INTERNAL_DB_CORRUPTION,"ERROR_INTERNAL_DB_CORRUPTION"},
{ERROR_INTERNAL_ERROR,"ERROR_INTERNAL_ERROR"},
{ERROR_GENERIC_NOT_MAPPED,"ERROR_GENERIC_NOT_MAPPED"},
{ERROR_BAD_DESCRIPTOR_FORMAT,"ERROR_BAD_DESCRIPTOR_FORMAT"},
{ERROR_NOT_LOGON_PROCESS,"ERROR_NOT_LOGON_PROCESS"},
{ERROR_LOGON_SESSION_EXISTS,"ERROR_LOGON_SESSION_EXISTS"},
{ERROR_NO_SUCH_PACKAGE,"ERROR_NO_SUCH_PACKAGE"},
{ERROR_BAD_LOGON_SESSION_STATE,"ERROR_BAD_LOGON_SESSION_STATE"},
{ERROR_LOGON_SESSION_COLLISION,"ERROR_LOGON_SESSION_COLLISION"},
{ERROR_INVALID_LOGON_TYPE,"ERROR_INVALID_LOGON_TYPE"},
{ERROR_CANNOT_IMPERSONATE,"ERROR_CANNOT_IMPERSONATE"},
{ERROR_RXACT_INVALID_STATE,"ERROR_RXACT_INVALID_STATE"},
{ERROR_RXACT_COMMIT_FAILURE,"ERROR_RXACT_COMMIT_FAILURE"},
{ERROR_SPECIAL_ACCOUNT,"ERROR_SPECIAL_ACCOUNT"},
{ERROR_SPECIAL_GROUP,"ERROR_SPECIAL_GROUP"},
{ERROR_SPECIAL_USER,"ERROR_SPECIAL_USER"},
{ERROR_MEMBERS_PRIMARY_GROUP,"ERROR_MEMBERS_PRIMARY_GROUP"},
{ERROR_TOKEN_ALREADY_IN_USE,"ERROR_TOKEN_ALREADY_IN_USE"},
{ERROR_NO_SUCH_ALIAS,"ERROR_NO_SUCH_ALIAS"},
{ERROR_MEMBER_NOT_IN_ALIAS,"ERROR_MEMBER_NOT_IN_ALIAS"},
{ERROR_MEMBER_IN_ALIAS,"ERROR_MEMBER_IN_ALIAS"},
{ERROR_ALIAS_EXISTS,"ERROR_ALIAS_EXISTS"},
{ERROR_LOGON_NOT_GRANTED,"ERROR_LOGON_NOT_GRANTED"},
{ERROR_TOO_MANY_SECRETS,"ERROR_TOO_MANY_SECRETS"},
{ERROR_SECRET_TOO_LONG,"ERROR_SECRET_TOO_LONG"},
{ERROR_INTERNAL_DB_ERROR,"ERROR_INTERNAL_DB_ERROR"},
{ERROR_TOO_MANY_CONTEXT_IDS,"ERROR_TOO_MANY_CONTEXT_IDS"},
{ERROR_LOGON_TYPE_NOT_GRANTED,"ERROR_LOGON_TYPE_NOT_GRANTED"},
{ERROR_NT_CROSS_ENCRYPTION_REQUIRED,"ERROR_NT_CROSS_ENCRYPTION_REQUIRED"},
{ERROR_NO_SUCH_MEMBER,"ERROR_NO_SUCH_MEMBER"},
{ERROR_INVALID_MEMBER,"ERROR_INVALID_MEMBER"},
{ERROR_TOO_MANY_SIDS,"ERROR_TOO_MANY_SIDS"},
{ERROR_LM_CROSS_ENCRYPTION_REQUIRED,"ERROR_LM_CROSS_ENCRYPTION_REQUIRED"},
{ERROR_NO_INHERITANCE,"ERROR_NO_INHERITANCE"},
{ERROR_FILE_CORRUPT,"ERROR_FILE_CORRUPT"},
{ERROR_DISK_CORRUPT,"ERROR_DISK_CORRUPT"},
{ERROR_NO_USER_SESSION_KEY,"ERROR_NO_USER_SESSION_KEY"},
{ERROR_LICENSE_QUOTA_EXCEEDED,"ERROR_LICENSE_QUOTA_EXCEEDED"},
{ERROR_WRONG_TARGET_NAME,"ERROR_WRONG_TARGET_NAME"},
{ERROR_MUTUAL_AUTH_FAILED,"ERROR_MUTUAL_AUTH_FAILED"},
{ERROR_TIME_SKEW,"ERROR_TIME_SKEW"},
{ERROR_CURRENT_DOMAIN_NOT_ALLOWED,"ERROR_CURRENT_DOMAIN_NOT_ALLOWED"},
{ERROR_INVALID_WINDOW_HANDLE,"ERROR_INVALID_WINDOW_HANDLE"},
{ERROR_INVALID_MENU_HANDLE,"ERROR_INVALID_MENU_HANDLE"},
{ERROR_INVALID_CURSOR_HANDLE,"ERROR_INVALID_CURSOR_HANDLE"},
{ERROR_INVALID_ACCEL_HANDLE,"ERROR_INVALID_ACCEL_HANDLE"},
{ERROR_INVALID_HOOK_HANDLE,"ERROR_INVALID_HOOK_HANDLE"},
{ERROR_INVALID_DWP_HANDLE,"ERROR_INVALID_DWP_HANDLE"},
{ERROR_TLW_WITH_WSCHILD,"ERROR_TLW_WITH_WSCHILD"},
{ERROR_CANNOT_FIND_WND_CLASS,"ERROR_CANNOT_FIND_WND_CLASS"},
{ERROR_WINDOW_OF_OTHER_THREAD,"ERROR_WINDOW_OF_OTHER_THREAD"},
{ERROR_HOTKEY_ALREADY_REGISTERED,"ERROR_HOTKEY_ALREADY_REGISTERED"},
{ERROR_CLASS_ALREADY_EXISTS,"ERROR_CLASS_ALREADY_EXISTS"},
{ERROR_CLASS_DOES_NOT_EXIST,"ERROR_CLASS_DOES_NOT_EXIST"},
{ERROR_CLASS_HAS_WINDOWS,"ERROR_CLASS_HAS_WINDOWS"},
{ERROR_INVALID_INDEX,"ERROR_INVALID_INDEX"},
{ERROR_INVALID_ICON_HANDLE,"ERROR_INVALID_ICON_HANDLE"},
{ERROR_PRIVATE_DIALOG_INDEX,"ERROR_PRIVATE_DIALOG_INDEX"},
{ERROR_LISTBOX_ID_NOT_FOUND,"ERROR_LISTBOX_ID_NOT_FOUND"},
{ERROR_NO_WILDCARD_CHARACTERS,"ERROR_NO_WILDCARD_CHARACTERS"},
{ERROR_CLIPBOARD_NOT_OPEN,"ERROR_CLIPBOARD_NOT_OPEN"},
{ERROR_HOTKEY_NOT_REGISTERED,"ERROR_HOTKEY_NOT_REGISTERED"},
{ERROR_WINDOW_NOT_DIALOG,"ERROR_WINDOW_NOT_DIALOG"},
{ERROR_CONTROL_ID_NOT_FOUND,"ERROR_CONTROL_ID_NOT_FOUND"},
{ERROR_INVALID_COMBOBOX_MESSAGE,"ERROR_INVALID_COMBOBOX_MESSAGE"},
{ERROR_WINDOW_NOT_COMBOBOX,"ERROR_WINDOW_NOT_COMBOBOX"},
{ERROR_INVALID_EDIT_HEIGHT,"ERROR_INVALID_EDIT_HEIGHT"},
{ERROR_DC_NOT_FOUND,"ERROR_DC_NOT_FOUND"},
{ERROR_INVALID_HOOK_FILTER,"ERROR_INVALID_HOOK_FILTER"},
{ERROR_INVALID_FILTER_PROC,"ERROR_INVALID_FILTER_PROC"},
{ERROR_HOOK_NEEDS_HMOD,"ERROR_HOOK_NEEDS_HMOD"},
{ERROR_GLOBAL_ONLY_HOOK,"ERROR_GLOBAL_ONLY_HOOK"},
{ERROR_JOURNAL_HOOK_SET,"ERROR_JOURNAL_HOOK_SET"},
{ERROR_HOOK_NOT_INSTALLED,"ERROR_HOOK_NOT_INSTALLED"},
{ERROR_INVALID_LB_MESSAGE,"ERROR_INVALID_LB_MESSAGE"},
{ERROR_SETCOUNT_ON_BAD_LB,"ERROR_SETCOUNT_ON_BAD_LB"},
{ERROR_LB_WITHOUT_TABSTOPS,"ERROR_LB_WITHOUT_TABSTOPS"},
{ERROR_DESTROY_OBJECT_OF_OTHER_THREAD,"ERROR_DESTROY_OBJECT_OF_OTHER_THREAD"},
{ERROR_CHILD_WINDOW_MENU,"ERROR_CHILD_WINDOW_MENU"},
{ERROR_NO_SYSTEM_MENU,"ERROR_NO_SYSTEM_MENU"},
{ERROR_INVALID_MSGBOX_STYLE,"ERROR_INVALID_MSGBOX_STYLE"},
{ERROR_INVALID_SPI_VALUE,"ERROR_INVALID_SPI_VALUE"},
{ERROR_SCREEN_ALREADY_LOCKED,"ERROR_SCREEN_ALREADY_LOCKED"},
{ERROR_HWNDS_HAVE_DIFF_PARENT,"ERROR_HWNDS_HAVE_DIFF_PARENT"},
{ERROR_NOT_CHILD_WINDOW,"ERROR_NOT_CHILD_WINDOW"},
{ERROR_INVALID_GW_COMMAND,"ERROR_INVALID_GW_COMMAND"},
{ERROR_INVALID_THREAD_ID,"ERROR_INVALID_THREAD_ID"},
{ERROR_NON_MDICHILD_WINDOW,"ERROR_NON_MDICHILD_WINDOW"},
{ERROR_POPUP_ALREADY_ACTIVE,"ERROR_POPUP_ALREADY_ACTIVE"},
{ERROR_NO_SCROLLBARS,"ERROR_NO_SCROLLBARS"},
{ERROR_INVALID_SCROLLBAR_RANGE,"ERROR_INVALID_SCROLLBAR_RANGE"},
{ERROR_INVALID_SHOWWIN_COMMAND,"ERROR_INVALID_SHOWWIN_COMMAND"},
{ERROR_NO_SYSTEM_RESOURCES,"ERROR_NO_SYSTEM_RESOURCES"},
{ERROR_NONPAGED_SYSTEM_RESOURCES,"ERROR_NONPAGED_SYSTEM_RESOURCES"},
{ERROR_PAGED_SYSTEM_RESOURCES,"ERROR_PAGED_SYSTEM_RESOURCES"},
{ERROR_WORKING_SET_QUOTA,"ERROR_WORKING_SET_QUOTA"},
{ERROR_PAGEFILE_QUOTA,"ERROR_PAGEFILE_QUOTA"},
{ERROR_COMMITMENT_LIMIT,"ERROR_COMMITMENT_LIMIT"},
{ERROR_MENU_ITEM_NOT_FOUND,"ERROR_MENU_ITEM_NOT_FOUND"},
{ERROR_INVALID_KEYBOARD_HANDLE,"ERROR_INVALID_KEYBOARD_HANDLE"},
{ERROR_HOOK_TYPE_NOT_ALLOWED,"ERROR_HOOK_TYPE_NOT_ALLOWED"},
{ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION,"ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION"},
{ERROR_TIMEOUT,"ERROR_TIMEOUT"},
{ERROR_INVALID_MONITOR_HANDLE,"ERROR_INVALID_MONITOR_HANDLE"},
{ERROR_EVENTLOG_FILE_CORRUPT,"ERROR_EVENTLOG_FILE_CORRUPT"},
{ERROR_EVENTLOG_CANT_START,"ERROR_EVENTLOG_CANT_START"},
{ERROR_LOG_FILE_FULL,"ERROR_LOG_FILE_FULL"},
{ERROR_EVENTLOG_FILE_CHANGED,"ERROR_EVENTLOG_FILE_CHANGED"},
{ERROR_INSTALL_SERVICE_FAILURE,"ERROR_INSTALL_SERVICE_FAILURE"},
{ERROR_INSTALL_USEREXIT,"ERROR_INSTALL_USEREXIT"},
{ERROR_INSTALL_FAILURE,"ERROR_INSTALL_FAILURE"},
{ERROR_INSTALL_SUSPEND,"ERROR_INSTALL_SUSPEND"},
{ERROR_UNKNOWN_PRODUCT,"ERROR_UNKNOWN_PRODUCT"},
{ERROR_UNKNOWN_FEATURE,"ERROR_UNKNOWN_FEATURE"},
{ERROR_UNKNOWN_COMPONENT,"ERROR_UNKNOWN_COMPONENT"},
{ERROR_UNKNOWN_PROPERTY,"ERROR_UNKNOWN_PROPERTY"},
{ERROR_INVALID_HANDLE_STATE,"ERROR_INVALID_HANDLE_STATE"},
{ERROR_BAD_CONFIGURATION,"ERROR_BAD_CONFIGURATION"},
{ERROR_INDEX_ABSENT,"ERROR_INDEX_ABSENT"},
{ERROR_INSTALL_SOURCE_ABSENT,"ERROR_INSTALL_SOURCE_ABSENT"},
{ERROR_INSTALL_PACKAGE_VERSION,"ERROR_INSTALL_PACKAGE_VERSION"},
{ERROR_PRODUCT_UNINSTALLED,"ERROR_PRODUCT_UNINSTALLED"},
{ERROR_BAD_QUERY_SYNTAX,"ERROR_BAD_QUERY_SYNTAX"},
{ERROR_INVALID_FIELD,"ERROR_INVALID_FIELD"},
{ERROR_DEVICE_REMOVED,"ERROR_DEVICE_REMOVED"},
{ERROR_INSTALL_ALREADY_RUNNING,"ERROR_INSTALL_ALREADY_RUNNING"},
{ERROR_INSTALL_PACKAGE_OPEN_FAILED,"ERROR_INSTALL_PACKAGE_OPEN_FAILED"},
{ERROR_INSTALL_PACKAGE_INVALID,"ERROR_INSTALL_PACKAGE_INVALID"},
{ERROR_INSTALL_UI_FAILURE,"ERROR_INSTALL_UI_FAILURE"},
{ERROR_INSTALL_LOG_FAILURE,"ERROR_INSTALL_LOG_FAILURE"},
{ERROR_INSTALL_LANGUAGE_UNSUPPORTED,"ERROR_INSTALL_LANGUAGE_UNSUPPORTED"},
{ERROR_INSTALL_TRANSFORM_FAILURE,"ERROR_INSTALL_TRANSFORM_FAILURE"},
{ERROR_INSTALL_PACKAGE_REJECTED,"ERROR_INSTALL_PACKAGE_REJECTED"},
{ERROR_FUNCTION_NOT_CALLED,"ERROR_FUNCTION_NOT_CALLED"},
{ERROR_FUNCTION_FAILED,"ERROR_FUNCTION_FAILED"},
{ERROR_INVALID_TABLE,"ERROR_INVALID_TABLE"},
{ERROR_DATATYPE_MISMATCH,"ERROR_DATATYPE_MISMATCH"},
{ERROR_UNSUPPORTED_TYPE,"ERROR_UNSUPPORTED_TYPE"},
{ERROR_CREATE_FAILED,"ERROR_CREATE_FAILED"},
{ERROR_INSTALL_TEMP_UNWRITABLE,"ERROR_INSTALL_TEMP_UNWRITABLE"},
{ERROR_INSTALL_PLATFORM_UNSUPPORTED,"ERROR_INSTALL_PLATFORM_UNSUPPORTED"},
{ERROR_INSTALL_NOTUSED,"ERROR_INSTALL_NOTUSED"},
{ERROR_PATCH_PACKAGE_OPEN_FAILED,"ERROR_PATCH_PACKAGE_OPEN_FAILED"},
{ERROR_PATCH_PACKAGE_INVALID,"ERROR_PATCH_PACKAGE_INVALID"},
{ERROR_PATCH_PACKAGE_UNSUPPORTED,"ERROR_PATCH_PACKAGE_UNSUPPORTED"},
{ERROR_PRODUCT_VERSION,"ERROR_PRODUCT_VERSION"},
{ERROR_INVALID_COMMAND_LINE,"ERROR_INVALID_COMMAND_LINE"},
{ERROR_INSTALL_REMOTE_DISALLOWED,"ERROR_INSTALL_REMOTE_DISALLOWED"},
{ERROR_SUCCESS_REBOOT_INITIATED,"ERROR_SUCCESS_REBOOT_INITIATED"},
{ERROR_PATCH_TARGET_NOT_FOUND,"ERROR_PATCH_TARGET_NOT_FOUND"},
{ERROR_PATCH_PACKAGE_REJECTED,"ERROR_PATCH_PACKAGE_REJECTED"},
{ERROR_INSTALL_TRANSFORM_REJECTED,"ERROR_INSTALL_TRANSFORM_REJECTED"},
{RPC_S_INVALID_STRING_BINDING,"RPC_S_INVALID_STRING_BINDING"},
{RPC_S_WRONG_KIND_OF_BINDING,"RPC_S_WRONG_KIND_OF_BINDING"},
{RPC_S_INVALID_BINDING,"RPC_S_INVALID_BINDING"},
{RPC_S_PROTSEQ_NOT_SUPPORTED,"RPC_S_PROTSEQ_NOT_SUPPORTED"},
{RPC_S_INVALID_RPC_PROTSEQ,"RPC_S_INVALID_RPC_PROTSEQ"},
{RPC_S_INVALID_STRING_UUID,"RPC_S_INVALID_STRING_UUID"},
{RPC_S_INVALID_ENDPOINT_FORMAT,"RPC_S_INVALID_ENDPOINT_FORMAT"},
{RPC_S_INVALID_NET_ADDR,"RPC_S_INVALID_NET_ADDR"},
{RPC_S_NO_ENDPOINT_FOUND,"RPC_S_NO_ENDPOINT_FOUND"},
{RPC_S_INVALID_TIMEOUT,"RPC_S_INVALID_TIMEOUT"},
{RPC_S_OBJECT_NOT_FOUND,"RPC_S_OBJECT_NOT_FOUND"},
{RPC_S_ALREADY_REGISTERED,"RPC_S_ALREADY_REGISTERED"},
{RPC_S_TYPE_ALREADY_REGISTERED,"RPC_S_TYPE_ALREADY_REGISTERED"},
{RPC_S_ALREADY_LISTENING,"RPC_S_ALREADY_LISTENING"},
{RPC_S_NO_PROTSEQS_REGISTERED,"RPC_S_NO_PROTSEQS_REGISTERED"},
{RPC_S_NOT_LISTENING,"RPC_S_NOT_LISTENING"},
{RPC_S_UNKNOWN_MGR_TYPE,"RPC_S_UNKNOWN_MGR_TYPE"},
{RPC_S_UNKNOWN_IF,"RPC_S_UNKNOWN_IF"},
{RPC_S_NO_BINDINGS,"RPC_S_NO_BINDINGS"},
{RPC_S_NO_PROTSEQS,"RPC_S_NO_PROTSEQS"},
{RPC_S_CANT_CREATE_ENDPOINT,"RPC_S_CANT_CREATE_ENDPOINT"},
{RPC_S_OUT_OF_RESOURCES,"RPC_S_OUT_OF_RESOURCES"},
{RPC_S_SERVER_UNAVAILABLE,"RPC_S_SERVER_UNAVAILABLE"},
{RPC_S_SERVER_TOO_BUSY,"RPC_S_SERVER_TOO_BUSY"},
{RPC_S_INVALID_NETWORK_OPTIONS,"RPC_S_INVALID_NETWORK_OPTIONS"},
{RPC_S_NO_CALL_ACTIVE,"RPC_S_NO_CALL_ACTIVE"},
{RPC_S_CALL_FAILED,"RPC_S_CALL_FAILED"},
{RPC_S_CALL_FAILED_DNE,"RPC_S_CALL_FAILED_DNE"},
{RPC_S_PROTOCOL_ERROR,"RPC_S_PROTOCOL_ERROR"},
{RPC_S_UNSUPPORTED_TRANS_SYN,"RPC_S_UNSUPPORTED_TRANS_SYN"},
{RPC_S_UNSUPPORTED_TYPE,"RPC_S_UNSUPPORTED_TYPE"},
{RPC_S_INVALID_TAG,"RPC_S_INVALID_TAG"},
{RPC_S_INVALID_BOUND,"RPC_S_INVALID_BOUND"},
{RPC_S_NO_ENTRY_NAME,"RPC_S_NO_ENTRY_NAME"},
{RPC_S_INVALID_NAME_SYNTAX,"RPC_S_INVALID_NAME_SYNTAX"},
{RPC_S_UNSUPPORTED_NAME_SYNTAX,"RPC_S_UNSUPPORTED_NAME_SYNTAX"},
{RPC_S_UUID_NO_ADDRESS,"RPC_S_UUID_NO_ADDRESS"},
{RPC_S_DUPLICATE_ENDPOINT,"RPC_S_DUPLICATE_ENDPOINT"},
{RPC_S_UNKNOWN_AUTHN_TYPE,"RPC_S_UNKNOWN_AUTHN_TYPE"},
{RPC_S_MAX_CALLS_TOO_SMALL,"RPC_S_MAX_CALLS_TOO_SMALL"},
{RPC_S_STRING_TOO_LONG,"RPC_S_STRING_TOO_LONG"},
{RPC_S_PROTSEQ_NOT_FOUND,"RPC_S_PROTSEQ_NOT_FOUND"},
{RPC_S_PROCNUM_OUT_OF_RANGE,"RPC_S_PROCNUM_OUT_OF_RANGE"},
{RPC_S_BINDING_HAS_NO_AUTH,"RPC_S_BINDING_HAS_NO_AUTH"},
{RPC_S_UNKNOWN_AUTHN_SERVICE,"RPC_S_UNKNOWN_AUTHN_SERVICE"},
{RPC_S_UNKNOWN_AUTHN_LEVEL,"RPC_S_UNKNOWN_AUTHN_LEVEL"},
{RPC_S_INVALID_AUTH_IDENTITY,"RPC_S_INVALID_AUTH_IDENTITY"},
{RPC_S_UNKNOWN_AUTHZ_SERVICE,"RPC_S_UNKNOWN_AUTHZ_SERVICE"},
{EPT_S_INVALID_ENTRY,"EPT_S_INVALID_ENTRY"},
{EPT_S_CANT_PERFORM_OP,"EPT_S_CANT_PERFORM_OP"},
{EPT_S_NOT_REGISTERED,"EPT_S_NOT_REGISTERED"},
{RPC_S_NOTHING_TO_EXPORT,"RPC_S_NOTHING_TO_EXPORT"},
{RPC_S_INCOMPLETE_NAME,"RPC_S_INCOMPLETE_NAME"},
{RPC_S_INVALID_VERS_OPTION,"RPC_S_INVALID_VERS_OPTION"},
{RPC_S_NO_MORE_MEMBERS,"RPC_S_NO_MORE_MEMBERS"},
{RPC_S_NOT_ALL_OBJS_UNEXPORTED,"RPC_S_NOT_ALL_OBJS_UNEXPORTED"},
{RPC_S_INTERFACE_NOT_FOUND,"RPC_S_INTERFACE_NOT_FOUND"},
{RPC_S_ENTRY_ALREADY_EXISTS,"RPC_S_ENTRY_ALREADY_EXISTS"},
{RPC_S_ENTRY_NOT_FOUND,"RPC_S_ENTRY_NOT_FOUND"},
{RPC_S_NAME_SERVICE_UNAVAILABLE,"RPC_S_NAME_SERVICE_UNAVAILABLE"},
{RPC_S_INVALID_NAF_ID,"RPC_S_INVALID_NAF_ID"},
{RPC_S_CANNOT_SUPPORT,"RPC_S_CANNOT_SUPPORT"},
{RPC_S_NO_CONTEXT_AVAILABLE,"RPC_S_NO_CONTEXT_AVAILABLE"},
{RPC_S_INTERNAL_ERROR,"RPC_S_INTERNAL_ERROR"},
{RPC_S_ZERO_DIVIDE,"RPC_S_ZERO_DIVIDE"},
{RPC_S_ADDRESS_ERROR,"RPC_S_ADDRESS_ERROR"},
{RPC_S_FP_DIV_ZERO,"RPC_S_FP_DIV_ZERO"},
{RPC_S_FP_UNDERFLOW,"RPC_S_FP_UNDERFLOW"},
{RPC_S_FP_OVERFLOW,"RPC_S_FP_OVERFLOW"},
{RPC_X_NO_MORE_ENTRIES,"RPC_X_NO_MORE_ENTRIES"},
{RPC_X_SS_CHAR_TRANS_OPEN_FAIL,"RPC_X_SS_CHAR_TRANS_OPEN_FAIL"},
{RPC_X_SS_CHAR_TRANS_SHORT_FILE,"RPC_X_SS_CHAR_TRANS_SHORT_FILE"},
{RPC_X_SS_IN_NULL_CONTEXT,"RPC_X_SS_IN_NULL_CONTEXT"},
{RPC_X_SS_CONTEXT_DAMAGED,"RPC_X_SS_CONTEXT_DAMAGED"},
{RPC_X_SS_HANDLES_MISMATCH,"RPC_X_SS_HANDLES_MISMATCH"},
{RPC_X_SS_CANNOT_GET_CALL_HANDLE,"RPC_X_SS_CANNOT_GET_CALL_HANDLE"},
{RPC_X_NULL_REF_POINTER,"RPC_X_NULL_REF_POINTER"},
{RPC_X_ENUM_VALUE_OUT_OF_RANGE,"RPC_X_ENUM_VALUE_OUT_OF_RANGE"},
{RPC_X_BYTE_COUNT_TOO_SMALL,"RPC_X_BYTE_COUNT_TOO_SMALL"},
{RPC_X_BAD_STUB_DATA,"RPC_X_BAD_STUB_DATA"},
{ERROR_INVALID_USER_BUFFER,"ERROR_INVALID_USER_BUFFER"},
{ERROR_UNRECOGNIZED_MEDIA,"ERROR_UNRECOGNIZED_MEDIA"},
{ERROR_NO_TRUST_LSA_SECRET,"ERROR_NO_TRUST_LSA_SECRET"},
{ERROR_NO_TRUST_SAM_ACCOUNT,"ERROR_NO_TRUST_SAM_ACCOUNT"},
{ERROR_TRUSTED_DOMAIN_FAILURE,"ERROR_TRUSTED_DOMAIN_FAILURE"},
{ERROR_TRUSTED_RELATIONSHIP_FAILURE,"ERROR_TRUSTED_RELATIONSHIP_FAILURE"},
{ERROR_TRUST_FAILURE,"ERROR_TRUST_FAILURE"},
{RPC_S_CALL_IN_PROGRESS,"RPC_S_CALL_IN_PROGRESS"},
{ERROR_NETLOGON_NOT_STARTED,"ERROR_NETLOGON_NOT_STARTED"},
{ERROR_ACCOUNT_EXPIRED,"ERROR_ACCOUNT_EXPIRED"},
{ERROR_REDIRECTOR_HAS_OPEN_HANDLES,"ERROR_REDIRECTOR_HAS_OPEN_HANDLES"},
{ERROR_PRINTER_DRIVER_ALREADY_INSTALLED,"ERROR_PRINTER_DRIVER_ALREADY_INSTALLED"},
{ERROR_UNKNOWN_PORT,"ERROR_UNKNOWN_PORT"},
{ERROR_UNKNOWN_PRINTER_DRIVER,"ERROR_UNKNOWN_PRINTER_DRIVER"},
{ERROR_UNKNOWN_PRINTPROCESSOR,"ERROR_UNKNOWN_PRINTPROCESSOR"},
{ERROR_INVALID_SEPARATOR_FILE,"ERROR_INVALID_SEPARATOR_FILE"},
{ERROR_INVALID_PRIORITY,"ERROR_INVALID_PRIORITY"},
{ERROR_INVALID_PRINTER_NAME,"ERROR_INVALID_PRINTER_NAME"},
{ERROR_PRINTER_ALREADY_EXISTS,"ERROR_PRINTER_ALREADY_EXISTS"},
{ERROR_INVALID_PRINTER_COMMAND,"ERROR_INVALID_PRINTER_COMMAND"},
{ERROR_INVALID_DATATYPE,"ERROR_INVALID_DATATYPE"},
{ERROR_INVALID_ENVIRONMENT,"ERROR_INVALID_ENVIRONMENT"},
{RPC_S_NO_MORE_BINDINGS,"RPC_S_NO_MORE_BINDINGS"},
{ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT,"ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT"},
{ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT,"ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT"},
{ERROR_NOLOGON_SERVER_TRUST_ACCOUNT,"ERROR_NOLOGON_SERVER_TRUST_ACCOUNT"},
{ERROR_DOMAIN_TRUST_INCONSISTENT,"ERROR_DOMAIN_TRUST_INCONSISTENT"},
{ERROR_SERVER_HAS_OPEN_HANDLES,"ERROR_SERVER_HAS_OPEN_HANDLES"},
{ERROR_RESOURCE_DATA_NOT_FOUND,"ERROR_RESOURCE_DATA_NOT_FOUND"},
{ERROR_RESOURCE_TYPE_NOT_FOUND,"ERROR_RESOURCE_TYPE_NOT_FOUND"},
{ERROR_RESOURCE_NAME_NOT_FOUND,"ERROR_RESOURCE_NAME_NOT_FOUND"},
{ERROR_RESOURCE_LANG_NOT_FOUND,"ERROR_RESOURCE_LANG_NOT_FOUND"},
{ERROR_NOT_ENOUGH_QUOTA,"ERROR_NOT_ENOUGH_QUOTA"},
{RPC_S_NO_INTERFACES,"RPC_S_NO_INTERFACES"},
{RPC_S_CALL_CANCELLED,"RPC_S_CALL_CANCELLED"},
{RPC_S_BINDING_INCOMPLETE,"RPC_S_BINDING_INCOMPLETE"},
{RPC_S_COMM_FAILURE,"RPC_S_COMM_FAILURE"},
{RPC_S_UNSUPPORTED_AUTHN_LEVEL,"RPC_S_UNSUPPORTED_AUTHN_LEVEL"},
{RPC_S_NO_PRINC_NAME,"RPC_S_NO_PRINC_NAME"},
{RPC_S_NOT_RPC_ERROR,"RPC_S_NOT_RPC_ERROR"},
{RPC_S_UUID_LOCAL_ONLY,"RPC_S_UUID_LOCAL_ONLY"},
{RPC_S_SEC_PKG_ERROR,"RPC_S_SEC_PKG_ERROR"},
{RPC_S_NOT_CANCELLED,"RPC_S_NOT_CANCELLED"},
{RPC_X_INVALID_ES_ACTION,"RPC_X_INVALID_ES_ACTION"},
{RPC_X_WRONG_ES_VERSION,"RPC_X_WRONG_ES_VERSION"},
{RPC_X_WRONG_STUB_VERSION,"RPC_X_WRONG_STUB_VERSION"},
{RPC_X_INVALID_PIPE_OBJECT,"RPC_X_INVALID_PIPE_OBJECT"},
{RPC_X_WRONG_PIPE_ORDER,"RPC_X_WRONG_PIPE_ORDER"},
{RPC_X_WRONG_PIPE_VERSION,"RPC_X_WRONG_PIPE_VERSION"},
{RPC_S_GROUP_MEMBER_NOT_FOUND,"RPC_S_GROUP_MEMBER_NOT_FOUND"},
{EPT_S_CANT_CREATE,"EPT_S_CANT_CREATE"},
{RPC_S_INVALID_OBJECT,"RPC_S_INVALID_OBJECT"},
{ERROR_INVALID_TIME,"ERROR_INVALID_TIME"},
{ERROR_INVALID_FORM_NAME,"ERROR_INVALID_FORM_NAME"},
{ERROR_INVALID_FORM_SIZE,"ERROR_INVALID_FORM_SIZE"},
{ERROR_ALREADY_WAITING,"ERROR_ALREADY_WAITING"},
{ERROR_PRINTER_DELETED,"ERROR_PRINTER_DELETED"},
{ERROR_INVALID_PRINTER_STATE,"ERROR_INVALID_PRINTER_STATE"},
{ERROR_PASSWORD_MUST_CHANGE,"ERROR_PASSWORD_MUST_CHANGE"},
{ERROR_DOMAIN_CONTROLLER_NOT_FOUND,"ERROR_DOMAIN_CONTROLLER_NOT_FOUND"},
{ERROR_ACCOUNT_LOCKED_OUT,"ERROR_ACCOUNT_LOCKED_OUT"},
{OR_INVALID_OXID,"OR_INVALID_OXID"},
{OR_INVALID_OID,"OR_INVALID_OID"},
{OR_INVALID_SET,"OR_INVALID_SET"},
{RPC_S_SEND_INCOMPLETE,"RPC_S_SEND_INCOMPLETE"},
{RPC_S_INVALID_ASYNC_HANDLE,"RPC_S_INVALID_ASYNC_HANDLE"},
{RPC_S_INVALID_ASYNC_CALL,"RPC_S_INVALID_ASYNC_CALL"},
{RPC_X_PIPE_CLOSED,"RPC_X_PIPE_CLOSED"},
{RPC_X_PIPE_DISCIPLINE_ERROR,"RPC_X_PIPE_DISCIPLINE_ERROR"},
{RPC_X_PIPE_EMPTY,"RPC_X_PIPE_EMPTY"},
{ERROR_NO_SITENAME,"ERROR_NO_SITENAME"},
{ERROR_CANT_ACCESS_FILE,"ERROR_CANT_ACCESS_FILE"},
{ERROR_CANT_RESOLVE_FILENAME,"ERROR_CANT_RESOLVE_FILENAME"},
{RPC_S_ENTRY_TYPE_MISMATCH,"RPC_S_ENTRY_TYPE_MISMATCH"},
{RPC_S_NOT_ALL_OBJS_EXPORTED,"RPC_S_NOT_ALL_OBJS_EXPORTED"},
{RPC_S_INTERFACE_NOT_EXPORTED,"RPC_S_INTERFACE_NOT_EXPORTED"},
{RPC_S_PROFILE_NOT_ADDED,"RPC_S_PROFILE_NOT_ADDED"},
{RPC_S_PRF_ELT_NOT_ADDED,"RPC_S_PRF_ELT_NOT_ADDED"},
{RPC_S_PRF_ELT_NOT_REMOVED,"RPC_S_PRF_ELT_NOT_REMOVED"},
{RPC_S_GRP_ELT_NOT_ADDED,"RPC_S_GRP_ELT_NOT_ADDED"},
{RPC_S_GRP_ELT_NOT_REMOVED,"RPC_S_GRP_ELT_NOT_REMOVED"},
{ERROR_KM_DRIVER_BLOCKED,"ERROR_KM_DRIVER_BLOCKED"},
{ERROR_CONTEXT_EXPIRED,"ERROR_CONTEXT_EXPIRED"},
{ERROR_INVALID_PIXEL_FORMAT,"ERROR_INVALID_PIXEL_FORMAT"},
{ERROR_BAD_DRIVER,"ERROR_BAD_DRIVER"},
{ERROR_INVALID_WINDOW_STYLE,"ERROR_INVALID_WINDOW_STYLE"},
{ERROR_METAFILE_NOT_SUPPORTED,"ERROR_METAFILE_NOT_SUPPORTED"},
{ERROR_TRANSFORM_NOT_SUPPORTED,"ERROR_TRANSFORM_NOT_SUPPORTED"},
{ERROR_CLIPPING_NOT_SUPPORTED,"ERROR_CLIPPING_NOT_SUPPORTED"},
{ERROR_INVALID_CMM,"ERROR_INVALID_CMM"},
{ERROR_INVALID_PROFILE,"ERROR_INVALID_PROFILE"},
{ERROR_TAG_NOT_FOUND,"ERROR_TAG_NOT_FOUND"},
{ERROR_TAG_NOT_PRESENT,"ERROR_TAG_NOT_PRESENT"},
{ERROR_DUPLICATE_TAG,"ERROR_DUPLICATE_TAG"},
{ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE,"ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE"},
{ERROR_PROFILE_NOT_FOUND,"ERROR_PROFILE_NOT_FOUND"},
{ERROR_INVALID_COLORSPACE,"ERROR_INVALID_COLORSPACE"},
{ERROR_ICM_NOT_ENABLED,"ERROR_ICM_NOT_ENABLED"},
{ERROR_DELETING_ICM_XFORM,"ERROR_DELETING_ICM_XFORM"},
{ERROR_INVALID_TRANSFORM,"ERROR_INVALID_TRANSFORM"},
{ERROR_COLORSPACE_MISMATCH,"ERROR_COLORSPACE_MISMATCH"},
{ERROR_INVALID_COLORINDEX,"ERROR_INVALID_COLORINDEX"},
{ERROR_CONNECTED_OTHER_PASSWORD,"ERROR_CONNECTED_OTHER_PASSWORD"},
{ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT,"ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT"},
{ERROR_BAD_USERNAME,"ERROR_BAD_USERNAME"},
{ERROR_NOT_CONNECTED,"ERROR_NOT_CONNECTED"},
{ERROR_OPEN_FILES,"ERROR_OPEN_FILES"},
{ERROR_ACTIVE_CONNECTIONS,"ERROR_ACTIVE_CONNECTIONS"},
{ERROR_DEVICE_IN_USE,"ERROR_DEVICE_IN_USE"},
{ERROR_UNKNOWN_PRINT_MONITOR,"ERROR_UNKNOWN_PRINT_MONITOR"},
{ERROR_PRINTER_DRIVER_IN_USE,"ERROR_PRINTER_DRIVER_IN_USE"},
{ERROR_SPOOL_FILE_NOT_FOUND,"ERROR_SPOOL_FILE_NOT_FOUND"},
{ERROR_SPL_NO_STARTDOC,"ERROR_SPL_NO_STARTDOC"},
{ERROR_SPL_NO_ADDJOB,"ERROR_SPL_NO_ADDJOB"},
{ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED,"ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED"},
{ERROR_PRINT_MONITOR_ALREADY_INSTALLED,"ERROR_PRINT_MONITOR_ALREADY_INSTALLED"},
{ERROR_INVALID_PRINT_MONITOR,"ERROR_INVALID_PRINT_MONITOR"},
{ERROR_PRINT_MONITOR_IN_USE,"ERROR_PRINT_MONITOR_IN_USE"},
{ERROR_PRINTER_HAS_JOBS_QUEUED,"ERROR_PRINTER_HAS_JOBS_QUEUED"},
{ERROR_SUCCESS_REBOOT_REQUIRED,"ERROR_SUCCESS_REBOOT_REQUIRED"},
{ERROR_SUCCESS_RESTART_REQUIRED,"ERROR_SUCCESS_RESTART_REQUIRED"},
{ERROR_PRINTER_NOT_FOUND,"ERROR_PRINTER_NOT_FOUND"},
{ERROR_PRINTER_DRIVER_WARNED,"ERROR_PRINTER_DRIVER_WARNED"},
{ERROR_PRINTER_DRIVER_BLOCKED,"ERROR_PRINTER_DRIVER_BLOCKED"},
{ERROR_WINS_INTERNAL,"ERROR_WINS_INTERNAL"},
{ERROR_CAN_NOT_DEL_LOCAL_WINS,"ERROR_CAN_NOT_DEL_LOCAL_WINS"},
{ERROR_STATIC_INIT,"ERROR_STATIC_INIT"},
{ERROR_INC_BACKUP,"ERROR_INC_BACKUP"},
{ERROR_FULL_BACKUP,"ERROR_FULL_BACKUP"},
{ERROR_REC_NON_EXISTENT,"ERROR_REC_NON_EXISTENT"},
{ERROR_RPL_NOT_ALLOWED,"ERROR_RPL_NOT_ALLOWED"},
{ERROR_DHCP_ADDRESS_CONFLICT,"ERROR_DHCP_ADDRESS_CONFLICT"},
{ERROR_WMI_GUID_NOT_FOUND,"ERROR_WMI_GUID_NOT_FOUND"},
{ERROR_WMI_INSTANCE_NOT_FOUND,"ERROR_WMI_INSTANCE_NOT_FOUND"},
{ERROR_WMI_ITEMID_NOT_FOUND,"ERROR_WMI_ITEMID_NOT_FOUND"},
{ERROR_WMI_TRY_AGAIN,"ERROR_WMI_TRY_AGAIN"},
{ERROR_WMI_DP_NOT_FOUND,"ERROR_WMI_DP_NOT_FOUND"},
{ERROR_WMI_UNRESOLVED_INSTANCE_REF,"ERROR_WMI_UNRESOLVED_INSTANCE_REF"},
{ERROR_WMI_ALREADY_ENABLED,"ERROR_WMI_ALREADY_ENABLED"},
{ERROR_WMI_GUID_DISCONNECTED,"ERROR_WMI_GUID_DISCONNECTED"},
{ERROR_WMI_SERVER_UNAVAILABLE,"ERROR_WMI_SERVER_UNAVAILABLE"},
{ERROR_WMI_DP_FAILED,"ERROR_WMI_DP_FAILED"},
{ERROR_WMI_INVALID_MOF,"ERROR_WMI_INVALID_MOF"},
{ERROR_WMI_INVALID_REGINFO,"ERROR_WMI_INVALID_REGINFO"},
{ERROR_WMI_ALREADY_DISABLED,"ERROR_WMI_ALREADY_DISABLED"},
{ERROR_WMI_READ_ONLY,"ERROR_WMI_READ_ONLY"},
{ERROR_WMI_SET_FAILURE,"ERROR_WMI_SET_FAILURE"},
{ERROR_INVALID_MEDIA,"ERROR_INVALID_MEDIA"},
{ERROR_INVALID_LIBRARY,"ERROR_INVALID_LIBRARY"},
{ERROR_INVALID_MEDIA_POOL,"ERROR_INVALID_MEDIA_POOL"},
{ERROR_DRIVE_MEDIA_MISMATCH,"ERROR_DRIVE_MEDIA_MISMATCH"},
{ERROR_MEDIA_OFFLINE,"ERROR_MEDIA_OFFLINE"},
{ERROR_LIBRARY_OFFLINE,"ERROR_LIBRARY_OFFLINE"},
{ERROR_EMPTY,"ERROR_EMPTY"},
{ERROR_NOT_EMPTY,"ERROR_NOT_EMPTY"},
{ERROR_MEDIA_UNAVAILABLE,"ERROR_MEDIA_UNAVAILABLE"},
{ERROR_RESOURCE_DISABLED,"ERROR_RESOURCE_DISABLED"},
{ERROR_INVALID_CLEANER,"ERROR_INVALID_CLEANER"},
{ERROR_UNABLE_TO_CLEAN,"ERROR_UNABLE_TO_CLEAN"},
{ERROR_OBJECT_NOT_FOUND,"ERROR_OBJECT_NOT_FOUND"},
{ERROR_DATABASE_FAILURE,"ERROR_DATABASE_FAILURE"},
{ERROR_DATABASE_FULL,"ERROR_DATABASE_FULL"},
{ERROR_MEDIA_INCOMPATIBLE,"ERROR_MEDIA_INCOMPATIBLE"},
{ERROR_RESOURCE_NOT_PRESENT,"ERROR_RESOURCE_NOT_PRESENT"},
{ERROR_INVALID_OPERATION,"ERROR_INVALID_OPERATION"},
{ERROR_MEDIA_NOT_AVAILABLE,"ERROR_MEDIA_NOT_AVAILABLE"},
{ERROR_DEVICE_NOT_AVAILABLE,"ERROR_DEVICE_NOT_AVAILABLE"},
{ERROR_REQUEST_REFUSED,"ERROR_REQUEST_REFUSED"},
{ERROR_INVALID_DRIVE_OBJECT,"ERROR_INVALID_DRIVE_OBJECT"},
{ERROR_LIBRARY_FULL,"ERROR_LIBRARY_FULL"},
{ERROR_MEDIUM_NOT_ACCESSIBLE,"ERROR_MEDIUM_NOT_ACCESSIBLE"},
{ERROR_UNABLE_TO_LOAD_MEDIUM,"ERROR_UNABLE_TO_LOAD_MEDIUM"},
{ERROR_UNABLE_TO_INVENTORY_DRIVE,"ERROR_UNABLE_TO_INVENTORY_DRIVE"},
{ERROR_UNABLE_TO_INVENTORY_SLOT,"ERROR_UNABLE_TO_INVENTORY_SLOT"},
{ERROR_UNABLE_TO_INVENTORY_TRANSPORT,"ERROR_UNABLE_TO_INVENTORY_TRANSPORT"},
{ERROR_TRANSPORT_FULL,"ERROR_TRANSPORT_FULL"},
{ERROR_CONTROLLING_IEPORT,"ERROR_CONTROLLING_IEPORT"},
{ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA,"ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA"},
{ERROR_CLEANER_SLOT_SET,"ERROR_CLEANER_SLOT_SET"},
{ERROR_CLEANER_SLOT_NOT_SET,"ERROR_CLEANER_SLOT_NOT_SET"},
{ERROR_CLEANER_CARTRIDGE_SPENT,"ERROR_CLEANER_CARTRIDGE_SPENT"},
{ERROR_UNEXPECTED_OMID,"ERROR_UNEXPECTED_OMID"},
{ERROR_CANT_DELETE_LAST_ITEM,"ERROR_CANT_DELETE_LAST_ITEM"},
{ERROR_MESSAGE_EXCEEDS_MAX_SIZE,"ERROR_MESSAGE_EXCEEDS_MAX_SIZE"},
{ERROR_VOLUME_CONTAINS_SYS_FILES,"ERROR_VOLUME_CONTAINS_SYS_FILES"},
{ERROR_INDIGENOUS_TYPE,"ERROR_INDIGENOUS_TYPE"},
{ERROR_NO_SUPPORTING_DRIVES,"ERROR_NO_SUPPORTING_DRIVES"},
{ERROR_CLEANER_CARTRIDGE_INSTALLED,"ERROR_CLEANER_CARTRIDGE_INSTALLED"},
{ERROR_FILE_OFFLINE,"ERROR_FILE_OFFLINE"},
{ERROR_REMOTE_STORAGE_NOT_ACTIVE,"ERROR_REMOTE_STORAGE_NOT_ACTIVE"},
{ERROR_REMOTE_STORAGE_MEDIA_ERROR,"ERROR_REMOTE_STORAGE_MEDIA_ERROR"},
{ERROR_NOT_A_REPARSE_POINT,"ERROR_NOT_A_REPARSE_POINT"},
{ERROR_REPARSE_ATTRIBUTE_CONFLICT,"ERROR_REPARSE_ATTRIBUTE_CONFLICT"},
{ERROR_INVALID_REPARSE_DATA,"ERROR_INVALID_REPARSE_DATA"},
{ERROR_REPARSE_TAG_INVALID,"ERROR_REPARSE_TAG_INVALID"},
{ERROR_REPARSE_TAG_MISMATCH,"ERROR_REPARSE_TAG_MISMATCH"},
{ERROR_VOLUME_NOT_SIS_ENABLED,"ERROR_VOLUME_NOT_SIS_ENABLED"},
{ERROR_DEPENDENT_RESOURCE_EXISTS,"ERROR_DEPENDENT_RESOURCE_EXISTS"},
{ERROR_DEPENDENCY_NOT_FOUND,"ERROR_DEPENDENCY_NOT_FOUND"},
{ERROR_DEPENDENCY_ALREADY_EXISTS,"ERROR_DEPENDENCY_ALREADY_EXISTS"},
{ERROR_RESOURCE_NOT_ONLINE,"ERROR_RESOURCE_NOT_ONLINE"},
{ERROR_HOST_NODE_NOT_AVAILABLE,"ERROR_HOST_NODE_NOT_AVAILABLE"},
{ERROR_RESOURCE_NOT_AVAILABLE,"ERROR_RESOURCE_NOT_AVAILABLE"},
{ERROR_RESOURCE_NOT_FOUND,"ERROR_RESOURCE_NOT_FOUND"},
{ERROR_SHUTDOWN_CLUSTER,"ERROR_SHUTDOWN_CLUSTER"},
{ERROR_CANT_EVICT_ACTIVE_NODE,"ERROR_CANT_EVICT_ACTIVE_NODE"},
{ERROR_OBJECT_ALREADY_EXISTS,"ERROR_OBJECT_ALREADY_EXISTS"},
{ERROR_OBJECT_IN_LIST,"ERROR_OBJECT_IN_LIST"},
{ERROR_GROUP_NOT_AVAILABLE,"ERROR_GROUP_NOT_AVAILABLE"},
{ERROR_GROUP_NOT_FOUND,"ERROR_GROUP_NOT_FOUND"},
{ERROR_GROUP_NOT_ONLINE,"ERROR_GROUP_NOT_ONLINE"},
{ERROR_HOST_NODE_NOT_RESOURCE_OWNER,"ERROR_HOST_NODE_NOT_RESOURCE_OWNER"},
{ERROR_HOST_NODE_NOT_GROUP_OWNER,"ERROR_HOST_NODE_NOT_GROUP_OWNER"},
{ERROR_RESMON_CREATE_FAILED,"ERROR_RESMON_CREATE_FAILED"},
{ERROR_RESMON_ONLINE_FAILED,"ERROR_RESMON_ONLINE_FAILED"},
{ERROR_RESOURCE_ONLINE,"ERROR_RESOURCE_ONLINE"},
{ERROR_QUORUM_RESOURCE,"ERROR_QUORUM_RESOURCE"},
{ERROR_NOT_QUORUM_CAPABLE,"ERROR_NOT_QUORUM_CAPABLE"},
{ERROR_CLUSTER_SHUTTING_DOWN,"ERROR_CLUSTER_SHUTTING_DOWN"},
{ERROR_INVALID_STATE,"ERROR_INVALID_STATE"},
{ERROR_RESOURCE_PROPERTIES_STORED,"ERROR_RESOURCE_PROPERTIES_STORED"},
{ERROR_NOT_QUORUM_CLASS,"ERROR_NOT_QUORUM_CLASS"},
{ERROR_CORE_RESOURCE,"ERROR_CORE_RESOURCE"},
{ERROR_QUORUM_RESOURCE_ONLINE_FAILED,"ERROR_QUORUM_RESOURCE_ONLINE_FAILED"},
{ERROR_QUORUMLOG_OPEN_FAILED,"ERROR_QUORUMLOG_OPEN_FAILED"},
{ERROR_CLUSTERLOG_CORRUPT,"ERROR_CLUSTERLOG_CORRUPT"},
{ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE,"ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE"},
{ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE,"ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE"},
{ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND,"ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND"},
{ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE,"ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE"},
{ERROR_QUORUM_OWNER_ALIVE,"ERROR_QUORUM_OWNER_ALIVE"},
{ERROR_NETWORK_NOT_AVAILABLE,"ERROR_NETWORK_NOT_AVAILABLE"},
{ERROR_NODE_NOT_AVAILABLE,"ERROR_NODE_NOT_AVAILABLE"},
{ERROR_ALL_NODES_NOT_AVAILABLE,"ERROR_ALL_NODES_NOT_AVAILABLE"},
{ERROR_RESOURCE_FAILED,"ERROR_RESOURCE_FAILED"},
{ERROR_CLUSTER_INVALID_NODE,"ERROR_CLUSTER_INVALID_NODE"},
{ERROR_CLUSTER_NODE_EXISTS,"ERROR_CLUSTER_NODE_EXISTS"},
{ERROR_CLUSTER_JOIN_IN_PROGRESS,"ERROR_CLUSTER_JOIN_IN_PROGRESS"},
{ERROR_CLUSTER_NODE_NOT_FOUND,"ERROR_CLUSTER_NODE_NOT_FOUND"},
{ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND,"ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND"},
{ERROR_CLUSTER_NETWORK_EXISTS,"ERROR_CLUSTER_NETWORK_EXISTS"},
{ERROR_CLUSTER_NETWORK_NOT_FOUND,"ERROR_CLUSTER_NETWORK_NOT_FOUND"},
{ERROR_CLUSTER_NETINTERFACE_EXISTS,"ERROR_CLUSTER_NETINTERFACE_EXISTS"},
{ERROR_CLUSTER_NETINTERFACE_NOT_FOUND,"ERROR_CLUSTER_NETINTERFACE_NOT_FOUND"},
{ERROR_CLUSTER_INVALID_REQUEST,"ERROR_CLUSTER_INVALID_REQUEST"},
{ERROR_CLUSTER_INVALID_NETWORK_PROVIDER,"ERROR_CLUSTER_INVALID_NETWORK_PROVIDER"},
{ERROR_CLUSTER_NODE_DOWN,"ERROR_CLUSTER_NODE_DOWN"},
{ERROR_CLUSTER_NODE_UNREACHABLE,"ERROR_CLUSTER_NODE_UNREACHABLE"},
{ERROR_CLUSTER_NODE_NOT_MEMBER,"ERROR_CLUSTER_NODE_NOT_MEMBER"},
{ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS,"ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS"},
{ERROR_CLUSTER_INVALID_NETWORK,"ERROR_CLUSTER_INVALID_NETWORK"},
{ERROR_CLUSTER_NODE_UP,"ERROR_CLUSTER_NODE_UP"},
{ERROR_CLUSTER_IPADDR_IN_USE,"ERROR_CLUSTER_IPADDR_IN_USE"},
{ERROR_CLUSTER_NODE_NOT_PAUSED,"ERROR_CLUSTER_NODE_NOT_PAUSED"},
{ERROR_CLUSTER_NO_SECURITY_CONTEXT,"ERROR_CLUSTER_NO_SECURITY_CONTEXT"},
{ERROR_CLUSTER_NETWORK_NOT_INTERNAL,"ERROR_CLUSTER_NETWORK_NOT_INTERNAL"},
{ERROR_CLUSTER_NODE_ALREADY_UP,"ERROR_CLUSTER_NODE_ALREADY_UP"},
{ERROR_CLUSTER_NODE_ALREADY_DOWN,"ERROR_CLUSTER_NODE_ALREADY_DOWN"},
{ERROR_CLUSTER_NETWORK_ALREADY_ONLINE,"ERROR_CLUSTER_NETWORK_ALREADY_ONLINE"},
{ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE,"ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE"},
{ERROR_CLUSTER_NODE_ALREADY_MEMBER,"ERROR_CLUSTER_NODE_ALREADY_MEMBER"},
{ERROR_CLUSTER_LAST_INTERNAL_NETWORK,"ERROR_CLUSTER_LAST_INTERNAL_NETWORK"},
{ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS,"ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS"},
{ERROR_INVALID_OPERATION_ON_QUORUM,"ERROR_INVALID_OPERATION_ON_QUORUM"},
{ERROR_DEPENDENCY_NOT_ALLOWED,"ERROR_DEPENDENCY_NOT_ALLOWED"},
{ERROR_CLUSTER_NODE_PAUSED,"ERROR_CLUSTER_NODE_PAUSED"},
{ERROR_NODE_CANT_HOST_RESOURCE,"ERROR_NODE_CANT_HOST_RESOURCE"},
{ERROR_CLUSTER_NODE_NOT_READY,"ERROR_CLUSTER_NODE_NOT_READY"},
{ERROR_CLUSTER_NODE_SHUTTING_DOWN,"ERROR_CLUSTER_NODE_SHUTTING_DOWN"},
{ERROR_CLUSTER_JOIN_ABORTED,"ERROR_CLUSTER_JOIN_ABORTED"},
{ERROR_CLUSTER_INCOMPATIBLE_VERSIONS,"ERROR_CLUSTER_INCOMPATIBLE_VERSIONS"},
{ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED,"ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED"},
{ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED,"ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED"},
{ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND,"ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND"},
{ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED,"ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED"},
{ERROR_CLUSTER_RESNAME_NOT_FOUND,"ERROR_CLUSTER_RESNAME_NOT_FOUND"},
{ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED,"ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED"},
{ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST,"ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST"},
{ERROR_CLUSTER_DATABASE_SEQMISMATCH,"ERROR_CLUSTER_DATABASE_SEQMISMATCH"},
{ERROR_RESMON_INVALID_STATE,"ERROR_RESMON_INVALID_STATE"},
{ERROR_CLUSTER_GUM_NOT_LOCKER,"ERROR_CLUSTER_GUM_NOT_LOCKER"},
{ERROR_QUORUM_DISK_NOT_FOUND,"ERROR_QUORUM_DISK_NOT_FOUND"},
{ERROR_DATABASE_BACKUP_CORRUPT,"ERROR_DATABASE_BACKUP_CORRUPT"},
{ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT,"ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT"},
{ERROR_RESOURCE_PROPERTY_UNCHANGEABLE,"ERROR_RESOURCE_PROPERTY_UNCHANGEABLE"},
{ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE,"ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE"},
{ERROR_CLUSTER_QUORUMLOG_NOT_FOUND,"ERROR_CLUSTER_QUORUMLOG_NOT_FOUND"},
{ERROR_CLUSTER_MEMBERSHIP_HALT,"ERROR_CLUSTER_MEMBERSHIP_HALT"},
{ERROR_CLUSTER_INSTANCE_ID_MISMATCH,"ERROR_CLUSTER_INSTANCE_ID_MISMATCH"},
{ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP,"ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP"},
{ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH,"ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH"},
{ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP,"ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP"},
{ERROR_CLUSTER_PARAMETER_MISMATCH,"ERROR_CLUSTER_PARAMETER_MISMATCH"},
{ERROR_NODE_CANNOT_BE_CLUSTERED,"ERROR_NODE_CANNOT_BE_CLUSTERED"},
{ERROR_CLUSTER_WRONG_OS_VERSION,"ERROR_CLUSTER_WRONG_OS_VERSION"},
{ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME,"ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME"},
{ERROR_ENCRYPTION_FAILED,"ERROR_ENCRYPTION_FAILED"},
{ERROR_DECRYPTION_FAILED,"ERROR_DECRYPTION_FAILED"},
{ERROR_FILE_ENCRYPTED,"ERROR_FILE_ENCRYPTED"},
{ERROR_NO_RECOVERY_POLICY,"ERROR_NO_RECOVERY_POLICY"},
{ERROR_NO_EFS,"ERROR_NO_EFS"},
{ERROR_WRONG_EFS,"ERROR_WRONG_EFS"},
{ERROR_NO_USER_KEYS,"ERROR_NO_USER_KEYS"},
{ERROR_FILE_NOT_ENCRYPTED,"ERROR_FILE_NOT_ENCRYPTED"},
{ERROR_NOT_EXPORT_FORMAT,"ERROR_NOT_EXPORT_FORMAT"},
{ERROR_FILE_READ_ONLY,"ERROR_FILE_READ_ONLY"},
{ERROR_DIR_EFS_DISALLOWED,"ERROR_DIR_EFS_DISALLOWED"},
{ERROR_EFS_SERVER_NOT_TRUSTED,"ERROR_EFS_SERVER_NOT_TRUSTED"},
{ERROR_BAD_RECOVERY_POLICY,"ERROR_BAD_RECOVERY_POLICY"},
{ERROR_EFS_ALG_BLOB_TOO_BIG,"ERROR_EFS_ALG_BLOB_TOO_BIG"},
{ERROR_VOLUME_NOT_SUPPORT_EFS,"ERROR_VOLUME_NOT_SUPPORT_EFS"},
{ERROR_EFS_DISABLED,"ERROR_EFS_DISABLED"},
{ERROR_EFS_VERSION_NOT_SUPPORT,"ERROR_EFS_VERSION_NOT_SUPPORT"},
{ERROR_NO_BROWSER_SERVERS_FOUND,"ERROR_NO_BROWSER_SERVERS_FOUND"},
{SCHED_E_SERVICE_NOT_LOCALSYSTEM,"SCHED_E_SERVICE_NOT_LOCALSYSTEM"},
{ERROR_CTX_WINSTATION_NAME_INVALID,"ERROR_CTX_WINSTATION_NAME_INVALID"},
{ERROR_CTX_INVALID_PD,"ERROR_CTX_INVALID_PD"},
{ERROR_CTX_PD_NOT_FOUND,"ERROR_CTX_PD_NOT_FOUND"},
{ERROR_CTX_WD_NOT_FOUND,"ERROR_CTX_WD_NOT_FOUND"},
{ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY,"ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY"},
{ERROR_CTX_SERVICE_NAME_COLLISION,"ERROR_CTX_SERVICE_NAME_COLLISION"},
{ERROR_CTX_CLOSE_PENDING,"ERROR_CTX_CLOSE_PENDING"},
{ERROR_CTX_NO_OUTBUF,"ERROR_CTX_NO_OUTBUF"},
{ERROR_CTX_MODEM_INF_NOT_FOUND,"ERROR_CTX_MODEM_INF_NOT_FOUND"},
{ERROR_CTX_INVALID_MODEMNAME,"ERROR_CTX_INVALID_MODEMNAME"},
{ERROR_CTX_MODEM_RESPONSE_ERROR,"ERROR_CTX_MODEM_RESPONSE_ERROR"},
{ERROR_CTX_MODEM_RESPONSE_TIMEOUT,"ERROR_CTX_MODEM_RESPONSE_TIMEOUT"},
{ERROR_CTX_MODEM_RESPONSE_NO_CARRIER,"ERROR_CTX_MODEM_RESPONSE_NO_CARRIER"},
{ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE,"ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE"},
{ERROR_CTX_MODEM_RESPONSE_BUSY,"ERROR_CTX_MODEM_RESPONSE_BUSY"},
{ERROR_CTX_MODEM_RESPONSE_VOICE,"ERROR_CTX_MODEM_RESPONSE_VOICE"},
{ERROR_CTX_TD_ERROR,"ERROR_CTX_TD_ERROR"},
{ERROR_CTX_WINSTATION_NOT_FOUND,"ERROR_CTX_WINSTATION_NOT_FOUND"},
{ERROR_CTX_WINSTATION_ALREADY_EXISTS,"ERROR_CTX_WINSTATION_ALREADY_EXISTS"},
{ERROR_CTX_WINSTATION_BUSY,"ERROR_CTX_WINSTATION_BUSY"},
{ERROR_CTX_BAD_VIDEO_MODE,"ERROR_CTX_BAD_VIDEO_MODE"},
{ERROR_CTX_GRAPHICS_INVALID,"ERROR_CTX_GRAPHICS_INVALID"},
{ERROR_CTX_LOGON_DISABLED,"ERROR_CTX_LOGON_DISABLED"},
{ERROR_CTX_NOT_CONSOLE,"ERROR_CTX_NOT_CONSOLE"},
{ERROR_CTX_CLIENT_QUERY_TIMEOUT,"ERROR_CTX_CLIENT_QUERY_TIMEOUT"},
{ERROR_CTX_CONSOLE_DISCONNECT,"ERROR_CTX_CONSOLE_DISCONNECT"},
{ERROR_CTX_CONSOLE_CONNECT,"ERROR_CTX_CONSOLE_CONNECT"},
{ERROR_CTX_SHADOW_DENIED,"ERROR_CTX_SHADOW_DENIED"},
{ERROR_CTX_WINSTATION_ACCESS_DENIED,"ERROR_CTX_WINSTATION_ACCESS_DENIED"},
{ERROR_CTX_INVALID_WD,"ERROR_CTX_INVALID_WD"},
{ERROR_CTX_SHADOW_INVALID,"ERROR_CTX_SHADOW_INVALID"},
{ERROR_CTX_SHADOW_DISABLED,"ERROR_CTX_SHADOW_DISABLED"},
{ERROR_CTX_CLIENT_LICENSE_IN_USE,"ERROR_CTX_CLIENT_LICENSE_IN_USE"},
{ERROR_CTX_CLIENT_LICENSE_NOT_SET,"ERROR_CTX_CLIENT_LICENSE_NOT_SET"},
{ERROR_CTX_LICENSE_NOT_AVAILABLE,"ERROR_CTX_LICENSE_NOT_AVAILABLE"},
{ERROR_CTX_LICENSE_CLIENT_INVALID,"ERROR_CTX_LICENSE_CLIENT_INVALID"},
{ERROR_CTX_LICENSE_EXPIRED,"ERROR_CTX_LICENSE_EXPIRED"},
{ERROR_CTX_SHADOW_NOT_RUNNING,"ERROR_CTX_SHADOW_NOT_RUNNING"},
{ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE,"ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE"},
{FRS_ERR_INVALID_API_SEQUENCE,"FRS_ERR_INVALID_API_SEQUENCE"},
{FRS_ERR_STARTING_SERVICE,"FRS_ERR_STARTING_SERVICE"},
{FRS_ERR_STOPPING_SERVICE,"FRS_ERR_STOPPING_SERVICE"},
{FRS_ERR_INTERNAL_API,"FRS_ERR_INTERNAL_API"},
{FRS_ERR_INTERNAL,"FRS_ERR_INTERNAL"},
{FRS_ERR_SERVICE_COMM,"FRS_ERR_SERVICE_COMM"},
{FRS_ERR_INSUFFICIENT_PRIV,"FRS_ERR_INSUFFICIENT_PRIV"},
{FRS_ERR_AUTHENTICATION,"FRS_ERR_AUTHENTICATION"},
{FRS_ERR_PARENT_INSUFFICIENT_PRIV,"FRS_ERR_PARENT_INSUFFICIENT_PRIV"},
{FRS_ERR_PARENT_AUTHENTICATION,"FRS_ERR_PARENT_AUTHENTICATION"},
{FRS_ERR_CHILD_TO_PARENT_COMM,"FRS_ERR_CHILD_TO_PARENT_COMM"},
{FRS_ERR_PARENT_TO_CHILD_COMM,"FRS_ERR_PARENT_TO_CHILD_COMM"},
{FRS_ERR_SYSVOL_POPULATE,"FRS_ERR_SYSVOL_POPULATE"},
{FRS_ERR_SYSVOL_POPULATE_TIMEOUT,"FRS_ERR_SYSVOL_POPULATE_TIMEOUT"},
{FRS_ERR_SYSVOL_IS_BUSY,"FRS_ERR_SYSVOL_IS_BUSY"},
{FRS_ERR_SYSVOL_DEMOTE,"FRS_ERR_SYSVOL_DEMOTE"},
{FRS_ERR_INVALID_SERVICE_PARAMETER,"FRS_ERR_INVALID_SERVICE_PARAMETER"},
{DS_S_SUCCESS,"DS_S_SUCCESS"},
{ERROR_DS_NOT_INSTALLED,"ERROR_DS_NOT_INSTALLED"},
{ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY,"ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY"},
{ERROR_DS_NO_ATTRIBUTE_OR_VALUE,"ERROR_DS_NO_ATTRIBUTE_OR_VALUE"},
{ERROR_DS_INVALID_ATTRIBUTE_SYNTAX,"ERROR_DS_INVALID_ATTRIBUTE_SYNTAX"},
{ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED,"ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED"},
{ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS,"ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS"},
{ERROR_DS_BUSY,"ERROR_DS_BUSY"},
{ERROR_DS_UNAVAILABLE,"ERROR_DS_UNAVAILABLE"},
{ERROR_DS_NO_RIDS_ALLOCATED,"ERROR_DS_NO_RIDS_ALLOCATED"},
{ERROR_DS_NO_MORE_RIDS,"ERROR_DS_NO_MORE_RIDS"},
{ERROR_DS_INCORRECT_ROLE_OWNER,"ERROR_DS_INCORRECT_ROLE_OWNER"},
{ERROR_DS_RIDMGR_INIT_ERROR,"ERROR_DS_RIDMGR_INIT_ERROR"},
{ERROR_DS_OBJ_CLASS_VIOLATION,"ERROR_DS_OBJ_CLASS_VIOLATION"},
{ERROR_DS_CANT_ON_NON_LEAF,"ERROR_DS_CANT_ON_NON_LEAF"},
{ERROR_DS_CANT_ON_RDN,"ERROR_DS_CANT_ON_RDN"},
{ERROR_DS_CANT_MOD_OBJ_CLASS,"ERROR_DS_CANT_MOD_OBJ_CLASS"},
{ERROR_DS_CROSS_DOM_MOVE_ERROR,"ERROR_DS_CROSS_DOM_MOVE_ERROR"},
{ERROR_DS_GC_NOT_AVAILABLE,"ERROR_DS_GC_NOT_AVAILABLE"},
{ERROR_SHARED_POLICY,"ERROR_SHARED_POLICY"},
{ERROR_POLICY_OBJECT_NOT_FOUND,"ERROR_POLICY_OBJECT_NOT_FOUND"},
{ERROR_POLICY_ONLY_IN_DS,"ERROR_POLICY_ONLY_IN_DS"},
{ERROR_PROMOTION_ACTIVE,"ERROR_PROMOTION_ACTIVE"},
{ERROR_NO_PROMOTION_ACTIVE,"ERROR_NO_PROMOTION_ACTIVE"},
{ERROR_DS_OPERATIONS_ERROR,"ERROR_DS_OPERATIONS_ERROR"},
{ERROR_DS_PROTOCOL_ERROR,"ERROR_DS_PROTOCOL_ERROR"},
{ERROR_DS_TIMELIMIT_EXCEEDED,"ERROR_DS_TIMELIMIT_EXCEEDED"},
{ERROR_DS_SIZELIMIT_EXCEEDED,"ERROR_DS_SIZELIMIT_EXCEEDED"},
{ERROR_DS_ADMIN_LIMIT_EXCEEDED,"ERROR_DS_ADMIN_LIMIT_EXCEEDED"},
{ERROR_DS_COMPARE_FALSE,"ERROR_DS_COMPARE_FALSE"},
{ERROR_DS_COMPARE_TRUE,"ERROR_DS_COMPARE_TRUE"},
{ERROR_DS_AUTH_METHOD_NOT_SUPPORTED,"ERROR_DS_AUTH_METHOD_NOT_SUPPORTED"},
{ERROR_DS_STRONG_AUTH_REQUIRED,"ERROR_DS_STRONG_AUTH_REQUIRED"},
{ERROR_DS_INAPPROPRIATE_AUTH,"ERROR_DS_INAPPROPRIATE_AUTH"},
{ERROR_DS_AUTH_UNKNOWN,"ERROR_DS_AUTH_UNKNOWN"},
{ERROR_DS_REFERRAL,"ERROR_DS_REFERRAL"},
{ERROR_DS_UNAVAILABLE_CRIT_EXTENSION,"ERROR_DS_UNAVAILABLE_CRIT_EXTENSION"},
{ERROR_DS_CONFIDENTIALITY_REQUIRED,"ERROR_DS_CONFIDENTIALITY_REQUIRED"},
{ERROR_DS_INAPPROPRIATE_MATCHING,"ERROR_DS_INAPPROPRIATE_MATCHING"},
{ERROR_DS_CONSTRAINT_VIOLATION,"ERROR_DS_CONSTRAINT_VIOLATION"},
{ERROR_DS_NO_SUCH_OBJECT,"ERROR_DS_NO_SUCH_OBJECT"},
{ERROR_DS_ALIAS_PROBLEM,"ERROR_DS_ALIAS_PROBLEM"},
{ERROR_DS_INVALID_DN_SYNTAX,"ERROR_DS_INVALID_DN_SYNTAX"},
{ERROR_DS_IS_LEAF,"ERROR_DS_IS_LEAF"},
{ERROR_DS_ALIAS_DEREF_PROBLEM,"ERROR_DS_ALIAS_DEREF_PROBLEM"},
{ERROR_DS_UNWILLING_TO_PERFORM,"ERROR_DS_UNWILLING_TO_PERFORM"},
{ERROR_DS_LOOP_DETECT,"ERROR_DS_LOOP_DETECT"},
{ERROR_DS_NAMING_VIOLATION,"ERROR_DS_NAMING_VIOLATION"},
{ERROR_DS_OBJECT_RESULTS_TOO_LARGE,"ERROR_DS_OBJECT_RESULTS_TOO_LARGE"},
{ERROR_DS_AFFECTS_MULTIPLE_DSAS,"ERROR_DS_AFFECTS_MULTIPLE_DSAS"},
{ERROR_DS_SERVER_DOWN,"ERROR_DS_SERVER_DOWN"},
{ERROR_DS_LOCAL_ERROR,"ERROR_DS_LOCAL_ERROR"},
{ERROR_DS_ENCODING_ERROR,"ERROR_DS_ENCODING_ERROR"},
{ERROR_DS_DECODING_ERROR,"ERROR_DS_DECODING_ERROR"},
{ERROR_DS_FILTER_UNKNOWN,"ERROR_DS_FILTER_UNKNOWN"},
{ERROR_DS_PARAM_ERROR,"ERROR_DS_PARAM_ERROR"},
{ERROR_DS_NOT_SUPPORTED,"ERROR_DS_NOT_SUPPORTED"},
{ERROR_DS_NO_RESULTS_RETURNED,"ERROR_DS_NO_RESULTS_RETURNED"},
{ERROR_DS_CONTROL_NOT_FOUND,"ERROR_DS_CONTROL_NOT_FOUND"},
{ERROR_DS_CLIENT_LOOP,"ERROR_DS_CLIENT_LOOP"},
{ERROR_DS_REFERRAL_LIMIT_EXCEEDED,"ERROR_DS_REFERRAL_LIMIT_EXCEEDED"},
{ERROR_DS_SORT_CONTROL_MISSING,"ERROR_DS_SORT_CONTROL_MISSING"},
{ERROR_DS_OFFSET_RANGE_ERROR,"ERROR_DS_OFFSET_RANGE_ERROR"},
{ERROR_DS_ROOT_MUST_BE_NC,"ERROR_DS_ROOT_MUST_BE_NC"},
{ERROR_DS_ADD_REPLICA_INHIBITED,"ERROR_DS_ADD_REPLICA_INHIBITED"},
{ERROR_DS_ATT_NOT_DEF_IN_SCHEMA,"ERROR_DS_ATT_NOT_DEF_IN_SCHEMA"},
{ERROR_DS_MAX_OBJ_SIZE_EXCEEDED,"ERROR_DS_MAX_OBJ_SIZE_EXCEEDED"},
{ERROR_DS_OBJ_STRING_NAME_EXISTS,"ERROR_DS_OBJ_STRING_NAME_EXISTS"},
{ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA,"ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA"},
{ERROR_DS_RDN_DOESNT_MATCH_SCHEMA,"ERROR_DS_RDN_DOESNT_MATCH_SCHEMA"},
{ERROR_DS_NO_REQUESTED_ATTS_FOUND,"ERROR_DS_NO_REQUESTED_ATTS_FOUND"},
{ERROR_DS_USER_BUFFER_TO_SMALL,"ERROR_DS_USER_BUFFER_TO_SMALL"},
{ERROR_DS_ATT_IS_NOT_ON_OBJ,"ERROR_DS_ATT_IS_NOT_ON_OBJ"},
{ERROR_DS_ILLEGAL_MOD_OPERATION,"ERROR_DS_ILLEGAL_MOD_OPERATION"},
{ERROR_DS_OBJ_TOO_LARGE,"ERROR_DS_OBJ_TOO_LARGE"},
{ERROR_DS_BAD_INSTANCE_TYPE,"ERROR_DS_BAD_INSTANCE_TYPE"},
{ERROR_DS_MASTERDSA_REQUIRED,"ERROR_DS_MASTERDSA_REQUIRED"},
{ERROR_DS_OBJECT_CLASS_REQUIRED,"ERROR_DS_OBJECT_CLASS_REQUIRED"},
{ERROR_DS_MISSING_REQUIRED_ATT,"ERROR_DS_MISSING_REQUIRED_ATT"},
{ERROR_DS_ATT_NOT_DEF_FOR_CLASS,"ERROR_DS_ATT_NOT_DEF_FOR_CLASS"},
{ERROR_DS_ATT_ALREADY_EXISTS,"ERROR_DS_ATT_ALREADY_EXISTS"},
{ERROR_DS_CANT_ADD_ATT_VALUES,"ERROR_DS_CANT_ADD_ATT_VALUES"},
{ERROR_DS_SINGLE_VALUE_CONSTRAINT,"ERROR_DS_SINGLE_VALUE_CONSTRAINT"},
{ERROR_DS_RANGE_CONSTRAINT,"ERROR_DS_RANGE_CONSTRAINT"},
{ERROR_DS_ATT_VAL_ALREADY_EXISTS,"ERROR_DS_ATT_VAL_ALREADY_EXISTS"},
{ERROR_DS_CANT_REM_MISSING_ATT,"ERROR_DS_CANT_REM_MISSING_ATT"},
{ERROR_DS_CANT_REM_MISSING_ATT_VAL,"ERROR_DS_CANT_REM_MISSING_ATT_VAL"},
{ERROR_DS_ROOT_CANT_BE_SUBREF,"ERROR_DS_ROOT_CANT_BE_SUBREF"},
{ERROR_DS_NO_CHAINING,"ERROR_DS_NO_CHAINING"},
{ERROR_DS_NO_CHAINED_EVAL,"ERROR_DS_NO_CHAINED_EVAL"},
{ERROR_DS_NO_PARENT_OBJECT,"ERROR_DS_NO_PARENT_OBJECT"},
{ERROR_DS_PARENT_IS_AN_ALIAS,"ERROR_DS_PARENT_IS_AN_ALIAS"},
{ERROR_DS_CANT_MIX_MASTER_AND_REPS,"ERROR_DS_CANT_MIX_MASTER_AND_REPS"},
{ERROR_DS_CHILDREN_EXIST,"ERROR_DS_CHILDREN_EXIST"},
{ERROR_DS_OBJ_NOT_FOUND,"ERROR_DS_OBJ_NOT_FOUND"},
{ERROR_DS_ALIASED_OBJ_MISSING,"ERROR_DS_ALIASED_OBJ_MISSING"},
{ERROR_DS_BAD_NAME_SYNTAX,"ERROR_DS_BAD_NAME_SYNTAX"},
{ERROR_DS_ALIAS_POINTS_TO_ALIAS,"ERROR_DS_ALIAS_POINTS_TO_ALIAS"},
{ERROR_DS_CANT_DEREF_ALIAS,"ERROR_DS_CANT_DEREF_ALIAS"},
{ERROR_DS_OUT_OF_SCOPE,"ERROR_DS_OUT_OF_SCOPE"},
{ERROR_DS_OBJECT_BEING_REMOVED,"ERROR_DS_OBJECT_BEING_REMOVED"},
{ERROR_DS_CANT_DELETE_DSA_OBJ,"ERROR_DS_CANT_DELETE_DSA_OBJ"},
{ERROR_DS_GENERIC_ERROR,"ERROR_DS_GENERIC_ERROR"},
{ERROR_DS_DSA_MUST_BE_INT_MASTER,"ERROR_DS_DSA_MUST_BE_INT_MASTER"},
{ERROR_DS_CLASS_NOT_DSA,"ERROR_DS_CLASS_NOT_DSA"},
{ERROR_DS_INSUFF_ACCESS_RIGHTS,"ERROR_DS_INSUFF_ACCESS_RIGHTS"},
{ERROR_DS_ILLEGAL_SUPERIOR,"ERROR_DS_ILLEGAL_SUPERIOR"},
{ERROR_DS_ATTRIBUTE_OWNED_BY_SAM,"ERROR_DS_ATTRIBUTE_OWNED_BY_SAM"},
{ERROR_DS_NAME_TOO_MANY_PARTS,"ERROR_DS_NAME_TOO_MANY_PARTS"},
{ERROR_DS_NAME_TOO_LONG,"ERROR_DS_NAME_TOO_LONG"},
{ERROR_DS_NAME_VALUE_TOO_LONG,"ERROR_DS_NAME_VALUE_TOO_LONG"},
{ERROR_DS_NAME_UNPARSEABLE,"ERROR_DS_NAME_UNPARSEABLE"},
{ERROR_DS_NAME_TYPE_UNKNOWN,"ERROR_DS_NAME_TYPE_UNKNOWN"},
{ERROR_DS_NOT_AN_OBJECT,"ERROR_DS_NOT_AN_OBJECT"},
{ERROR_DS_SEC_DESC_TOO_SHORT,"ERROR_DS_SEC_DESC_TOO_SHORT"},
{ERROR_DS_SEC_DESC_INVALID,"ERROR_DS_SEC_DESC_INVALID"},
{ERROR_DS_NO_DELETED_NAME,"ERROR_DS_NO_DELETED_NAME"},
{ERROR_DS_SUBREF_MUST_HAVE_PARENT,"ERROR_DS_SUBREF_MUST_HAVE_PARENT"},
{ERROR_DS_NCNAME_MUST_BE_NC,"ERROR_DS_NCNAME_MUST_BE_NC"},
{ERROR_DS_CANT_ADD_SYSTEM_ONLY,"ERROR_DS_CANT_ADD_SYSTEM_ONLY"},
{ERROR_DS_CLASS_MUST_BE_CONCRETE,"ERROR_DS_CLASS_MUST_BE_CONCRETE"},
{ERROR_DS_INVALID_DMD,"ERROR_DS_INVALID_DMD"},
{ERROR_DS_OBJ_GUID_EXISTS,"ERROR_DS_OBJ_GUID_EXISTS"},
{ERROR_DS_NOT_ON_BACKLINK,"ERROR_DS_NOT_ON_BACKLINK"},
{ERROR_DS_NO_CROSSREF_FOR_NC,"ERROR_DS_NO_CROSSREF_FOR_NC"},
{ERROR_DS_SHUTTING_DOWN,"ERROR_DS_SHUTTING_DOWN"},
{ERROR_DS_UNKNOWN_OPERATION,"ERROR_DS_UNKNOWN_OPERATION"},
{ERROR_DS_INVALID_ROLE_OWNER,"ERROR_DS_INVALID_ROLE_OWNER"},
{ERROR_DS_COULDNT_CONTACT_FSMO,"ERROR_DS_COULDNT_CONTACT_FSMO"},
{ERROR_DS_CROSS_NC_DN_RENAME,"ERROR_DS_CROSS_NC_DN_RENAME"},
{ERROR_DS_CANT_MOD_SYSTEM_ONLY,"ERROR_DS_CANT_MOD_SYSTEM_ONLY"},
{ERROR_DS_REPLICATOR_ONLY,"ERROR_DS_REPLICATOR_ONLY"},
{ERROR_DS_OBJ_CLASS_NOT_DEFINED,"ERROR_DS_OBJ_CLASS_NOT_DEFINED"},
{ERROR_DS_OBJ_CLASS_NOT_SUBCLASS,"ERROR_DS_OBJ_CLASS_NOT_SUBCLASS"},
{ERROR_DS_NAME_REFERENCE_INVALID,"ERROR_DS_NAME_REFERENCE_INVALID"},
{ERROR_DS_CROSS_REF_EXISTS,"ERROR_DS_CROSS_REF_EXISTS"},
{ERROR_DS_CANT_DEL_MASTER_CROSSREF,"ERROR_DS_CANT_DEL_MASTER_CROSSREF"},
{ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD,"ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD"},
{ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX,"ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX"},
{ERROR_DS_DUP_RDN,"ERROR_DS_DUP_RDN"},
{ERROR_DS_DUP_OID,"ERROR_DS_DUP_OID"},
{ERROR_DS_DUP_MAPI_ID,"ERROR_DS_DUP_MAPI_ID"},
{ERROR_DS_DUP_SCHEMA_ID_GUID,"ERROR_DS_DUP_SCHEMA_ID_GUID"},
{ERROR_DS_DUP_LDAP_DISPLAY_NAME,"ERROR_DS_DUP_LDAP_DISPLAY_NAME"},
{ERROR_DS_SEMANTIC_ATT_TEST,"ERROR_DS_SEMANTIC_ATT_TEST"},
{ERROR_DS_SYNTAX_MISMATCH,"ERROR_DS_SYNTAX_MISMATCH"},
{ERROR_DS_EXISTS_IN_MUST_HAVE,"ERROR_DS_EXISTS_IN_MUST_HAVE"},
{ERROR_DS_EXISTS_IN_MAY_HAVE,"ERROR_DS_EXISTS_IN_MAY_HAVE"},
{ERROR_DS_NONEXISTENT_MAY_HAVE,"ERROR_DS_NONEXISTENT_MAY_HAVE"},
{ERROR_DS_NONEXISTENT_MUST_HAVE,"ERROR_DS_NONEXISTENT_MUST_HAVE"},
{ERROR_DS_AUX_CLS_TEST_FAIL,"ERROR_DS_AUX_CLS_TEST_FAIL"},
{ERROR_DS_NONEXISTENT_POSS_SUP,"ERROR_DS_NONEXISTENT_POSS_SUP"},
{ERROR_DS_SUB_CLS_TEST_FAIL,"ERROR_DS_SUB_CLS_TEST_FAIL"},
{ERROR_DS_BAD_RDN_ATT_ID_SYNTAX,"ERROR_DS_BAD_RDN_ATT_ID_SYNTAX"},
{ERROR_DS_EXISTS_IN_AUX_CLS,"ERROR_DS_EXISTS_IN_AUX_CLS"},
{ERROR_DS_EXISTS_IN_SUB_CLS,"ERROR_DS_EXISTS_IN_SUB_CLS"},
{ERROR_DS_EXISTS_IN_POSS_SUP,"ERROR_DS_EXISTS_IN_POSS_SUP"},
{ERROR_DS_RECALCSCHEMA_FAILED,"ERROR_DS_RECALCSCHEMA_FAILED"},
{ERROR_DS_TREE_DELETE_NOT_FINISHED,"ERROR_DS_TREE_DELETE_NOT_FINISHED"},
{ERROR_DS_CANT_DELETE,"ERROR_DS_CANT_DELETE"},
{ERROR_DS_ATT_SCHEMA_REQ_ID,"ERROR_DS_ATT_SCHEMA_REQ_ID"},
{ERROR_DS_BAD_ATT_SCHEMA_SYNTAX,"ERROR_DS_BAD_ATT_SCHEMA_SYNTAX"},
{ERROR_DS_CANT_CACHE_ATT,"ERROR_DS_CANT_CACHE_ATT"},
{ERROR_DS_CANT_CACHE_CLASS,"ERROR_DS_CANT_CACHE_CLASS"},
{ERROR_DS_CANT_REMOVE_ATT_CACHE,"ERROR_DS_CANT_REMOVE_ATT_CACHE"},
{ERROR_DS_CANT_REMOVE_CLASS_CACHE,"ERROR_DS_CANT_REMOVE_CLASS_CACHE"},
{ERROR_DS_CANT_RETRIEVE_DN,"ERROR_DS_CANT_RETRIEVE_DN"},
{ERROR_DS_MISSING_SUPREF,"ERROR_DS_MISSING_SUPREF"},
{ERROR_DS_CANT_RETRIEVE_INSTANCE,"ERROR_DS_CANT_RETRIEVE_INSTANCE"},
{ERROR_DS_CODE_INCONSISTENCY,"ERROR_DS_CODE_INCONSISTENCY"},
{ERROR_DS_DATABASE_ERROR,"ERROR_DS_DATABASE_ERROR"},
{ERROR_DS_GOVERNSID_MISSING,"ERROR_DS_GOVERNSID_MISSING"},
{ERROR_DS_MISSING_EXPECTED_ATT,"ERROR_DS_MISSING_EXPECTED_ATT"},
{ERROR_DS_NCNAME_MISSING_CR_REF,"ERROR_DS_NCNAME_MISSING_CR_REF"},
{ERROR_DS_SECURITY_CHECKING_ERROR,"ERROR_DS_SECURITY_CHECKING_ERROR"},
{ERROR_DS_SCHEMA_NOT_LOADED,"ERROR_DS_SCHEMA_NOT_LOADED"},
{ERROR_DS_SCHEMA_ALLOC_FAILED,"ERROR_DS_SCHEMA_ALLOC_FAILED"},
{ERROR_DS_ATT_SCHEMA_REQ_SYNTAX,"ERROR_DS_ATT_SCHEMA_REQ_SYNTAX"},
{ERROR_DS_GCVERIFY_ERROR,"ERROR_DS_GCVERIFY_ERROR"},
{ERROR_DS_DRA_SCHEMA_MISMATCH,"ERROR_DS_DRA_SCHEMA_MISMATCH"},
{ERROR_DS_CANT_FIND_DSA_OBJ,"ERROR_DS_CANT_FIND_DSA_OBJ"},
{ERROR_DS_CANT_FIND_EXPECTED_NC,"ERROR_DS_CANT_FIND_EXPECTED_NC"},
{ERROR_DS_CANT_FIND_NC_IN_CACHE,"ERROR_DS_CANT_FIND_NC_IN_CACHE"},
{ERROR_DS_CANT_RETRIEVE_CHILD,"ERROR_DS_CANT_RETRIEVE_CHILD"},
{ERROR_DS_SECURITY_ILLEGAL_MODIFY,"ERROR_DS_SECURITY_ILLEGAL_MODIFY"},
{ERROR_DS_CANT_REPLACE_HIDDEN_REC,"ERROR_DS_CANT_REPLACE_HIDDEN_REC"},
{ERROR_DS_BAD_HIERARCHY_FILE,"ERROR_DS_BAD_HIERARCHY_FILE"},
{ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED,"ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED"},
{ERROR_DS_CONFIG_PARAM_MISSING,"ERROR_DS_CONFIG_PARAM_MISSING"},
{ERROR_DS_COUNTING_AB_INDICES_FAILED,"ERROR_DS_COUNTING_AB_INDICES_FAILED"},
{ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED,"ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED"},
{ERROR_DS_INTERNAL_FAILURE,"ERROR_DS_INTERNAL_FAILURE"},
{ERROR_DS_UNKNOWN_ERROR,"ERROR_DS_UNKNOWN_ERROR"},
{ERROR_DS_ROOT_REQUIRES_CLASS_TOP,"ERROR_DS_ROOT_REQUIRES_CLASS_TOP"},
{ERROR_DS_REFUSING_FSMO_ROLES,"ERROR_DS_REFUSING_FSMO_ROLES"},
{ERROR_DS_MISSING_FSMO_SETTINGS,"ERROR_DS_MISSING_FSMO_SETTINGS"},
{ERROR_DS_UNABLE_TO_SURRENDER_ROLES,"ERROR_DS_UNABLE_TO_SURRENDER_ROLES"},
{ERROR_DS_DRA_GENERIC,"ERROR_DS_DRA_GENERIC"},
{ERROR_DS_DRA_INVALID_PARAMETER,"ERROR_DS_DRA_INVALID_PARAMETER"},
{ERROR_DS_DRA_BUSY,"ERROR_DS_DRA_BUSY"},
{ERROR_DS_DRA_BAD_DN,"ERROR_DS_DRA_BAD_DN"},
{ERROR_DS_DRA_BAD_NC,"ERROR_DS_DRA_BAD_NC"},
{ERROR_DS_DRA_DN_EXISTS,"ERROR_DS_DRA_DN_EXISTS"},
{ERROR_DS_DRA_INTERNAL_ERROR,"ERROR_DS_DRA_INTERNAL_ERROR"},
{ERROR_DS_DRA_INCONSISTENT_DIT,"ERROR_DS_DRA_INCONSISTENT_DIT"},
{ERROR_DS_DRA_CONNECTION_FAILED,"ERROR_DS_DRA_CONNECTION_FAILED"},
{ERROR_DS_DRA_BAD_INSTANCE_TYPE,"ERROR_DS_DRA_BAD_INSTANCE_TYPE"},
{ERROR_DS_DRA_OUT_OF_MEM,"ERROR_DS_DRA_OUT_OF_MEM"},
{ERROR_DS_DRA_MAIL_PROBLEM,"ERROR_DS_DRA_MAIL_PROBLEM"},
{ERROR_DS_DRA_REF_ALREADY_EXISTS,"ERROR_DS_DRA_REF_ALREADY_EXISTS"},
{ERROR_DS_DRA_REF_NOT_FOUND,"ERROR_DS_DRA_REF_NOT_FOUND"},
{ERROR_DS_DRA_OBJ_IS_REP_SOURCE,"ERROR_DS_DRA_OBJ_IS_REP_SOURCE"},
{ERROR_DS_DRA_DB_ERROR,"ERROR_DS_DRA_DB_ERROR"},
{ERROR_DS_DRA_NO_REPLICA,"ERROR_DS_DRA_NO_REPLICA"},
{ERROR_DS_DRA_ACCESS_DENIED,"ERROR_DS_DRA_ACCESS_DENIED"},
{ERROR_DS_DRA_NOT_SUPPORTED,"ERROR_DS_DRA_NOT_SUPPORTED"},
{ERROR_DS_DRA_RPC_CANCELLED,"ERROR_DS_DRA_RPC_CANCELLED"},
{ERROR_DS_DRA_SOURCE_DISABLED,"ERROR_DS_DRA_SOURCE_DISABLED"},
{ERROR_DS_DRA_SINK_DISABLED,"ERROR_DS_DRA_SINK_DISABLED"},
{ERROR_DS_DRA_NAME_COLLISION,"ERROR_DS_DRA_NAME_COLLISION"},
{ERROR_DS_DRA_SOURCE_REINSTALLED,"ERROR_DS_DRA_SOURCE_REINSTALLED"},
{ERROR_DS_DRA_MISSING_PARENT,"ERROR_DS_DRA_MISSING_PARENT"},
{ERROR_DS_DRA_PREEMPTED,"ERROR_DS_DRA_PREEMPTED"},
{ERROR_DS_DRA_ABANDON_SYNC,"ERROR_DS_DRA_ABANDON_SYNC"},
{ERROR_DS_DRA_SHUTDOWN,"ERROR_DS_DRA_SHUTDOWN"},
{ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET,"ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET"},
{ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA,"ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA"},
{ERROR_DS_DRA_EXTN_CONNECTION_FAILED,"ERROR_DS_DRA_EXTN_CONNECTION_FAILED"},
{ERROR_DS_INSTALL_SCHEMA_MISMATCH,"ERROR_DS_INSTALL_SCHEMA_MISMATCH"},
{ERROR_DS_DUP_LINK_ID,"ERROR_DS_DUP_LINK_ID"},
{ERROR_DS_NAME_ERROR_RESOLVING,"ERROR_DS_NAME_ERROR_RESOLVING"},
{ERROR_DS_NAME_ERROR_NOT_FOUND,"ERROR_DS_NAME_ERROR_NOT_FOUND"},
{ERROR_DS_NAME_ERROR_NOT_UNIQUE,"ERROR_DS_NAME_ERROR_NOT_UNIQUE"},
{ERROR_DS_NAME_ERROR_NO_MAPPING,"ERROR_DS_NAME_ERROR_NO_MAPPING"},
{ERROR_DS_NAME_ERROR_DOMAIN_ONLY,"ERROR_DS_NAME_ERROR_DOMAIN_ONLY"},
{ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING,"ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING"},
{ERROR_DS_CONSTRUCTED_ATT_MOD,"ERROR_DS_CONSTRUCTED_ATT_MOD"},
{ERROR_DS_WRONG_OM_OBJ_CLASS,"ERROR_DS_WRONG_OM_OBJ_CLASS"},
{ERROR_DS_DRA_REPL_PENDING,"ERROR_DS_DRA_REPL_PENDING"},
{ERROR_DS_DS_REQUIRED,"ERROR_DS_DS_REQUIRED"},
{ERROR_DS_INVALID_LDAP_DISPLAY_NAME,"ERROR_DS_INVALID_LDAP_DISPLAY_NAME"},
{ERROR_DS_NON_BASE_SEARCH,"ERROR_DS_NON_BASE_SEARCH"},
{ERROR_DS_CANT_RETRIEVE_ATTS,"ERROR_DS_CANT_RETRIEVE_ATTS"},
{ERROR_DS_BACKLINK_WITHOUT_LINK,"ERROR_DS_BACKLINK_WITHOUT_LINK"},
{ERROR_DS_EPOCH_MISMATCH,"ERROR_DS_EPOCH_MISMATCH"},
{ERROR_DS_SRC_NAME_MISMATCH,"ERROR_DS_SRC_NAME_MISMATCH"},
{ERROR_DS_SRC_AND_DST_NC_IDENTICAL,"ERROR_DS_SRC_AND_DST_NC_IDENTICAL"},
{ERROR_DS_DST_NC_MISMATCH,"ERROR_DS_DST_NC_MISMATCH"},
{ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC,"ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC"},
{ERROR_DS_SRC_GUID_MISMATCH,"ERROR_DS_SRC_GUID_MISMATCH"},
{ERROR_DS_CANT_MOVE_DELETED_OBJECT,"ERROR_DS_CANT_MOVE_DELETED_OBJECT"},
{ERROR_DS_PDC_OPERATION_IN_PROGRESS,"ERROR_DS_PDC_OPERATION_IN_PROGRESS"},
{ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD,"ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD"},
{ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION,"ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION"},
{ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS,"ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS"},
{ERROR_DS_NC_MUST_HAVE_NC_PARENT,"ERROR_DS_NC_MUST_HAVE_NC_PARENT"},
{ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE,"ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE"},
{ERROR_DS_DST_DOMAIN_NOT_NATIVE,"ERROR_DS_DST_DOMAIN_NOT_NATIVE"},
{ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER,"ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER"},
{ERROR_DS_CANT_MOVE_ACCOUNT_GROUP,"ERROR_DS_CANT_MOVE_ACCOUNT_GROUP"},
{ERROR_DS_CANT_MOVE_RESOURCE_GROUP,"ERROR_DS_CANT_MOVE_RESOURCE_GROUP"},
{ERROR_DS_INVALID_SEARCH_FLAG,"ERROR_DS_INVALID_SEARCH_FLAG"},
{ERROR_DS_NO_TREE_DELETE_ABOVE_NC,"ERROR_DS_NO_TREE_DELETE_ABOVE_NC"},
{ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE,"ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE"},
{ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE,"ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE"},
{ERROR_DS_SAM_INIT_FAILURE,"ERROR_DS_SAM_INIT_FAILURE"},
{ERROR_DS_SENSITIVE_GROUP_VIOLATION,"ERROR_DS_SENSITIVE_GROUP_VIOLATION"},
{ERROR_DS_CANT_MOD_PRIMARYGROUPID,"ERROR_DS_CANT_MOD_PRIMARYGROUPID"},
{ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD,"ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD"},
{ERROR_DS_NONSAFE_SCHEMA_CHANGE,"ERROR_DS_NONSAFE_SCHEMA_CHANGE"},
{ERROR_DS_SCHEMA_UPDATE_DISALLOWED,"ERROR_DS_SCHEMA_UPDATE_DISALLOWED"},
{ERROR_DS_CANT_CREATE_UNDER_SCHEMA,"ERROR_DS_CANT_CREATE_UNDER_SCHEMA"},
{ERROR_DS_INSTALL_NO_SRC_SCH_VERSION,"ERROR_DS_INSTALL_NO_SRC_SCH_VERSION"},
{ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE,"ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE"},
{ERROR_DS_INVALID_GROUP_TYPE,"ERROR_DS_INVALID_GROUP_TYPE"},
{ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN,"ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN"},
{ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN,"ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN"},
{ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER,"ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER"},
{ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER,"ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER"},
{ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER,"ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER"},
{ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER,"ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER"},
{ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER,"ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER"},
{ERROR_DS_HAVE_PRIMARY_MEMBERS,"ERROR_DS_HAVE_PRIMARY_MEMBERS"},
{ERROR_DS_STRING_SD_CONVERSION_FAILED,"ERROR_DS_STRING_SD_CONVERSION_FAILED"},
{ERROR_DS_NAMING_MASTER_GC,"ERROR_DS_NAMING_MASTER_GC"},
{ERROR_DS_DNS_LOOKUP_FAILURE,"ERROR_DS_DNS_LOOKUP_FAILURE"},
{ERROR_DS_COULDNT_UPDATE_SPNS,"ERROR_DS_COULDNT_UPDATE_SPNS"},
{ERROR_DS_CANT_RETRIEVE_SD,"ERROR_DS_CANT_RETRIEVE_SD"},
{ERROR_DS_KEY_NOT_UNIQUE,"ERROR_DS_KEY_NOT_UNIQUE"},
{ERROR_DS_WRONG_LINKED_ATT_SYNTAX,"ERROR_DS_WRONG_LINKED_ATT_SYNTAX"},
{ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD,"ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD"},
{ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY,"ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY"},
{ERROR_DS_CANT_START,"ERROR_DS_CANT_START"},
{ERROR_DS_INIT_FAILURE,"ERROR_DS_INIT_FAILURE"},
{ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION,"ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION"},
{ERROR_DS_SOURCE_DOMAIN_IN_FOREST,"ERROR_DS_SOURCE_DOMAIN_IN_FOREST"},
{ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST,"ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST"},
{ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED,"ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED"},
{ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN,"ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN"},
{ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER,"ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER"},
{ERROR_DS_SRC_SID_EXISTS_IN_FOREST,"ERROR_DS_SRC_SID_EXISTS_IN_FOREST"},
{ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH,"ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH"},
{ERROR_SAM_INIT_FAILURE,"ERROR_SAM_INIT_FAILURE"},
{ERROR_DS_DRA_SCHEMA_INFO_SHIP,"ERROR_DS_DRA_SCHEMA_INFO_SHIP"},
{ERROR_DS_DRA_SCHEMA_CONFLICT,"ERROR_DS_DRA_SCHEMA_CONFLICT"},
{ERROR_DS_DRA_EARLIER_SCHEMA_CONFLICT,"ERROR_DS_DRA_EARLIER_SCHEMA_CONFLICT"},
{ERROR_DS_DRA_OBJ_NC_MISMATCH,"ERROR_DS_DRA_OBJ_NC_MISMATCH"},
{ERROR_DS_NC_STILL_HAS_DSAS,"ERROR_DS_NC_STILL_HAS_DSAS"},
{ERROR_DS_GC_REQUIRED,"ERROR_DS_GC_REQUIRED"},
{ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY,"ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY"},
{ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS,"ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS"},
{ERROR_DS_CANT_ADD_TO_GC,"ERROR_DS_CANT_ADD_TO_GC"},
{ERROR_DS_NO_CHECKPOINT_WITH_PDC,"ERROR_DS_NO_CHECKPOINT_WITH_PDC"},
{ERROR_DS_SOURCE_AUDITING_NOT_ENABLED,"ERROR_DS_SOURCE_AUDITING_NOT_ENABLED"},
{ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC,"ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC"},
{ERROR_DS_INVALID_NAME_FOR_SPN,"ERROR_DS_INVALID_NAME_FOR_SPN"},
{ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS,"ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS"},
{ERROR_DS_UNICODEPWD_NOT_IN_QUOTES,"ERROR_DS_UNICODEPWD_NOT_IN_QUOTES"},
{ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED,"ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED"},
{ERROR_DS_MUST_BE_RUN_ON_DST_DC,"ERROR_DS_MUST_BE_RUN_ON_DST_DC"},
{ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER,"ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER"},
{ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ,"ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ"},
{ERROR_DS_INIT_FAILURE_CONSOLE,"ERROR_DS_INIT_FAILURE_CONSOLE"},
{ERROR_DS_SAM_INIT_FAILURE_CONSOLE,"ERROR_DS_SAM_INIT_FAILURE_CONSOLE"},
{ERROR_DS_FOREST_VERSION_TOO_HIGH,"ERROR_DS_FOREST_VERSION_TOO_HIGH"},
{ERROR_DS_DOMAIN_VERSION_TOO_HIGH,"ERROR_DS_DOMAIN_VERSION_TOO_HIGH"},
{ERROR_DS_FOREST_VERSION_TOO_LOW,"ERROR_DS_FOREST_VERSION_TOO_LOW"},
{ERROR_DS_DOMAIN_VERSION_TOO_LOW,"ERROR_DS_DOMAIN_VERSION_TOO_LOW"},
{ERROR_DS_INCOMPATIBLE_VERSION,"ERROR_DS_INCOMPATIBLE_VERSION"},
{ERROR_DS_LOW_DSA_VERSION,"ERROR_DS_LOW_DSA_VERSION"},
{ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN,"ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN"},
{ERROR_DS_NOT_SUPPORTED_SORT_ORDER,"ERROR_DS_NOT_SUPPORTED_SORT_ORDER"},
{ERROR_DS_NAME_NOT_UNIQUE,"ERROR_DS_NAME_NOT_UNIQUE"},
{ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4,"ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4"},
{ERROR_DS_OUT_OF_VERSION_STORE,"ERROR_DS_OUT_OF_VERSION_STORE"},
{ERROR_DS_INCOMPATIBLE_CONTROLS_USED,"ERROR_DS_INCOMPATIBLE_CONTROLS_USED"},
{ERROR_DS_NO_REF_DOMAIN,"ERROR_DS_NO_REF_DOMAIN"},
{ERROR_DS_RESERVED_LINK_ID,"ERROR_DS_RESERVED_LINK_ID"},
{ERROR_DS_LINK_ID_NOT_AVAILABLE,"ERROR_DS_LINK_ID_NOT_AVAILABLE"},
{ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER,"ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER"},
{ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE,"ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE"},
{ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC,"ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC"},
{ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG,"ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG"},
{ERROR_DS_MODIFYDN_WRONG_GRANDPARENT,"ERROR_DS_MODIFYDN_WRONG_GRANDPARENT"},
{ERROR_DS_NAME_ERROR_TRUST_REFERRAL,"ERROR_DS_NAME_ERROR_TRUST_REFERRAL"},
{ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER,"ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER"},
{ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD,"ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD"},
{ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2,"ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2"},
{ERROR_DS_THREAD_LIMIT_EXCEEDED,"ERROR_DS_THREAD_LIMIT_EXCEEDED"},
{ERROR_DS_NOT_CLOSEST,"ERROR_DS_NOT_CLOSEST"},
{ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF,"ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF"},
{ERROR_DS_SINGLE_USER_MODE_FAILED,"ERROR_DS_SINGLE_USER_MODE_FAILED"},
{ERROR_DS_NTDSCRIPT_SYNTAX_ERROR,"ERROR_DS_NTDSCRIPT_SYNTAX_ERROR"},
{ERROR_DS_NTDSCRIPT_PROCESS_ERROR,"ERROR_DS_NTDSCRIPT_PROCESS_ERROR"},
{ERROR_DS_DIFFERENT_REPL_EPOCHS,"ERROR_DS_DIFFERENT_REPL_EPOCHS"},
{ERROR_DS_DRS_EXTENSIONS_CHANGED,"ERROR_DS_DRS_EXTENSIONS_CHANGED"},
{ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR,"ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR"},
{ERROR_DS_NO_MSDS_INTID,"ERROR_DS_NO_MSDS_INTID"},
{ERROR_DS_DUP_MSDS_INTID,"ERROR_DS_DUP_MSDS_INTID"},
{ERROR_DS_EXISTS_IN_RDNATTID,"ERROR_DS_EXISTS_IN_RDNATTID"},
{ERROR_DS_AUTHORIZATION_FAILED,"ERROR_DS_AUTHORIZATION_FAILED"},
{ERROR_DS_INVALID_SCRIPT,"ERROR_DS_INVALID_SCRIPT"},
{ERROR_DS_REMOTE_CROSSREF_OP_FAILED,"ERROR_DS_REMOTE_CROSSREF_OP_FAILED"},
{DNS_ERROR_RESPONSE_CODES_BASE,"DNS_ERROR_RESPONSE_CODES_BASE"},
{DNS_ERROR_RCODE_NO_ERROR,"DNS_ERROR_RCODE_NO_ERROR"},
{DNS_ERROR_RCODE_FORMAT_ERROR,"DNS_ERROR_RCODE_FORMAT_ERROR"},
{DNS_ERROR_RCODE_SERVER_FAILURE,"DNS_ERROR_RCODE_SERVER_FAILURE"},
{DNS_ERROR_RCODE_NAME_ERROR,"DNS_ERROR_RCODE_NAME_ERROR"},
{DNS_ERROR_RCODE_NOT_IMPLEMENTED,"DNS_ERROR_RCODE_NOT_IMPLEMENTED"},
{DNS_ERROR_RCODE_REFUSED,"DNS_ERROR_RCODE_REFUSED"},
{DNS_ERROR_RCODE_YXDOMAIN,"DNS_ERROR_RCODE_YXDOMAIN"},
{DNS_ERROR_RCODE_YXRRSET,"DNS_ERROR_RCODE_YXRRSET"},
{DNS_ERROR_RCODE_NXRRSET,"DNS_ERROR_RCODE_NXRRSET"},
{DNS_ERROR_RCODE_NOTAUTH,"DNS_ERROR_RCODE_NOTAUTH"},
{DNS_ERROR_RCODE_NOTZONE,"DNS_ERROR_RCODE_NOTZONE"},
{DNS_ERROR_RCODE_BADSIG,"DNS_ERROR_RCODE_BADSIG"},
{DNS_ERROR_RCODE_BADKEY,"DNS_ERROR_RCODE_BADKEY"},
{DNS_ERROR_RCODE_BADTIME,"DNS_ERROR_RCODE_BADTIME"},
{DNS_ERROR_RCODE_LAST,"DNS_ERROR_RCODE_LAST"},
{DNS_ERROR_PACKET_FMT_BASE,"DNS_ERROR_PACKET_FMT_BASE"},
{DNS_INFO_NO_RECORDS,"DNS_INFO_NO_RECORDS"},
{DNS_ERROR_BAD_PACKET,"DNS_ERROR_BAD_PACKET"},
{DNS_ERROR_NO_PACKET,"DNS_ERROR_NO_PACKET"},
{DNS_ERROR_RCODE,"DNS_ERROR_RCODE"},
{DNS_ERROR_UNSECURE_PACKET,"DNS_ERROR_UNSECURE_PACKET"},
{DNS_STATUS_PACKET_UNSECURE,"DNS_STATUS_PACKET_UNSECURE"},
{DNS_ERROR_NO_MEMORY,"DNS_ERROR_NO_MEMORY"},
{DNS_ERROR_INVALID_NAME,"DNS_ERROR_INVALID_NAME"},
{DNS_ERROR_INVALID_DATA,"DNS_ERROR_INVALID_DATA"},
{DNS_ERROR_GENERAL_API_BASE,"DNS_ERROR_GENERAL_API_BASE"},
{DNS_ERROR_INVALID_TYPE,"DNS_ERROR_INVALID_TYPE"},
{DNS_ERROR_INVALID_IP_ADDRESS,"DNS_ERROR_INVALID_IP_ADDRESS"},
{DNS_ERROR_INVALID_PROPERTY,"DNS_ERROR_INVALID_PROPERTY"},
{DNS_ERROR_TRY_AGAIN_LATER,"DNS_ERROR_TRY_AGAIN_LATER"},
{DNS_ERROR_NOT_UNIQUE,"DNS_ERROR_NOT_UNIQUE"},
{DNS_ERROR_NON_RFC_NAME,"DNS_ERROR_NON_RFC_NAME"},
{DNS_STATUS_FQDN,"DNS_STATUS_FQDN"},
{DNS_STATUS_DOTTED_NAME,"DNS_STATUS_DOTTED_NAME"},
{DNS_STATUS_SINGLE_PART_NAME,"DNS_STATUS_SINGLE_PART_NAME"},
{DNS_ERROR_INVALID_NAME_CHAR,"DNS_ERROR_INVALID_NAME_CHAR"},
{DNS_ERROR_NUMERIC_NAME,"DNS_ERROR_NUMERIC_NAME"},
{DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER,"DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER"},
{DNS_ERROR_ZONE_BASE,"DNS_ERROR_ZONE_BASE"},
{DNS_ERROR_ZONE_DOES_NOT_EXIST,"DNS_ERROR_ZONE_DOES_NOT_EXIST"},
{DNS_ERROR_NO_ZONE_INFO,"DNS_ERROR_NO_ZONE_INFO"},
{DNS_ERROR_INVALID_ZONE_OPERATION,"DNS_ERROR_INVALID_ZONE_OPERATION"},
{DNS_ERROR_ZONE_CONFIGURATION_ERROR,"DNS_ERROR_ZONE_CONFIGURATION_ERROR"},
{DNS_ERROR_ZONE_HAS_NO_SOA_RECORD,"DNS_ERROR_ZONE_HAS_NO_SOA_RECORD"},
{DNS_ERROR_ZONE_HAS_NO_NS_RECORDS,"DNS_ERROR_ZONE_HAS_NO_NS_RECORDS"},
{DNS_ERROR_ZONE_LOCKED,"DNS_ERROR_ZONE_LOCKED"},
{DNS_ERROR_ZONE_CREATION_FAILED,"DNS_ERROR_ZONE_CREATION_FAILED"},
{DNS_ERROR_ZONE_ALREADY_EXISTS,"DNS_ERROR_ZONE_ALREADY_EXISTS"},
{DNS_ERROR_AUTOZONE_ALREADY_EXISTS,"DNS_ERROR_AUTOZONE_ALREADY_EXISTS"},
{DNS_ERROR_INVALID_ZONE_TYPE,"DNS_ERROR_INVALID_ZONE_TYPE"},
{DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP,"DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP"},
{DNS_ERROR_ZONE_NOT_SECONDARY,"DNS_ERROR_ZONE_NOT_SECONDARY"},
{DNS_ERROR_NEED_SECONDARY_ADDRESSES,"DNS_ERROR_NEED_SECONDARY_ADDRESSES"},
{DNS_ERROR_WINS_INIT_FAILED,"DNS_ERROR_WINS_INIT_FAILED"},
{DNS_ERROR_NEED_WINS_SERVERS,"DNS_ERROR_NEED_WINS_SERVERS"},
{DNS_ERROR_NBSTAT_INIT_FAILED,"DNS_ERROR_NBSTAT_INIT_FAILED"},
{DNS_ERROR_SOA_DELETE_INVALID,"DNS_ERROR_SOA_DELETE_INVALID"},
{DNS_ERROR_FORWARDER_ALREADY_EXISTS,"DNS_ERROR_FORWARDER_ALREADY_EXISTS"},
{DNS_ERROR_ZONE_REQUIRES_MASTER_IP,"DNS_ERROR_ZONE_REQUIRES_MASTER_IP"},
{DNS_ERROR_ZONE_IS_SHUTDOWN,"DNS_ERROR_ZONE_IS_SHUTDOWN"},
{DNS_ERROR_DATAFILE_BASE,"DNS_ERROR_DATAFILE_BASE"},
{DNS_ERROR_PRIMARY_REQUIRES_DATAFILE,"DNS_ERROR_PRIMARY_REQUIRES_DATAFILE"},
{DNS_ERROR_INVALID_DATAFILE_NAME,"DNS_ERROR_INVALID_DATAFILE_NAME"},
{DNS_ERROR_DATAFILE_OPEN_FAILURE,"DNS_ERROR_DATAFILE_OPEN_FAILURE"},
{DNS_ERROR_FILE_WRITEBACK_FAILED,"DNS_ERROR_FILE_WRITEBACK_FAILED"},
{DNS_ERROR_DATAFILE_PARSING,"DNS_ERROR_DATAFILE_PARSING"},
{DNS_ERROR_DATABASE_BASE,"DNS_ERROR_DATABASE_BASE"},
{DNS_ERROR_RECORD_DOES_NOT_EXIST,"DNS_ERROR_RECORD_DOES_NOT_EXIST"},
{DNS_ERROR_RECORD_FORMAT,"DNS_ERROR_RECORD_FORMAT"},
{DNS_ERROR_NODE_CREATION_FAILED,"DNS_ERROR_NODE_CREATION_FAILED"},
{DNS_ERROR_UNKNOWN_RECORD_TYPE,"DNS_ERROR_UNKNOWN_RECORD_TYPE"},
{DNS_ERROR_RECORD_TIMED_OUT,"DNS_ERROR_RECORD_TIMED_OUT"},
{DNS_ERROR_NAME_NOT_IN_ZONE,"DNS_ERROR_NAME_NOT_IN_ZONE"},
{DNS_ERROR_CNAME_LOOP,"DNS_ERROR_CNAME_LOOP"},
{DNS_ERROR_NODE_IS_CNAME,"DNS_ERROR_NODE_IS_CNAME"},
{DNS_ERROR_CNAME_COLLISION,"DNS_ERROR_CNAME_COLLISION"},
{DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT,"DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT"},
{DNS_ERROR_RECORD_ALREADY_EXISTS,"DNS_ERROR_RECORD_ALREADY_EXISTS"},
{DNS_ERROR_SECONDARY_DATA,"DNS_ERROR_SECONDARY_DATA"},
{DNS_ERROR_NO_CREATE_CACHE_DATA,"DNS_ERROR_NO_CREATE_CACHE_DATA"},
{DNS_ERROR_NAME_DOES_NOT_EXIST,"DNS_ERROR_NAME_DOES_NOT_EXIST"},
{DNS_WARNING_PTR_CREATE_FAILED,"DNS_WARNING_PTR_CREATE_FAILED"},
{DNS_WARNING_DOMAIN_UNDELETED,"DNS_WARNING_DOMAIN_UNDELETED"},
{DNS_ERROR_DS_UNAVAILABLE,"DNS_ERROR_DS_UNAVAILABLE"},
{DNS_ERROR_DS_ZONE_ALREADY_EXISTS,"DNS_ERROR_DS_ZONE_ALREADY_EXISTS"},
{DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE,"DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE"},
{DNS_ERROR_OPERATION_BASE,"DNS_ERROR_OPERATION_BASE"},
{DNS_INFO_AXFR_COMPLETE,"DNS_INFO_AXFR_COMPLETE"},
{DNS_ERROR_AXFR,"DNS_ERROR_AXFR"},
{DNS_INFO_ADDED_LOCAL_WINS,"DNS_INFO_ADDED_LOCAL_WINS"},
{DNS_ERROR_SECURE_BASE,"DNS_ERROR_SECURE_BASE"},
{DNS_STATUS_CONTINUE_NEEDED,"DNS_STATUS_CONTINUE_NEEDED"},
{DNS_ERROR_SETUP_BASE,"DNS_ERROR_SETUP_BASE"},
{DNS_ERROR_NO_TCPIP,"DNS_ERROR_NO_TCPIP"},
{DNS_ERROR_NO_DNS_SERVERS,"DNS_ERROR_NO_DNS_SERVERS"},
{DNS_ERROR_DP_BASE,"DNS_ERROR_DP_BASE"},
{DNS_ERROR_DP_DOES_NOT_EXIST,"DNS_ERROR_DP_DOES_NOT_EXIST"},
{DNS_ERROR_DP_ALREADY_EXISTS,"DNS_ERROR_DP_ALREADY_EXISTS"},
{DNS_ERROR_DP_NOT_ENLISTED,"DNS_ERROR_DP_NOT_ENLISTED"},
{DNS_ERROR_DP_ALREADY_ENLISTED,"DNS_ERROR_DP_ALREADY_ENLISTED"},
{WSABASEERR,"WSABASEERR"},
{WSAEINTR,"WSAEINTR"},
{WSAEBADF,"WSAEBADF"},
{WSAEACCES,"WSAEACCES"},
{WSAEFAULT,"WSAEFAULT"},
{WSAEINVAL,"WSAEINVAL"},
{WSAEMFILE,"WSAEMFILE"},
{WSAEWOULDBLOCK,"WSAEWOULDBLOCK"},
{WSAEINPROGRESS,"WSAEINPROGRESS"},
{WSAEALREADY,"WSAEALREADY"},
{WSAENOTSOCK,"WSAENOTSOCK"},
{WSAEDESTADDRREQ,"WSAEDESTADDRREQ"},
{WSAEMSGSIZE,"WSAEMSGSIZE"},
{WSAEPROTOTYPE,"WSAEPROTOTYPE"},
{WSAENOPROTOOPT,"WSAENOPROTOOPT"},
{WSAEPROTONOSUPPORT,"WSAEPROTONOSUPPORT"},
{WSAESOCKTNOSUPPORT,"WSAESOCKTNOSUPPORT"},
{WSAEOPNOTSUPP,"WSAEOPNOTSUPP"},
{WSAEPFNOSUPPORT,"WSAEPFNOSUPPORT"},
{WSAEAFNOSUPPORT,"WSAEAFNOSUPPORT"},
{WSAEADDRINUSE,"WSAEADDRINUSE"},
{WSAEADDRNOTAVAIL,"WSAEADDRNOTAVAIL"},
{WSAENETDOWN,"WSAENETDOWN"},
{WSAENETUNREACH,"WSAENETUNREACH"},
{WSAENETRESET,"WSAENETRESET"},
{WSAECONNABORTED,"WSAECONNABORTED"},
{WSAECONNRESET,"WSAECONNRESET"},
{WSAENOBUFS,"WSAENOBUFS"},
{WSAEISCONN,"WSAEISCONN"},
{WSAENOTCONN,"WSAENOTCONN"},
{WSAESHUTDOWN,"WSAESHUTDOWN"},
{WSAETOOMANYREFS,"WSAETOOMANYREFS"},
{WSAETIMEDOUT,"WSAETIMEDOUT"},
{WSAECONNREFUSED,"WSAECONNREFUSED"},
{WSAELOOP,"WSAELOOP"},
{WSAENAMETOOLONG,"WSAENAMETOOLONG"},
{WSAEHOSTDOWN,"WSAEHOSTDOWN"},
{WSAEHOSTUNREACH,"WSAEHOSTUNREACH"},
{WSAENOTEMPTY,"WSAENOTEMPTY"},
{WSAEPROCLIM,"WSAEPROCLIM"},
{WSAEUSERS,"WSAEUSERS"},
{WSAEDQUOT,"WSAEDQUOT"},
{WSAESTALE,"WSAESTALE"},
{WSAEREMOTE,"WSAEREMOTE"},
{WSASYSNOTREADY,"WSASYSNOTREADY"},
{WSAVERNOTSUPPORTED,"WSAVERNOTSUPPORTED"},
{WSANOTINITIALISED,"WSANOTINITIALISED"},
{WSAEDISCON,"WSAEDISCON"},
{WSAENOMORE,"WSAENOMORE"},
{WSAECANCELLED,"WSAECANCELLED"},
{WSAEINVALIDPROCTABLE,"WSAEINVALIDPROCTABLE"},
{WSAEINVALIDPROVIDER,"WSAEINVALIDPROVIDER"},
{WSAEPROVIDERFAILEDINIT,"WSAEPROVIDERFAILEDINIT"},
{WSASYSCALLFAILURE,"WSASYSCALLFAILURE"},
{WSASERVICE_NOT_FOUND,"WSASERVICE_NOT_FOUND"},
{WSATYPE_NOT_FOUND,"WSATYPE_NOT_FOUND"},
{WSA_E_NO_MORE,"WSA_E_NO_MORE"},
{WSA_E_CANCELLED,"WSA_E_CANCELLED"},
{WSAEREFUSED,"WSAEREFUSED"},
{WSAHOST_NOT_FOUND,"WSAHOST_NOT_FOUND"},
{WSATRY_AGAIN,"WSATRY_AGAIN"},
{WSANO_RECOVERY,"WSANO_RECOVERY"},
{WSANO_DATA,"WSANO_DATA"},
{WSA_QOS_RECEIVERS,"WSA_QOS_RECEIVERS"},
{WSA_QOS_SENDERS,"WSA_QOS_SENDERS"},
{WSA_QOS_NO_SENDERS,"WSA_QOS_NO_SENDERS"},
{WSA_QOS_NO_RECEIVERS,"WSA_QOS_NO_RECEIVERS"},
{WSA_QOS_REQUEST_CONFIRMED,"WSA_QOS_REQUEST_CONFIRMED"},
{WSA_QOS_ADMISSION_FAILURE,"WSA_QOS_ADMISSION_FAILURE"},
{WSA_QOS_POLICY_FAILURE,"WSA_QOS_POLICY_FAILURE"},
{WSA_QOS_BAD_STYLE,"WSA_QOS_BAD_STYLE"},
{WSA_QOS_BAD_OBJECT,"WSA_QOS_BAD_OBJECT"},
{WSA_QOS_TRAFFIC_CTRL_ERROR,"WSA_QOS_TRAFFIC_CTRL_ERROR"},
{WSA_QOS_GENERIC_ERROR,"WSA_QOS_GENERIC_ERROR"},
{WSA_QOS_ESERVICETYPE,"WSA_QOS_ESERVICETYPE"},
{WSA_QOS_EFLOWSPEC,"WSA_QOS_EFLOWSPEC"},
{WSA_QOS_EPROVSPECBUF,"WSA_QOS_EPROVSPECBUF"},
{WSA_QOS_EFILTERSTYLE,"WSA_QOS_EFILTERSTYLE"},
{WSA_QOS_EFILTERTYPE,"WSA_QOS_EFILTERTYPE"},
{WSA_QOS_EFILTERCOUNT,"WSA_QOS_EFILTERCOUNT"},
{WSA_QOS_EOBJLENGTH,"WSA_QOS_EOBJLENGTH"},
{WSA_QOS_EFLOWCOUNT,"WSA_QOS_EFLOWCOUNT"},
{WSA_QOS_EUNKOWNPSOBJ,"WSA_QOS_EUNKOWNPSOBJ"},
{WSA_QOS_EPOLICYOBJ,"WSA_QOS_EPOLICYOBJ"},
{WSA_QOS_EFLOWDESC,"WSA_QOS_EFLOWDESC"},
{WSA_QOS_EPSFLOWSPEC,"WSA_QOS_EPSFLOWSPEC"},
{WSA_QOS_EPSFILTERSPEC,"WSA_QOS_EPSFILTERSPEC"},
{WSA_QOS_ESDMODEOBJ,"WSA_QOS_ESDMODEOBJ"},
{WSA_QOS_ESHAPERATEOBJ,"WSA_QOS_ESHAPERATEOBJ"},
{WSA_QOS_RESERVED_PETYPE,"WSA_QOS_RESERVED_PETYPE"},
{ERROR_SXS_SECTION_NOT_FOUND,"ERROR_SXS_SECTION_NOT_FOUND"},
{ERROR_SXS_CANT_GEN_ACTCTX,"ERROR_SXS_CANT_GEN_ACTCTX"},
{ERROR_SXS_INVALID_ACTCTXDATA_FORMAT,"ERROR_SXS_INVALID_ACTCTXDATA_FORMAT"},
{ERROR_SXS_ASSEMBLY_NOT_FOUND,"ERROR_SXS_ASSEMBLY_NOT_FOUND"},
{ERROR_SXS_MANIFEST_FORMAT_ERROR,"ERROR_SXS_MANIFEST_FORMAT_ERROR"},
{ERROR_SXS_MANIFEST_PARSE_ERROR,"ERROR_SXS_MANIFEST_PARSE_ERROR"},
{ERROR_SXS_ACTIVATION_CONTEXT_DISABLED,"ERROR_SXS_ACTIVATION_CONTEXT_DISABLED"},
{ERROR_SXS_KEY_NOT_FOUND,"ERROR_SXS_KEY_NOT_FOUND"},
{ERROR_SXS_VERSION_CONFLICT,"ERROR_SXS_VERSION_CONFLICT"},
{ERROR_SXS_WRONG_SECTION_TYPE,"ERROR_SXS_WRONG_SECTION_TYPE"},
{ERROR_SXS_THREAD_QUERIES_DISABLED,"ERROR_SXS_THREAD_QUERIES_DISABLED"},
{ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET,"ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET"},
{ERROR_SXS_UNKNOWN_ENCODING_GROUP,"ERROR_SXS_UNKNOWN_ENCODING_GROUP"},
{ERROR_SXS_UNKNOWN_ENCODING,"ERROR_SXS_UNKNOWN_ENCODING"},
{ERROR_SXS_INVALID_XML_NAMESPACE_URI,"ERROR_SXS_INVALID_XML_NAMESPACE_URI"},
{ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED,"ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED"},
{ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED,"ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED"},
{ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE,"ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE"},
{ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE,"ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE"},
{ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE,"ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE"},
{ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT,"ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT"},
{ERROR_SXS_DUPLICATE_DLL_NAME,"ERROR_SXS_DUPLICATE_DLL_NAME"},
{ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME,"ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME"},
{ERROR_SXS_DUPLICATE_CLSID,"ERROR_SXS_DUPLICATE_CLSID"},
{ERROR_SXS_DUPLICATE_IID,"ERROR_SXS_DUPLICATE_IID"},
{ERROR_SXS_DUPLICATE_TLBID,"ERROR_SXS_DUPLICATE_TLBID"},
{ERROR_SXS_DUPLICATE_PROGID,"ERROR_SXS_DUPLICATE_PROGID"},
{ERROR_SXS_DUPLICATE_ASSEMBLY_NAME,"ERROR_SXS_DUPLICATE_ASSEMBLY_NAME"},
{ERROR_SXS_FILE_HASH_MISMATCH,"ERROR_SXS_FILE_HASH_MISMATCH"},
{ERROR_SXS_POLICY_PARSE_ERROR,"ERROR_SXS_POLICY_PARSE_ERROR"},
{ERROR_SXS_XML_E_MISSINGQUOTE,"ERROR_SXS_XML_E_MISSINGQUOTE"},
{ERROR_SXS_XML_E_COMMENTSYNTAX,"ERROR_SXS_XML_E_COMMENTSYNTAX"},
{ERROR_SXS_XML_E_BADSTARTNAMECHAR,"ERROR_SXS_XML_E_BADSTARTNAMECHAR"},
{ERROR_SXS_XML_E_BADNAMECHAR,"ERROR_SXS_XML_E_BADNAMECHAR"},
{ERROR_SXS_XML_E_BADCHARINSTRING,"ERROR_SXS_XML_E_BADCHARINSTRING"},
{ERROR_SXS_XML_E_XMLDECLSYNTAX,"ERROR_SXS_XML_E_XMLDECLSYNTAX"},
{ERROR_SXS_XML_E_BADCHARDATA,"ERROR_SXS_XML_E_BADCHARDATA"},
{ERROR_SXS_XML_E_MISSINGWHITESPACE,"ERROR_SXS_XML_E_MISSINGWHITESPACE"},
{ERROR_SXS_XML_E_EXPECTINGTAGEND,"ERROR_SXS_XML_E_EXPECTINGTAGEND"},
{ERROR_SXS_XML_E_MISSINGSEMICOLON,"ERROR_SXS_XML_E_MISSINGSEMICOLON"},
{ERROR_SXS_XML_E_UNBALANCEDPAREN,"ERROR_SXS_XML_E_UNBALANCEDPAREN"},
{ERROR_SXS_XML_E_INTERNALERROR,"ERROR_SXS_XML_E_INTERNALERROR"},
{ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE,"ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE"},
{ERROR_SXS_XML_E_INCOMPLETE_ENCODING,"ERROR_SXS_XML_E_INCOMPLETE_ENCODING"},
{ERROR_SXS_XML_E_MISSING_PAREN,"ERROR_SXS_XML_E_MISSING_PAREN"},
{ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE,"ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE"},
{ERROR_SXS_XML_E_MULTIPLE_COLONS,"ERROR_SXS_XML_E_MULTIPLE_COLONS"},
{ERROR_SXS_XML_E_INVALID_DECIMAL,"ERROR_SXS_XML_E_INVALID_DECIMAL"},
{ERROR_SXS_XML_E_INVALID_HEXIDECIMAL,"ERROR_SXS_XML_E_INVALID_HEXIDECIMAL"},
{ERROR_SXS_XML_E_INVALID_UNICODE,"ERROR_SXS_XML_E_INVALID_UNICODE"},
{ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK,"ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK"},
{ERROR_SXS_XML_E_UNEXPECTEDENDTAG,"ERROR_SXS_XML_E_UNEXPECTEDENDTAG"},
{ERROR_SXS_XML_E_UNCLOSEDTAG,"ERROR_SXS_XML_E_UNCLOSEDTAG"},
{ERROR_SXS_XML_E_DUPLICATEATTRIBUTE,"ERROR_SXS_XML_E_DUPLICATEATTRIBUTE"},
{ERROR_SXS_XML_E_MULTIPLEROOTS,"ERROR_SXS_XML_E_MULTIPLEROOTS"},
{ERROR_SXS_XML_E_INVALIDATROOTLEVEL,"ERROR_SXS_XML_E_INVALIDATROOTLEVEL"},
{ERROR_SXS_XML_E_BADXMLDECL,"ERROR_SXS_XML_E_BADXMLDECL"},
{ERROR_SXS_XML_E_MISSINGROOT,"ERROR_SXS_XML_E_MISSINGROOT"},
{ERROR_SXS_XML_E_UNEXPECTEDEOF,"ERROR_SXS_XML_E_UNEXPECTEDEOF"},
{ERROR_SXS_XML_E_BADPEREFINSUBSET,"ERROR_SXS_XML_E_BADPEREFINSUBSET"},
{ERROR_SXS_XML_E_UNCLOSEDSTARTTAG,"ERROR_SXS_XML_E_UNCLOSEDSTARTTAG"},
{ERROR_SXS_XML_E_UNCLOSEDENDTAG,"ERROR_SXS_XML_E_UNCLOSEDENDTAG"},
{ERROR_SXS_XML_E_UNCLOSEDSTRING,"ERROR_SXS_XML_E_UNCLOSEDSTRING"},
{ERROR_SXS_XML_E_UNCLOSEDCOMMENT,"ERROR_SXS_XML_E_UNCLOSEDCOMMENT"},
{ERROR_SXS_XML_E_UNCLOSEDDECL,"ERROR_SXS_XML_E_UNCLOSEDDECL"},
{ERROR_SXS_XML_E_UNCLOSEDCDATA,"ERROR_SXS_XML_E_UNCLOSEDCDATA"},
{ERROR_SXS_XML_E_RESERVEDNAMESPACE,"ERROR_SXS_XML_E_RESERVEDNAMESPACE"},
{ERROR_SXS_XML_E_INVALIDENCODING,"ERROR_SXS_XML_E_INVALIDENCODING"},
{ERROR_SXS_XML_E_INVALIDSWITCH,"ERROR_SXS_XML_E_INVALIDSWITCH"},
{ERROR_SXS_XML_E_BADXMLCASE,"ERROR_SXS_XML_E_BADXMLCASE"},
{ERROR_SXS_XML_E_INVALID_STANDALONE,"ERROR_SXS_XML_E_INVALID_STANDALONE"},
{ERROR_SXS_XML_E_UNEXPECTED_STANDALONE,"ERROR_SXS_XML_E_UNEXPECTED_STANDALONE"},
{ERROR_SXS_XML_E_INVALID_VERSION,"ERROR_SXS_XML_E_INVALID_VERSION"},
{ERROR_SXS_XML_E_MISSINGEQUALS,"ERROR_SXS_XML_E_MISSINGEQUALS"},
{ERROR_SXS_PROTECTION_RECOVERY_FAILED,"ERROR_SXS_PROTECTION_RECOVERY_FAILED"},
{ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT,"ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT"},
{ERROR_SXS_PROTECTION_CATALOG_NOT_VALID,"ERROR_SXS_PROTECTION_CATALOG_NOT_VALID"},
{ERROR_SXS_UNTRANSLATABLE_HRESULT,"ERROR_SXS_UNTRANSLATABLE_HRESULT"},
{ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING,"ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING"},
{ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE,"ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE"},
{ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME,"ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME"},
{ERROR_IPSEC_QM_POLICY_EXISTS,"ERROR_IPSEC_QM_POLICY_EXISTS"},
{ERROR_IPSEC_QM_POLICY_NOT_FOUND,"ERROR_IPSEC_QM_POLICY_NOT_FOUND"},
{ERROR_IPSEC_QM_POLICY_IN_USE,"ERROR_IPSEC_QM_POLICY_IN_USE"},
{ERROR_IPSEC_MM_POLICY_EXISTS,"ERROR_IPSEC_MM_POLICY_EXISTS"},
{ERROR_IPSEC_MM_POLICY_NOT_FOUND,"ERROR_IPSEC_MM_POLICY_NOT_FOUND"},
{ERROR_IPSEC_MM_POLICY_IN_USE,"ERROR_IPSEC_MM_POLICY_IN_USE"},
{ERROR_IPSEC_MM_FILTER_EXISTS,"ERROR_IPSEC_MM_FILTER_EXISTS"},
{ERROR_IPSEC_MM_FILTER_NOT_FOUND,"ERROR_IPSEC_MM_FILTER_NOT_FOUND"},
{ERROR_IPSEC_TRANSPORT_FILTER_EXISTS,"ERROR_IPSEC_TRANSPORT_FILTER_EXISTS"},
{ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND,"ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND"},
{ERROR_IPSEC_MM_AUTH_EXISTS,"ERROR_IPSEC_MM_AUTH_EXISTS"},
{ERROR_IPSEC_MM_AUTH_NOT_FOUND,"ERROR_IPSEC_MM_AUTH_NOT_FOUND"},
{ERROR_IPSEC_MM_AUTH_IN_USE,"ERROR_IPSEC_MM_AUTH_IN_USE"},
{ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND,"ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND"},
{ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND,"ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND"},
{ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND,"ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND"},
{ERROR_IPSEC_TUNNEL_FILTER_EXISTS,"ERROR_IPSEC_TUNNEL_FILTER_EXISTS"},
{ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND,"ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND"},
{ERROR_IPSEC_MM_FILTER_PENDING_DELETION,"ERROR_IPSEC_MM_FILTER_PENDING_DELETION"},
{ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION,"ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION"},
{ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION,"ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION"},
{ERROR_IPSEC_MM_POLICY_PENDING_DELETION,"ERROR_IPSEC_MM_POLICY_PENDING_DELETION"},
{ERROR_IPSEC_MM_AUTH_PENDING_DELETION,"ERROR_IPSEC_MM_AUTH_PENDING_DELETION"},
{ERROR_IPSEC_QM_POLICY_PENDING_DELETION,"ERROR_IPSEC_QM_POLICY_PENDING_DELETION"},
{ERROR_IPSEC_IKE_NEG_STATUS_BEGIN,"ERROR_IPSEC_IKE_NEG_STATUS_BEGIN"},
{ERROR_IPSEC_IKE_AUTH_FAIL,"ERROR_IPSEC_IKE_AUTH_FAIL"},
{ERROR_IPSEC_IKE_ATTRIB_FAIL,"ERROR_IPSEC_IKE_ATTRIB_FAIL"},
{ERROR_IPSEC_IKE_NEGOTIATION_PENDING,"ERROR_IPSEC_IKE_NEGOTIATION_PENDING"},
{ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR,"ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR"},
{ERROR_IPSEC_IKE_TIMED_OUT,"ERROR_IPSEC_IKE_TIMED_OUT"},
{ERROR_IPSEC_IKE_NO_CERT,"ERROR_IPSEC_IKE_NO_CERT"},
{ERROR_IPSEC_IKE_SA_DELETED,"ERROR_IPSEC_IKE_SA_DELETED"},
{ERROR_IPSEC_IKE_SA_REAPED,"ERROR_IPSEC_IKE_SA_REAPED"},
{ERROR_IPSEC_IKE_MM_ACQUIRE_DROP,"ERROR_IPSEC_IKE_MM_ACQUIRE_DROP"},
{ERROR_IPSEC_IKE_QM_ACQUIRE_DROP,"ERROR_IPSEC_IKE_QM_ACQUIRE_DROP"},
{ERROR_IPSEC_IKE_QUEUE_DROP_MM,"ERROR_IPSEC_IKE_QUEUE_DROP_MM"},
{ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM,"ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM"},
{ERROR_IPSEC_IKE_DROP_NO_RESPONSE,"ERROR_IPSEC_IKE_DROP_NO_RESPONSE"},
{ERROR_IPSEC_IKE_MM_DELAY_DROP,"ERROR_IPSEC_IKE_MM_DELAY_DROP"},
{ERROR_IPSEC_IKE_QM_DELAY_DROP,"ERROR_IPSEC_IKE_QM_DELAY_DROP"},
{ERROR_IPSEC_IKE_ERROR,"ERROR_IPSEC_IKE_ERROR"},
{ERROR_IPSEC_IKE_CRL_FAILED,"ERROR_IPSEC_IKE_CRL_FAILED"},
{ERROR_IPSEC_IKE_INVALID_KEY_USAGE,"ERROR_IPSEC_IKE_INVALID_KEY_USAGE"},
{ERROR_IPSEC_IKE_INVALID_CERT_TYPE,"ERROR_IPSEC_IKE_INVALID_CERT_TYPE"},
{ERROR_IPSEC_IKE_NO_PRIVATE_KEY,"ERROR_IPSEC_IKE_NO_PRIVATE_KEY"},
{ERROR_IPSEC_IKE_DH_FAIL,"ERROR_IPSEC_IKE_DH_FAIL"},
{ERROR_IPSEC_IKE_INVALID_HEADER,"ERROR_IPSEC_IKE_INVALID_HEADER"},
{ERROR_IPSEC_IKE_NO_POLICY,"ERROR_IPSEC_IKE_NO_POLICY"},
{ERROR_IPSEC_IKE_INVALID_SIGNATURE,"ERROR_IPSEC_IKE_INVALID_SIGNATURE"},
{ERROR_IPSEC_IKE_KERBEROS_ERROR,"ERROR_IPSEC_IKE_KERBEROS_ERROR"},
{ERROR_IPSEC_IKE_NO_PUBLIC_KEY,"ERROR_IPSEC_IKE_NO_PUBLIC_KEY"},
{ERROR_IPSEC_IKE_PROCESS_ERR,"ERROR_IPSEC_IKE_PROCESS_ERR"},
{ERROR_IPSEC_IKE_PROCESS_ERR_SA,"ERROR_IPSEC_IKE_PROCESS_ERR_SA"},
{ERROR_IPSEC_IKE_PROCESS_ERR_PROP,"ERROR_IPSEC_IKE_PROCESS_ERR_PROP"},
{ERROR_IPSEC_IKE_PROCESS_ERR_TRANS,"ERROR_IPSEC_IKE_PROCESS_ERR_TRANS"},
{ERROR_IPSEC_IKE_PROCESS_ERR_KE,"ERROR_IPSEC_IKE_PROCESS_ERR_KE"},
{ERROR_IPSEC_IKE_PROCESS_ERR_ID,"ERROR_IPSEC_IKE_PROCESS_ERR_ID"},
{ERROR_IPSEC_IKE_PROCESS_ERR_CERT,"ERROR_IPSEC_IKE_PROCESS_ERR_CERT"},
{ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ,"ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ"},
{ERROR_IPSEC_IKE_PROCESS_ERR_HASH,"ERROR_IPSEC_IKE_PROCESS_ERR_HASH"},
{ERROR_IPSEC_IKE_PROCESS_ERR_SIG,"ERROR_IPSEC_IKE_PROCESS_ERR_SIG"},
{ERROR_IPSEC_IKE_PROCESS_ERR_NONCE,"ERROR_IPSEC_IKE_PROCESS_ERR_NONCE"},
{ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY,"ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY"},
{ERROR_IPSEC_IKE_PROCESS_ERR_DELETE,"ERROR_IPSEC_IKE_PROCESS_ERR_DELETE"},
{ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR,"ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR"},
{ERROR_IPSEC_IKE_INVALID_PAYLOAD,"ERROR_IPSEC_IKE_INVALID_PAYLOAD"},
{ERROR_IPSEC_IKE_LOAD_SOFT_SA,"ERROR_IPSEC_IKE_LOAD_SOFT_SA"},
{ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN,"ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN"},
{ERROR_IPSEC_IKE_INVALID_COOKIE,"ERROR_IPSEC_IKE_INVALID_COOKIE"},
{ERROR_IPSEC_IKE_NO_PEER_CERT,"ERROR_IPSEC_IKE_NO_PEER_CERT"},
{ERROR_IPSEC_IKE_PEER_CRL_FAILED,"ERROR_IPSEC_IKE_PEER_CRL_FAILED"},
{ERROR_IPSEC_IKE_POLICY_CHANGE,"ERROR_IPSEC_IKE_POLICY_CHANGE"},
{ERROR_IPSEC_IKE_NO_MM_POLICY,"ERROR_IPSEC_IKE_NO_MM_POLICY"},
{ERROR_IPSEC_IKE_NOTCBPRIV,"ERROR_IPSEC_IKE_NOTCBPRIV"},
{ERROR_IPSEC_IKE_SECLOADFAIL,"ERROR_IPSEC_IKE_SECLOADFAIL"},
{ERROR_IPSEC_IKE_FAILSSPINIT,"ERROR_IPSEC_IKE_FAILSSPINIT"},
{ERROR_IPSEC_IKE_FAILQUERYSSP,"ERROR_IPSEC_IKE_FAILQUERYSSP"},
{ERROR_IPSEC_IKE_SRVACQFAIL,"ERROR_IPSEC_IKE_SRVACQFAIL"},
{ERROR_IPSEC_IKE_SRVQUERYCRED,"ERROR_IPSEC_IKE_SRVQUERYCRED"},
{ERROR_IPSEC_IKE_GETSPIFAIL,"ERROR_IPSEC_IKE_GETSPIFAIL"},
{ERROR_IPSEC_IKE_INVALID_FILTER,"ERROR_IPSEC_IKE_INVALID_FILTER"},
{ERROR_IPSEC_IKE_OUT_OF_MEMORY,"ERROR_IPSEC_IKE_OUT_OF_MEMORY"},
{ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED,"ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED"},
{ERROR_IPSEC_IKE_INVALID_POLICY,"ERROR_IPSEC_IKE_INVALID_POLICY"},
{ERROR_IPSEC_IKE_UNKNOWN_DOI,"ERROR_IPSEC_IKE_UNKNOWN_DOI"},
{ERROR_IPSEC_IKE_INVALID_SITUATION,"ERROR_IPSEC_IKE_INVALID_SITUATION"},
{ERROR_IPSEC_IKE_DH_FAILURE,"ERROR_IPSEC_IKE_DH_FAILURE"},
{ERROR_IPSEC_IKE_INVALID_GROUP,"ERROR_IPSEC_IKE_INVALID_GROUP"},
{ERROR_IPSEC_IKE_ENCRYPT,"ERROR_IPSEC_IKE_ENCRYPT"},
{ERROR_IPSEC_IKE_DECRYPT,"ERROR_IPSEC_IKE_DECRYPT"},
{ERROR_IPSEC_IKE_POLICY_MATCH,"ERROR_IPSEC_IKE_POLICY_MATCH"},
{ERROR_IPSEC_IKE_UNSUPPORTED_ID,"ERROR_IPSEC_IKE_UNSUPPORTED_ID"},
{ERROR_IPSEC_IKE_INVALID_HASH,"ERROR_IPSEC_IKE_INVALID_HASH"},
{ERROR_IPSEC_IKE_INVALID_HASH_ALG,"ERROR_IPSEC_IKE_INVALID_HASH_ALG"},
{ERROR_IPSEC_IKE_INVALID_HASH_SIZE,"ERROR_IPSEC_IKE_INVALID_HASH_SIZE"},
{ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG,"ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG"},
{ERROR_IPSEC_IKE_INVALID_AUTH_ALG,"ERROR_IPSEC_IKE_INVALID_AUTH_ALG"},
{ERROR_IPSEC_IKE_INVALID_SIG,"ERROR_IPSEC_IKE_INVALID_SIG"},
{ERROR_IPSEC_IKE_LOAD_FAILED,"ERROR_IPSEC_IKE_LOAD_FAILED"},
{ERROR_IPSEC_IKE_RPC_DELETE,"ERROR_IPSEC_IKE_RPC_DELETE"},
{ERROR_IPSEC_IKE_BENIGN_REINIT,"ERROR_IPSEC_IKE_BENIGN_REINIT"},
{ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY,"ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY"},
{ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN,"ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN"},
{ERROR_IPSEC_IKE_MM_LIMIT,"ERROR_IPSEC_IKE_MM_LIMIT"},
{ERROR_IPSEC_IKE_NEGOTIATION_DISABLED,"ERROR_IPSEC_IKE_NEGOTIATION_DISABLED"},
{ERROR_IPSEC_IKE_NEG_STATUS_END,"ERROR_IPSEC_IKE_NEG_STATUS_END"},
{E_UNEXPECTED,"E_UNEXPECTED"},
{E_NOTIMPL,"E_NOTIMPL"},
{E_OUTOFMEMORY,"E_OUTOFMEMORY"},
{E_INVALIDARG,"E_INVALIDARG"},
{E_NOINTERFACE,"E_NOINTERFACE"},
{E_POINTER,"E_POINTER"},
{E_HANDLE,"E_HANDLE"},
{E_ABORT,"E_ABORT"},
{E_FAIL,"E_FAIL"},
{E_ACCESSDENIED,"E_ACCESSDENIED"},
{E_NOTIMPL,"E_NOTIMPL"},
{E_OUTOFMEMORY,"E_OUTOFMEMORY"},
{E_INVALIDARG,"E_INVALIDARG"},
{E_NOINTERFACE,"E_NOINTERFACE"},
{E_POINTER,"E_POINTER"},
{E_HANDLE,"E_HANDLE"},
{E_ABORT,"E_ABORT"},
{E_FAIL,"E_FAIL"},
{E_ACCESSDENIED,"E_ACCESSDENIED"},
{E_PENDING,"E_PENDING"},
{CO_E_INIT_TLS,"CO_E_INIT_TLS"},
{CO_E_INIT_SHARED_ALLOCATOR,"CO_E_INIT_SHARED_ALLOCATOR"},
{CO_E_INIT_MEMORY_ALLOCATOR,"CO_E_INIT_MEMORY_ALLOCATOR"},
{CO_E_INIT_CLASS_CACHE,"CO_E_INIT_CLASS_CACHE"},
{CO_E_INIT_RPC_CHANNEL,"CO_E_INIT_RPC_CHANNEL"},
{CO_E_INIT_TLS_SET_CHANNEL_CONTROL,"CO_E_INIT_TLS_SET_CHANNEL_CONTROL"},
{CO_E_INIT_TLS_CHANNEL_CONTROL,"CO_E_INIT_TLS_CHANNEL_CONTROL"},
{CO_E_INIT_UNACCEPTED_USER_ALLOCATOR,"CO_E_INIT_UNACCEPTED_USER_ALLOCATOR"},
{CO_E_INIT_SCM_MUTEX_EXISTS,"CO_E_INIT_SCM_MUTEX_EXISTS"},
{CO_E_INIT_SCM_FILE_MAPPING_EXISTS,"CO_E_INIT_SCM_FILE_MAPPING_EXISTS"},
{CO_E_INIT_SCM_MAP_VIEW_OF_FILE,"CO_E_INIT_SCM_MAP_VIEW_OF_FILE"},
{CO_E_INIT_SCM_EXEC_FAILURE,"CO_E_INIT_SCM_EXEC_FAILURE"},
{CO_E_INIT_ONLY_SINGLE_THREADED,"CO_E_INIT_ONLY_SINGLE_THREADED"},
{CO_E_CANT_REMOTE,"CO_E_CANT_REMOTE"},
{CO_E_BAD_SERVER_NAME,"CO_E_BAD_SERVER_NAME"},
{CO_E_WRONG_SERVER_IDENTITY,"CO_E_WRONG_SERVER_IDENTITY"},
{CO_E_OLE1DDE_DISABLED,"CO_E_OLE1DDE_DISABLED"},
{CO_E_RUNAS_SYNTAX,"CO_E_RUNAS_SYNTAX"},
{CO_E_CREATEPROCESS_FAILURE,"CO_E_CREATEPROCESS_FAILURE"},
{CO_E_RUNAS_CREATEPROCESS_FAILURE,"CO_E_RUNAS_CREATEPROCESS_FAILURE"},
{CO_E_RUNAS_LOGON_FAILURE,"CO_E_RUNAS_LOGON_FAILURE"},
{CO_E_LAUNCH_PERMSSION_DENIED,"CO_E_LAUNCH_PERMSSION_DENIED"},
{CO_E_START_SERVICE_FAILURE,"CO_E_START_SERVICE_FAILURE"},
{CO_E_REMOTE_COMMUNICATION_FAILURE,"CO_E_REMOTE_COMMUNICATION_FAILURE"},
{CO_E_SERVER_START_TIMEOUT,"CO_E_SERVER_START_TIMEOUT"},
{CO_E_CLSREG_INCONSISTENT,"CO_E_CLSREG_INCONSISTENT"},
{CO_E_IIDREG_INCONSISTENT,"CO_E_IIDREG_INCONSISTENT"},
{CO_E_NOT_SUPPORTED,"CO_E_NOT_SUPPORTED"},
{CO_E_RELOAD_DLL,"CO_E_RELOAD_DLL"},
{CO_E_MSI_ERROR,"CO_E_MSI_ERROR"},
{CO_E_ATTEMPT_TO_CREATE_OUTSIDE_CLIENT_CONTEXT,"CO_E_ATTEMPT_TO_CREATE_OUTSIDE_CLIENT_CONTEXT"},
{CO_E_SERVER_PAUSED,"CO_E_SERVER_PAUSED"},
{CO_E_SERVER_NOT_PAUSED,"CO_E_SERVER_NOT_PAUSED"},
{CO_E_CLASS_DISABLED,"CO_E_CLASS_DISABLED"},
{CO_E_CLRNOTAVAILABLE,"CO_E_CLRNOTAVAILABLE"},
{CO_E_ASYNC_WORK_REJECTED,"CO_E_ASYNC_WORK_REJECTED"},
{CO_E_SERVER_INIT_TIMEOUT,"CO_E_SERVER_INIT_TIMEOUT"},
{CO_E_NO_SECCTX_IN_ACTIVATE,"CO_E_NO_SECCTX_IN_ACTIVATE"},
{CO_E_TRACKER_CONFIG,"CO_E_TRACKER_CONFIG"},
{CO_E_THREADPOOL_CONFIG,"CO_E_THREADPOOL_CONFIG"},
{CO_E_SXS_CONFIG,"CO_E_SXS_CONFIG"},
{CO_E_MALFORMED_SPN,"CO_E_MALFORMED_SPN"},
{OLE_E_FIRST,"OLE_E_FIRST"},
{OLE_E_LAST,"OLE_E_LAST"},
{OLE_S_FIRST,"OLE_S_FIRST"},
{OLE_S_LAST,"OLE_S_LAST"},
{OLE_E_OLEVERB,"OLE_E_OLEVERB"},
{OLE_E_ADVF,"OLE_E_ADVF"},
{OLE_E_ENUM_NOMORE,"OLE_E_ENUM_NOMORE"},
{OLE_E_ADVISENOTSUPPORTED,"OLE_E_ADVISENOTSUPPORTED"},
{OLE_E_NOCONNECTION,"OLE_E_NOCONNECTION"},
{OLE_E_NOTRUNNING,"OLE_E_NOTRUNNING"},
{OLE_E_NOCACHE,"OLE_E_NOCACHE"},
{OLE_E_BLANK,"OLE_E_BLANK"},
{OLE_E_CLASSDIFF,"OLE_E_CLASSDIFF"},
{OLE_E_CANT_GETMONIKER,"OLE_E_CANT_GETMONIKER"},
{OLE_E_CANT_BINDTOSOURCE,"OLE_E_CANT_BINDTOSOURCE"},
{OLE_E_STATIC,"OLE_E_STATIC"},
{OLE_E_PROMPTSAVECANCELLED,"OLE_E_PROMPTSAVECANCELLED"},
{OLE_E_INVALIDRECT,"OLE_E_INVALIDRECT"},
{OLE_E_WRONGCOMPOBJ,"OLE_E_WRONGCOMPOBJ"},
{OLE_E_INVALIDHWND,"OLE_E_INVALIDHWND"},
{OLE_E_NOT_INPLACEACTIVE,"OLE_E_NOT_INPLACEACTIVE"},
{OLE_E_CANTCONVERT,"OLE_E_CANTCONVERT"},
{OLE_E_NOSTORAGE,"OLE_E_NOSTORAGE"},
{DV_E_FORMATETC,"DV_E_FORMATETC"},
{DV_E_DVTARGETDEVICE,"DV_E_DVTARGETDEVICE"},
{DV_E_STGMEDIUM,"DV_E_STGMEDIUM"},
{DV_E_STATDATA,"DV_E_STATDATA"},
{DV_E_LINDEX,"DV_E_LINDEX"},
{DV_E_TYMED,"DV_E_TYMED"},
{DV_E_CLIPFORMAT,"DV_E_CLIPFORMAT"},
{DV_E_DVASPECT,"DV_E_DVASPECT"},
{DV_E_DVTARGETDEVICE_SIZE,"DV_E_DVTARGETDEVICE_SIZE"},
{DV_E_NOIVIEWOBJECT,"DV_E_NOIVIEWOBJECT"},
{DRAGDROP_E_FIRST,"DRAGDROP_E_FIRST"},
{DRAGDROP_E_LAST,"DRAGDROP_E_LAST"},
{DRAGDROP_S_FIRST,"DRAGDROP_S_FIRST"},
{DRAGDROP_S_LAST,"DRAGDROP_S_LAST"},
{DRAGDROP_E_NOTREGISTERED,"DRAGDROP_E_NOTREGISTERED"},
{DRAGDROP_E_ALREADYREGISTERED,"DRAGDROP_E_ALREADYREGISTERED"},
{DRAGDROP_E_INVALIDHWND,"DRAGDROP_E_INVALIDHWND"},
{CLASSFACTORY_E_FIRST,"CLASSFACTORY_E_FIRST"},
{CLASSFACTORY_E_LAST,"CLASSFACTORY_E_LAST"},
{CLASSFACTORY_S_FIRST,"CLASSFACTORY_S_FIRST"},
{CLASSFACTORY_S_LAST,"CLASSFACTORY_S_LAST"},
{CLASS_E_NOAGGREGATION,"CLASS_E_NOAGGREGATION"},
{CLASS_E_CLASSNOTAVAILABLE,"CLASS_E_CLASSNOTAVAILABLE"},
{CLASS_E_NOTLICENSED,"CLASS_E_NOTLICENSED"},
{MARSHAL_E_FIRST,"MARSHAL_E_FIRST"},
{MARSHAL_E_LAST,"MARSHAL_E_LAST"},
{MARSHAL_S_FIRST,"MARSHAL_S_FIRST"},
{MARSHAL_S_LAST,"MARSHAL_S_LAST"},
{DATA_E_FIRST,"DATA_E_FIRST"},
{DATA_E_LAST,"DATA_E_LAST"},
{DATA_S_FIRST,"DATA_S_FIRST"},
{DATA_S_LAST,"DATA_S_LAST"},
{VIEW_E_FIRST,"VIEW_E_FIRST"},
{VIEW_E_LAST,"VIEW_E_LAST"},
{VIEW_S_FIRST,"VIEW_S_FIRST"},
{VIEW_S_LAST,"VIEW_S_LAST"},
{VIEW_E_DRAW,"VIEW_E_DRAW"},
{REGDB_E_FIRST,"REGDB_E_FIRST"},
{REGDB_E_LAST,"REGDB_E_LAST"},
{REGDB_S_FIRST,"REGDB_S_FIRST"},
{REGDB_S_LAST,"REGDB_S_LAST"},
{REGDB_E_READREGDB,"REGDB_E_READREGDB"},
{REGDB_E_WRITEREGDB,"REGDB_E_WRITEREGDB"},
{REGDB_E_KEYMISSING,"REGDB_E_KEYMISSING"},
{REGDB_E_INVALIDVALUE,"REGDB_E_INVALIDVALUE"},
{REGDB_E_CLASSNOTREG,"REGDB_E_CLASSNOTREG"},
{REGDB_E_IIDNOTREG,"REGDB_E_IIDNOTREG"},
{REGDB_E_BADTHREADINGMODEL,"REGDB_E_BADTHREADINGMODEL"},
{CAT_E_FIRST,"CAT_E_FIRST"},
{CAT_E_LAST,"CAT_E_LAST"},
{CAT_E_CATIDNOEXIST,"CAT_E_CATIDNOEXIST"},
{CAT_E_NODESCRIPTION,"CAT_E_NODESCRIPTION"},
{CS_E_FIRST,"CS_E_FIRST"},
{CS_E_LAST,"CS_E_LAST"},
{CS_E_PACKAGE_NOTFOUND,"CS_E_PACKAGE_NOTFOUND"},
{CS_E_NOT_DELETABLE,"CS_E_NOT_DELETABLE"},
{CS_E_CLASS_NOTFOUND,"CS_E_CLASS_NOTFOUND"},
{CS_E_INVALID_VERSION,"CS_E_INVALID_VERSION"},
{CS_E_NO_CLASSSTORE,"CS_E_NO_CLASSSTORE"},
{CS_E_OBJECT_NOTFOUND,"CS_E_OBJECT_NOTFOUND"},
{CS_E_OBJECT_ALREADY_EXISTS,"CS_E_OBJECT_ALREADY_EXISTS"},
{CS_E_INVALID_PATH,"CS_E_INVALID_PATH"},
{CS_E_NETWORK_ERROR,"CS_E_NETWORK_ERROR"},
{CS_E_ADMIN_LIMIT_EXCEEDED,"CS_E_ADMIN_LIMIT_EXCEEDED"},
{CS_E_SCHEMA_MISMATCH,"CS_E_SCHEMA_MISMATCH"},
{CS_E_INTERNAL_ERROR,"CS_E_INTERNAL_ERROR"},
{CACHE_E_FIRST,"CACHE_E_FIRST"},
{CACHE_E_LAST,"CACHE_E_LAST"},
{CACHE_S_FIRST,"CACHE_S_FIRST"},
{CACHE_S_LAST,"CACHE_S_LAST"},
{CACHE_E_NOCACHE_UPDATED,"CACHE_E_NOCACHE_UPDATED"},
{OLEOBJ_E_FIRST,"OLEOBJ_E_FIRST"},
{OLEOBJ_E_LAST,"OLEOBJ_E_LAST"},
{OLEOBJ_S_FIRST,"OLEOBJ_S_FIRST"},
{OLEOBJ_S_LAST,"OLEOBJ_S_LAST"},
{OLEOBJ_E_NOVERBS,"OLEOBJ_E_NOVERBS"},
{OLEOBJ_E_INVALIDVERB,"OLEOBJ_E_INVALIDVERB"},
{CLIENTSITE_E_FIRST,"CLIENTSITE_E_FIRST"},
{CLIENTSITE_E_LAST,"CLIENTSITE_E_LAST"},
{CLIENTSITE_S_FIRST,"CLIENTSITE_S_FIRST"},
{CLIENTSITE_S_LAST,"CLIENTSITE_S_LAST"},
{INPLACE_E_NOTUNDOABLE,"INPLACE_E_NOTUNDOABLE"},
{INPLACE_E_NOTOOLSPACE,"INPLACE_E_NOTOOLSPACE"},
{INPLACE_E_FIRST,"INPLACE_E_FIRST"},
{INPLACE_E_LAST,"INPLACE_E_LAST"},
{INPLACE_S_FIRST,"INPLACE_S_FIRST"},
{INPLACE_S_LAST,"INPLACE_S_LAST"},
{ENUM_E_FIRST,"ENUM_E_FIRST"},
{ENUM_E_LAST,"ENUM_E_LAST"},
{ENUM_S_FIRST,"ENUM_S_FIRST"},
{ENUM_S_LAST,"ENUM_S_LAST"},
{CONVERT10_E_FIRST,"CONVERT10_E_FIRST"},
{CONVERT10_E_LAST,"CONVERT10_E_LAST"},
{CONVERT10_S_FIRST,"CONVERT10_S_FIRST"},
{CONVERT10_S_LAST,"CONVERT10_S_LAST"},
{CONVERT10_E_OLESTREAM_GET,"CONVERT10_E_OLESTREAM_GET"},
{CONVERT10_E_OLESTREAM_PUT,"CONVERT10_E_OLESTREAM_PUT"},
{CONVERT10_E_OLESTREAM_FMT,"CONVERT10_E_OLESTREAM_FMT"},
{CONVERT10_E_OLESTREAM_BITMAP_TO_DIB,"CONVERT10_E_OLESTREAM_BITMAP_TO_DIB"},
{CONVERT10_E_STG_FMT,"CONVERT10_E_STG_FMT"},
{CONVERT10_E_STG_NO_STD_STREAM,"CONVERT10_E_STG_NO_STD_STREAM"},
{CONVERT10_E_STG_DIB_TO_BITMAP,"CONVERT10_E_STG_DIB_TO_BITMAP"},
{CLIPBRD_E_FIRST,"CLIPBRD_E_FIRST"},
{CLIPBRD_E_LAST,"CLIPBRD_E_LAST"},
{CLIPBRD_S_FIRST,"CLIPBRD_S_FIRST"},
{CLIPBRD_S_LAST,"CLIPBRD_S_LAST"},
{CLIPBRD_E_CANT_OPEN,"CLIPBRD_E_CANT_OPEN"},
{CLIPBRD_E_CANT_EMPTY,"CLIPBRD_E_CANT_EMPTY"},
{CLIPBRD_E_CANT_SET,"CLIPBRD_E_CANT_SET"},
{CLIPBRD_E_BAD_DATA,"CLIPBRD_E_BAD_DATA"},
{CLIPBRD_E_CANT_CLOSE,"CLIPBRD_E_CANT_CLOSE"},
{MK_E_FIRST,"MK_E_FIRST"},
{MK_E_LAST,"MK_E_LAST"},
{MK_S_FIRST,"MK_S_FIRST"},
{MK_S_LAST,"MK_S_LAST"},
{MK_E_CONNECTMANUALLY,"MK_E_CONNECTMANUALLY"},
{MK_E_EXCEEDEDDEADLINE,"MK_E_EXCEEDEDDEADLINE"},
{MK_E_NEEDGENERIC,"MK_E_NEEDGENERIC"},
{MK_E_UNAVAILABLE,"MK_E_UNAVAILABLE"},
{MK_E_SYNTAX,"MK_E_SYNTAX"},
{MK_E_NOOBJECT,"MK_E_NOOBJECT"},
{MK_E_INVALIDEXTENSION,"MK_E_INVALIDEXTENSION"},
{MK_E_INTERMEDIATEINTERFACENOTSUPPORTED,"MK_E_INTERMEDIATEINTERFACENOTSUPPORTED"},
{MK_E_NOTBINDABLE,"MK_E_NOTBINDABLE"},
{MK_E_NOTBOUND,"MK_E_NOTBOUND"},
{MK_E_CANTOPENFILE,"MK_E_CANTOPENFILE"},
{MK_E_MUSTBOTHERUSER,"MK_E_MUSTBOTHERUSER"},
{MK_E_NOINVERSE,"MK_E_NOINVERSE"},
{MK_E_NOSTORAGE,"MK_E_NOSTORAGE"},
{MK_E_NOPREFIX,"MK_E_NOPREFIX"},
{MK_E_ENUMERATION_FAILED,"MK_E_ENUMERATION_FAILED"},
{CO_E_FIRST,"CO_E_FIRST"},
{CO_E_LAST,"CO_E_LAST"},
{CO_S_FIRST,"CO_S_FIRST"},
{CO_S_LAST,"CO_S_LAST"},
{CO_E_NOTINITIALIZED,"CO_E_NOTINITIALIZED"},
{CO_E_ALREADYINITIALIZED,"CO_E_ALREADYINITIALIZED"},
{CO_E_CANTDETERMINECLASS,"CO_E_CANTDETERMINECLASS"},
{CO_E_CLASSSTRING,"CO_E_CLASSSTRING"},
{CO_E_IIDSTRING,"CO_E_IIDSTRING"},
{CO_E_APPNOTFOUND,"CO_E_APPNOTFOUND"},
{CO_E_APPSINGLEUSE,"CO_E_APPSINGLEUSE"},
{CO_E_ERRORINAPP,"CO_E_ERRORINAPP"},
{CO_E_DLLNOTFOUND,"CO_E_DLLNOTFOUND"},
{CO_E_ERRORINDLL,"CO_E_ERRORINDLL"},
{CO_E_WRONGOSFORAPP,"CO_E_WRONGOSFORAPP"},
{CO_E_OBJNOTREG,"CO_E_OBJNOTREG"},
{CO_E_OBJISREG,"CO_E_OBJISREG"},
{CO_E_OBJNOTCONNECTED,"CO_E_OBJNOTCONNECTED"},
{CO_E_APPDIDNTREG,"CO_E_APPDIDNTREG"},
{CO_E_RELEASED,"CO_E_RELEASED"},
{EVENT_E_FIRST,"EVENT_E_FIRST"},
{EVENT_E_LAST,"EVENT_E_LAST"},
{EVENT_S_FIRST,"EVENT_S_FIRST"},
{EVENT_S_LAST,"EVENT_S_LAST"},
{EVENT_S_SOME_SUBSCRIBERS_FAILED,"EVENT_S_SOME_SUBSCRIBERS_FAILED"},
{EVENT_E_ALL_SUBSCRIBERS_FAILED,"EVENT_E_ALL_SUBSCRIBERS_FAILED"},
{EVENT_S_NOSUBSCRIBERS,"EVENT_S_NOSUBSCRIBERS"},
{EVENT_E_QUERYSYNTAX,"EVENT_E_QUERYSYNTAX"},
{EVENT_E_QUERYFIELD,"EVENT_E_QUERYFIELD"},
{EVENT_E_INTERNALEXCEPTION,"EVENT_E_INTERNALEXCEPTION"},
{EVENT_E_INTERNALERROR,"EVENT_E_INTERNALERROR"},
{EVENT_E_INVALID_PER_USER_SID,"EVENT_E_INVALID_PER_USER_SID"},
{EVENT_E_USER_EXCEPTION,"EVENT_E_USER_EXCEPTION"},
{EVENT_E_TOO_MANY_METHODS,"EVENT_E_TOO_MANY_METHODS"},
{EVENT_E_MISSING_EVENTCLASS,"EVENT_E_MISSING_EVENTCLASS"},
{EVENT_E_NOT_ALL_REMOVED,"EVENT_E_NOT_ALL_REMOVED"},
{EVENT_E_COMPLUS_NOT_INSTALLED,"EVENT_E_COMPLUS_NOT_INSTALLED"},
{EVENT_E_CANT_MODIFY_OR_DELETE_UNCONFIGURED_OBJECT,"EVENT_E_CANT_MODIFY_OR_DELETE_UNCONFIGURED_OBJECT"},
{EVENT_E_CANT_MODIFY_OR_DELETE_CONFIGURED_OBJECT,"EVENT_E_CANT_MODIFY_OR_DELETE_CONFIGURED_OBJECT"},
{EVENT_E_INVALID_EVENT_CLASS_PARTITION,"EVENT_E_INVALID_EVENT_CLASS_PARTITION"},
{EVENT_E_PER_USER_SID_NOT_LOGGED_ON,"EVENT_E_PER_USER_SID_NOT_LOGGED_ON"},
{XACT_E_FIRST,"XACT_E_FIRST"},
{XACT_E_LAST,"XACT_E_LAST"},
{XACT_S_FIRST,"XACT_S_FIRST"},
{XACT_S_LAST,"XACT_S_LAST"},
{XACT_E_ALREADYOTHERSINGLEPHASE,"XACT_E_ALREADYOTHERSINGLEPHASE"},
{XACT_E_CANTRETAIN,"XACT_E_CANTRETAIN"},
{XACT_E_COMMITFAILED,"XACT_E_COMMITFAILED"},
{XACT_E_COMMITPREVENTED,"XACT_E_COMMITPREVENTED"},
{XACT_E_HEURISTICABORT,"XACT_E_HEURISTICABORT"},
{XACT_E_HEURISTICCOMMIT,"XACT_E_HEURISTICCOMMIT"},
{XACT_E_HEURISTICDAMAGE,"XACT_E_HEURISTICDAMAGE"},
{XACT_E_HEURISTICDANGER,"XACT_E_HEURISTICDANGER"},
{XACT_E_ISOLATIONLEVEL,"XACT_E_ISOLATIONLEVEL"},
{XACT_E_NOASYNC,"XACT_E_NOASYNC"},
{XACT_E_NOENLIST,"XACT_E_NOENLIST"},
{XACT_E_NOISORETAIN,"XACT_E_NOISORETAIN"},
{XACT_E_NORESOURCE,"XACT_E_NORESOURCE"},
{XACT_E_NOTCURRENT,"XACT_E_NOTCURRENT"},
{XACT_E_NOTRANSACTION,"XACT_E_NOTRANSACTION"},
{XACT_E_NOTSUPPORTED,"XACT_E_NOTSUPPORTED"},
{XACT_E_UNKNOWNRMGRID,"XACT_E_UNKNOWNRMGRID"},
{XACT_E_WRONGSTATE,"XACT_E_WRONGSTATE"},
{XACT_E_WRONGUOW,"XACT_E_WRONGUOW"},
{XACT_E_XTIONEXISTS,"XACT_E_XTIONEXISTS"},
{XACT_E_NOIMPORTOBJECT,"XACT_E_NOIMPORTOBJECT"},
{XACT_E_INVALIDCOOKIE,"XACT_E_INVALIDCOOKIE"},
{XACT_E_INDOUBT,"XACT_E_INDOUBT"},
{XACT_E_NOTIMEOUT,"XACT_E_NOTIMEOUT"},
{XACT_E_ALREADYINPROGRESS,"XACT_E_ALREADYINPROGRESS"},
{XACT_E_ABORTED,"XACT_E_ABORTED"},
{XACT_E_LOGFULL,"XACT_E_LOGFULL"},
{XACT_E_TMNOTAVAILABLE,"XACT_E_TMNOTAVAILABLE"},
{XACT_E_CONNECTION_DOWN,"XACT_E_CONNECTION_DOWN"},
{XACT_E_CONNECTION_DENIED,"XACT_E_CONNECTION_DENIED"},
{XACT_E_REENLISTTIMEOUT,"XACT_E_REENLISTTIMEOUT"},
{XACT_E_TIP_CONNECT_FAILED,"XACT_E_TIP_CONNECT_FAILED"},
{XACT_E_TIP_PROTOCOL_ERROR,"XACT_E_TIP_PROTOCOL_ERROR"},
{XACT_E_TIP_PULL_FAILED,"XACT_E_TIP_PULL_FAILED"},
{XACT_E_DEST_TMNOTAVAILABLE,"XACT_E_DEST_TMNOTAVAILABLE"},
{XACT_E_TIP_DISABLED,"XACT_E_TIP_DISABLED"},
{XACT_E_NETWORK_TX_DISABLED,"XACT_E_NETWORK_TX_DISABLED"},
{XACT_E_PARTNER_NETWORK_TX_DISABLED,"XACT_E_PARTNER_NETWORK_TX_DISABLED"},
{XACT_E_XA_TX_DISABLED,"XACT_E_XA_TX_DISABLED"},
{XACT_E_UNABLE_TO_READ_DTC_CONFIG,"XACT_E_UNABLE_TO_READ_DTC_CONFIG"},
{XACT_E_UNABLE_TO_LOAD_DTC_PROXY,"XACT_E_UNABLE_TO_LOAD_DTC_PROXY"},
{XACT_E_ABORTING,"XACT_E_ABORTING"},
{XACT_E_CLERKNOTFOUND,"XACT_E_CLERKNOTFOUND"},
{XACT_E_CLERKEXISTS,"XACT_E_CLERKEXISTS"},
{XACT_E_RECOVERYINPROGRESS,"XACT_E_RECOVERYINPROGRESS"},
{XACT_E_TRANSACTIONCLOSED,"XACT_E_TRANSACTIONCLOSED"},
{XACT_E_INVALIDLSN,"XACT_E_INVALIDLSN"},
{XACT_E_REPLAYREQUEST,"XACT_E_REPLAYREQUEST"},
{XACT_S_ASYNC,"XACT_S_ASYNC"},
{XACT_S_DEFECT,"XACT_S_DEFECT"},
{XACT_S_READONLY,"XACT_S_READONLY"},
{XACT_S_SOMENORETAIN,"XACT_S_SOMENORETAIN"},
{XACT_S_OKINFORM,"XACT_S_OKINFORM"},
{XACT_S_MADECHANGESCONTENT,"XACT_S_MADECHANGESCONTENT"},
{XACT_S_MADECHANGESINFORM,"XACT_S_MADECHANGESINFORM"},
{XACT_S_ALLNORETAIN,"XACT_S_ALLNORETAIN"},
{XACT_S_ABORTING,"XACT_S_ABORTING"},
{XACT_S_SINGLEPHASE,"XACT_S_SINGLEPHASE"},
{XACT_S_LOCALLY_OK,"XACT_S_LOCALLY_OK"},
{XACT_S_LASTRESOURCEMANAGER,"XACT_S_LASTRESOURCEMANAGER"},
{CONTEXT_E_FIRST,"CONTEXT_E_FIRST"},
{CONTEXT_E_LAST,"CONTEXT_E_LAST"},
{CONTEXT_S_FIRST,"CONTEXT_S_FIRST"},
{CONTEXT_S_LAST,"CONTEXT_S_LAST"},
{CONTEXT_E_ABORTED,"CONTEXT_E_ABORTED"},
{CONTEXT_E_ABORTING,"CONTEXT_E_ABORTING"},
{CONTEXT_E_NOCONTEXT,"CONTEXT_E_NOCONTEXT"},
{CONTEXT_E_SYNCH_TIMEOUT,"CONTEXT_E_SYNCH_TIMEOUT"},
{CONTEXT_E_OLDREF,"CONTEXT_E_OLDREF"},
{CONTEXT_E_ROLENOTFOUND,"CONTEXT_E_ROLENOTFOUND"},
{CONTEXT_E_TMNOTAVAILABLE,"CONTEXT_E_TMNOTAVAILABLE"},
{CO_E_ACTIVATIONFAILED,"CO_E_ACTIVATIONFAILED"},
{CO_E_ACTIVATIONFAILED_EVENTLOGGED,"CO_E_ACTIVATIONFAILED_EVENTLOGGED"},
{CO_E_ACTIVATIONFAILED_CATALOGERROR,"CO_E_ACTIVATIONFAILED_CATALOGERROR"},
{CO_E_ACTIVATIONFAILED_TIMEOUT,"CO_E_ACTIVATIONFAILED_TIMEOUT"},
{CO_E_INITIALIZATIONFAILED,"CO_E_INITIALIZATIONFAILED"},
{CONTEXT_E_NOJIT,"CONTEXT_E_NOJIT"},
{CONTEXT_E_NOTRANSACTION,"CONTEXT_E_NOTRANSACTION"},
{CO_E_THREADINGMODEL_CHANGED,"CO_E_THREADINGMODEL_CHANGED"},
{CO_E_NOIISINTRINSICS,"CO_E_NOIISINTRINSICS"},
{CO_E_NOCOOKIES,"CO_E_NOCOOKIES"},
{CO_E_DBERROR,"CO_E_DBERROR"},
{CO_E_NOTPOOLED,"CO_E_NOTPOOLED"},
{CO_E_NOTCONSTRUCTED,"CO_E_NOTCONSTRUCTED"},
{CO_E_NOSYNCHRONIZATION,"CO_E_NOSYNCHRONIZATION"},
{CO_E_ISOLEVELMISMATCH,"CO_E_ISOLEVELMISMATCH"},
{OLE_S_USEREG,"OLE_S_USEREG"},
{OLE_S_STATIC,"OLE_S_STATIC"},
{OLE_S_MAC_CLIPFORMAT,"OLE_S_MAC_CLIPFORMAT"},
{DRAGDROP_S_DROP,"DRAGDROP_S_DROP"},
{DRAGDROP_S_CANCEL,"DRAGDROP_S_CANCEL"},
{DRAGDROP_S_USEDEFAULTCURSORS,"DRAGDROP_S_USEDEFAULTCURSORS"},
{DATA_S_SAMEFORMATETC,"DATA_S_SAMEFORMATETC"},
{VIEW_S_ALREADY_FROZEN,"VIEW_S_ALREADY_FROZEN"},
{CACHE_S_FORMATETC_NOTSUPPORTED,"CACHE_S_FORMATETC_NOTSUPPORTED"},
{CACHE_S_SAMECACHE,"CACHE_S_SAMECACHE"},
{CACHE_S_SOMECACHES_NOTUPDATED,"CACHE_S_SOMECACHES_NOTUPDATED"},
{OLEOBJ_S_INVALIDVERB,"OLEOBJ_S_INVALIDVERB"},
{OLEOBJ_S_CANNOT_DOVERB_NOW,"OLEOBJ_S_CANNOT_DOVERB_NOW"},
{OLEOBJ_S_INVALIDHWND,"OLEOBJ_S_INVALIDHWND"},
{INPLACE_S_TRUNCATED,"INPLACE_S_TRUNCATED"},
{CONVERT10_S_NO_PRESENTATION,"CONVERT10_S_NO_PRESENTATION"},
{MK_S_REDUCED_TO_SELF,"MK_S_REDUCED_TO_SELF"},
{MK_S_ME,"MK_S_ME"},
{MK_S_HIM,"MK_S_HIM"},
{MK_S_US,"MK_S_US"},
{MK_S_MONIKERALREADYREGISTERED,"MK_S_MONIKERALREADYREGISTERED"},
{SCHED_S_TASK_READY,"SCHED_S_TASK_READY"},
{SCHED_S_TASK_RUNNING,"SCHED_S_TASK_RUNNING"},
{SCHED_S_TASK_DISABLED,"SCHED_S_TASK_DISABLED"},
{SCHED_S_TASK_HAS_NOT_RUN,"SCHED_S_TASK_HAS_NOT_RUN"},
{SCHED_S_TASK_NO_MORE_RUNS,"SCHED_S_TASK_NO_MORE_RUNS"},
{SCHED_S_TASK_NOT_SCHEDULED,"SCHED_S_TASK_NOT_SCHEDULED"},
{SCHED_S_TASK_TERMINATED,"SCHED_S_TASK_TERMINATED"},
{SCHED_S_TASK_NO_VALID_TRIGGERS,"SCHED_S_TASK_NO_VALID_TRIGGERS"},
{SCHED_S_EVENT_TRIGGER,"SCHED_S_EVENT_TRIGGER"},
{SCHED_E_TRIGGER_NOT_FOUND,"SCHED_E_TRIGGER_NOT_FOUND"},
{SCHED_E_TASK_NOT_READY,"SCHED_E_TASK_NOT_READY"},
{SCHED_E_TASK_NOT_RUNNING,"SCHED_E_TASK_NOT_RUNNING"},
{SCHED_E_SERVICE_NOT_INSTALLED,"SCHED_E_SERVICE_NOT_INSTALLED"},
{SCHED_E_CANNOT_OPEN_TASK,"SCHED_E_CANNOT_OPEN_TASK"},
{SCHED_E_INVALID_TASK,"SCHED_E_INVALID_TASK"},
{SCHED_E_ACCOUNT_INFORMATION_NOT_SET,"SCHED_E_ACCOUNT_INFORMATION_NOT_SET"},
{SCHED_E_ACCOUNT_NAME_NOT_FOUND,"SCHED_E_ACCOUNT_NAME_NOT_FOUND"},
{SCHED_E_ACCOUNT_DBASE_CORRUPT,"SCHED_E_ACCOUNT_DBASE_CORRUPT"},
{SCHED_E_NO_SECURITY_SERVICES,"SCHED_E_NO_SECURITY_SERVICES"},
{SCHED_E_UNKNOWN_OBJECT_VERSION,"SCHED_E_UNKNOWN_OBJECT_VERSION"},
{SCHED_E_UNSUPPORTED_ACCOUNT_OPTION,"SCHED_E_UNSUPPORTED_ACCOUNT_OPTION"},
{SCHED_E_SERVICE_NOT_RUNNING,"SCHED_E_SERVICE_NOT_RUNNING"},
{CO_E_CLASS_CREATE_FAILED,"CO_E_CLASS_CREATE_FAILED"},
{CO_E_SCM_ERROR,"CO_E_SCM_ERROR"},
{CO_E_SCM_RPC_FAILURE,"CO_E_SCM_RPC_FAILURE"},
{CO_E_BAD_PATH,"CO_E_BAD_PATH"},
{CO_E_SERVER_EXEC_FAILURE,"CO_E_SERVER_EXEC_FAILURE"},
{CO_E_OBJSRV_RPC_FAILURE,"CO_E_OBJSRV_RPC_FAILURE"},
{MK_E_NO_NORMALIZED,"MK_E_NO_NORMALIZED"},
{CO_E_SERVER_STOPPING,"CO_E_SERVER_STOPPING"},
{MEM_E_INVALID_ROOT,"MEM_E_INVALID_ROOT"},
{MEM_E_INVALID_LINK,"MEM_E_INVALID_LINK"},
{MEM_E_INVALID_SIZE,"MEM_E_INVALID_SIZE"},
{CO_S_NOTALLINTERFACES,"CO_S_NOTALLINTERFACES"},
{CO_S_MACHINENAMENOTFOUND,"CO_S_MACHINENAMENOTFOUND"},
{DISP_E_UNKNOWNINTERFACE,"DISP_E_UNKNOWNINTERFACE"},
{DISP_E_MEMBERNOTFOUND,"DISP_E_MEMBERNOTFOUND"},
{DISP_E_PARAMNOTFOUND,"DISP_E_PARAMNOTFOUND"},
{DISP_E_TYPEMISMATCH,"DISP_E_TYPEMISMATCH"},
{DISP_E_UNKNOWNNAME,"DISP_E_UNKNOWNNAME"},
{DISP_E_NONAMEDARGS,"DISP_E_NONAMEDARGS"},
{DISP_E_BADVARTYPE,"DISP_E_BADVARTYPE"},
{DISP_E_EXCEPTION,"DISP_E_EXCEPTION"},
{DISP_E_OVERFLOW,"DISP_E_OVERFLOW"},
{DISP_E_BADINDEX,"DISP_E_BADINDEX"},
{DISP_E_UNKNOWNLCID,"DISP_E_UNKNOWNLCID"},
{DISP_E_ARRAYISLOCKED,"DISP_E_ARRAYISLOCKED"},
{DISP_E_BADPARAMCOUNT,"DISP_E_BADPARAMCOUNT"},
{DISP_E_PARAMNOTOPTIONAL,"DISP_E_PARAMNOTOPTIONAL"},
{DISP_E_BADCALLEE,"DISP_E_BADCALLEE"},
{DISP_E_NOTACOLLECTION,"DISP_E_NOTACOLLECTION"},
{DISP_E_DIVBYZERO,"DISP_E_DIVBYZERO"},
{DISP_E_BUFFERTOOSMALL,"DISP_E_BUFFERTOOSMALL"},
{TYPE_E_BUFFERTOOSMALL,"TYPE_E_BUFFERTOOSMALL"},
{TYPE_E_FIELDNOTFOUND,"TYPE_E_FIELDNOTFOUND"},
{TYPE_E_INVDATAREAD,"TYPE_E_INVDATAREAD"},
{TYPE_E_UNSUPFORMAT,"TYPE_E_UNSUPFORMAT"},
{TYPE_E_REGISTRYACCESS,"TYPE_E_REGISTRYACCESS"},
{TYPE_E_LIBNOTREGISTERED,"TYPE_E_LIBNOTREGISTERED"},
{TYPE_E_UNDEFINEDTYPE,"TYPE_E_UNDEFINEDTYPE"},
{TYPE_E_QUALIFIEDNAMEDISALLOWED,"TYPE_E_QUALIFIEDNAMEDISALLOWED"},
{TYPE_E_INVALIDSTATE,"TYPE_E_INVALIDSTATE"},
{TYPE_E_WRONGTYPEKIND,"TYPE_E_WRONGTYPEKIND"},
{TYPE_E_ELEMENTNOTFOUND,"TYPE_E_ELEMENTNOTFOUND"},
{TYPE_E_AMBIGUOUSNAME,"TYPE_E_AMBIGUOUSNAME"},
{TYPE_E_NAMECONFLICT,"TYPE_E_NAMECONFLICT"},
{TYPE_E_UNKNOWNLCID,"TYPE_E_UNKNOWNLCID"},
{TYPE_E_DLLFUNCTIONNOTFOUND,"TYPE_E_DLLFUNCTIONNOTFOUND"},
{TYPE_E_BADMODULEKIND,"TYPE_E_BADMODULEKIND"},
{TYPE_E_SIZETOOBIG,"TYPE_E_SIZETOOBIG"},
{TYPE_E_DUPLICATEID,"TYPE_E_DUPLICATEID"},
{TYPE_E_INVALIDID,"TYPE_E_INVALIDID"},
{TYPE_E_TYPEMISMATCH,"TYPE_E_TYPEMISMATCH"},
{TYPE_E_OUTOFBOUNDS,"TYPE_E_OUTOFBOUNDS"},
{TYPE_E_IOERROR,"TYPE_E_IOERROR"},
{TYPE_E_CANTCREATETMPFILE,"TYPE_E_CANTCREATETMPFILE"},
{TYPE_E_CANTLOADLIBRARY,"TYPE_E_CANTLOADLIBRARY"},
{TYPE_E_INCONSISTENTPROPFUNCS,"TYPE_E_INCONSISTENTPROPFUNCS"},
{TYPE_E_CIRCULARTYPE,"TYPE_E_CIRCULARTYPE"},
{STG_E_INVALIDFUNCTION,"STG_E_INVALIDFUNCTION"},
{STG_E_FILENOTFOUND,"STG_E_FILENOTFOUND"},
{STG_E_PATHNOTFOUND,"STG_E_PATHNOTFOUND"},
{STG_E_TOOMANYOPENFILES,"STG_E_TOOMANYOPENFILES"},
{STG_E_ACCESSDENIED,"STG_E_ACCESSDENIED"},
{STG_E_INVALIDHANDLE,"STG_E_INVALIDHANDLE"},
{STG_E_INSUFFICIENTMEMORY,"STG_E_INSUFFICIENTMEMORY"},
{STG_E_INVALIDPOINTER,"STG_E_INVALIDPOINTER"},
{STG_E_NOMOREFILES,"STG_E_NOMOREFILES"},
{STG_E_DISKISWRITEPROTECTED,"STG_E_DISKISWRITEPROTECTED"},
{STG_E_SEEKERROR,"STG_E_SEEKERROR"},
{STG_E_WRITEFAULT,"STG_E_WRITEFAULT"},
{STG_E_READFAULT,"STG_E_READFAULT"},
{STG_E_SHAREVIOLATION,"STG_E_SHAREVIOLATION"},
{STG_E_LOCKVIOLATION,"STG_E_LOCKVIOLATION"},
{STG_E_FILEALREADYEXISTS,"STG_E_FILEALREADYEXISTS"},
{STG_E_INVALIDPARAMETER,"STG_E_INVALIDPARAMETER"},
{STG_E_MEDIUMFULL,"STG_E_MEDIUMFULL"},
{STG_E_PROPSETMISMATCHED,"STG_E_PROPSETMISMATCHED"},
{STG_E_ABNORMALAPIEXIT,"STG_E_ABNORMALAPIEXIT"},
{STG_E_INVALIDHEADER,"STG_E_INVALIDHEADER"},
{STG_E_INVALIDNAME,"STG_E_INVALIDNAME"},
{STG_E_UNKNOWN,"STG_E_UNKNOWN"},
{STG_E_UNIMPLEMENTEDFUNCTION,"STG_E_UNIMPLEMENTEDFUNCTION"},
{STG_E_INVALIDFLAG,"STG_E_INVALIDFLAG"},
{STG_E_INUSE,"STG_E_INUSE"},
{STG_E_NOTCURRENT,"STG_E_NOTCURRENT"},
{STG_E_REVERTED,"STG_E_REVERTED"},
{STG_E_CANTSAVE,"STG_E_CANTSAVE"},
{STG_E_OLDFORMAT,"STG_E_OLDFORMAT"},
{STG_E_OLDDLL,"STG_E_OLDDLL"},
{STG_E_SHAREREQUIRED,"STG_E_SHAREREQUIRED"},
{STG_E_NOTFILEBASEDSTORAGE,"STG_E_NOTFILEBASEDSTORAGE"},
{STG_E_EXTANTMARSHALLINGS,"STG_E_EXTANTMARSHALLINGS"},
{STG_E_DOCFILECORRUPT,"STG_E_DOCFILECORRUPT"},
{STG_E_BADBASEADDRESS,"STG_E_BADBASEADDRESS"},
{STG_E_DOCFILETOOLARGE,"STG_E_DOCFILETOOLARGE"},
{STG_E_NOTSIMPLEFORMAT,"STG_E_NOTSIMPLEFORMAT"},
{STG_E_INCOMPLETE,"STG_E_INCOMPLETE"},
{STG_E_TERMINATED,"STG_E_TERMINATED"},
{STG_S_CONVERTED,"STG_S_CONVERTED"},
{STG_S_BLOCK,"STG_S_BLOCK"},
{STG_S_RETRYNOW,"STG_S_RETRYNOW"},
{STG_S_MONITORING,"STG_S_MONITORING"},
{STG_S_MULTIPLEOPENS,"STG_S_MULTIPLEOPENS"},
{STG_S_CONSOLIDATIONFAILED,"STG_S_CONSOLIDATIONFAILED"},
{STG_S_CANNOTCONSOLIDATE,"STG_S_CANNOTCONSOLIDATE"},
{STG_E_STATUS_COPY_PROTECTION_FAILURE,"STG_E_STATUS_COPY_PROTECTION_FAILURE"},
{STG_E_CSS_AUTHENTICATION_FAILURE,"STG_E_CSS_AUTHENTICATION_FAILURE"},
{STG_E_CSS_KEY_NOT_PRESENT,"STG_E_CSS_KEY_NOT_PRESENT"},
{STG_E_CSS_KEY_NOT_ESTABLISHED,"STG_E_CSS_KEY_NOT_ESTABLISHED"},
{STG_E_CSS_SCRAMBLED_SECTOR,"STG_E_CSS_SCRAMBLED_SECTOR"},
{STG_E_CSS_REGION_MISMATCH,"STG_E_CSS_REGION_MISMATCH"},
{STG_E_RESETS_EXHAUSTED,"STG_E_RESETS_EXHAUSTED"},
{RPC_E_CALL_REJECTED,"RPC_E_CALL_REJECTED"},
{RPC_E_CALL_CANCELED,"RPC_E_CALL_CANCELED"},
{RPC_E_CANTPOST_INSENDCALL,"RPC_E_CANTPOST_INSENDCALL"},
{RPC_E_CANTCALLOUT_INASYNCCALL,"RPC_E_CANTCALLOUT_INASYNCCALL"},
{RPC_E_CANTCALLOUT_INEXTERNALCALL,"RPC_E_CANTCALLOUT_INEXTERNALCALL"},
{RPC_E_CONNECTION_TERMINATED,"RPC_E_CONNECTION_TERMINATED"},
{RPC_E_SERVER_DIED,"RPC_E_SERVER_DIED"},
{RPC_E_CLIENT_DIED,"RPC_E_CLIENT_DIED"},
{RPC_E_INVALID_DATAPACKET,"RPC_E_INVALID_DATAPACKET"},
{RPC_E_CANTTRANSMIT_CALL,"RPC_E_CANTTRANSMIT_CALL"},
{RPC_E_CLIENT_CANTMARSHAL_DATA,"RPC_E_CLIENT_CANTMARSHAL_DATA"},
{RPC_E_CLIENT_CANTUNMARSHAL_DATA,"RPC_E_CLIENT_CANTUNMARSHAL_DATA"},
{RPC_E_SERVER_CANTMARSHAL_DATA,"RPC_E_SERVER_CANTMARSHAL_DATA"},
{RPC_E_SERVER_CANTUNMARSHAL_DATA,"RPC_E_SERVER_CANTUNMARSHAL_DATA"},
{RPC_E_INVALID_DATA,"RPC_E_INVALID_DATA"},
{RPC_E_INVALID_PARAMETER,"RPC_E_INVALID_PARAMETER"},
{RPC_E_CANTCALLOUT_AGAIN,"RPC_E_CANTCALLOUT_AGAIN"},
{RPC_E_SERVER_DIED_DNE,"RPC_E_SERVER_DIED_DNE"},
{RPC_E_SYS_CALL_FAILED,"RPC_E_SYS_CALL_FAILED"},
{RPC_E_OUT_OF_RESOURCES,"RPC_E_OUT_OF_RESOURCES"},
{RPC_E_ATTEMPTED_MULTITHREAD,"RPC_E_ATTEMPTED_MULTITHREAD"},
{RPC_E_NOT_REGISTERED,"RPC_E_NOT_REGISTERED"},
{RPC_E_FAULT,"RPC_E_FAULT"},
{RPC_E_SERVERFAULT,"RPC_E_SERVERFAULT"},
{RPC_E_CHANGED_MODE,"RPC_E_CHANGED_MODE"},
{RPC_E_INVALIDMETHOD,"RPC_E_INVALIDMETHOD"},
{RPC_E_DISCONNECTED,"RPC_E_DISCONNECTED"},
{RPC_E_RETRY,"RPC_E_RETRY"},
{RPC_E_SERVERCALL_RETRYLATER,"RPC_E_SERVERCALL_RETRYLATER"},
{RPC_E_SERVERCALL_REJECTED,"RPC_E_SERVERCALL_REJECTED"},
{RPC_E_INVALID_CALLDATA,"RPC_E_INVALID_CALLDATA"},
{RPC_E_CANTCALLOUT_ININPUTSYNCCALL,"RPC_E_CANTCALLOUT_ININPUTSYNCCALL"},
{RPC_E_WRONG_THREAD,"RPC_E_WRONG_THREAD"},
{RPC_E_THREAD_NOT_INIT,"RPC_E_THREAD_NOT_INIT"},
{RPC_E_VERSION_MISMATCH,"RPC_E_VERSION_MISMATCH"},
{RPC_E_INVALID_HEADER,"RPC_E_INVALID_HEADER"},
{RPC_E_INVALID_EXTENSION,"RPC_E_INVALID_EXTENSION"},
{RPC_E_INVALID_IPID,"RPC_E_INVALID_IPID"},
{RPC_E_INVALID_OBJECT,"RPC_E_INVALID_OBJECT"},
{RPC_S_CALLPENDING,"RPC_S_CALLPENDING"},
{RPC_S_WAITONTIMER,"RPC_S_WAITONTIMER"},
{RPC_E_CALL_COMPLETE,"RPC_E_CALL_COMPLETE"},
{RPC_E_UNSECURE_CALL,"RPC_E_UNSECURE_CALL"},
{RPC_E_TOO_LATE,"RPC_E_TOO_LATE"},
{RPC_E_NO_GOOD_SECURITY_PACKAGES,"RPC_E_NO_GOOD_SECURITY_PACKAGES"},
{RPC_E_ACCESS_DENIED,"RPC_E_ACCESS_DENIED"},
{RPC_E_REMOTE_DISABLED,"RPC_E_REMOTE_DISABLED"},
{RPC_E_INVALID_OBJREF,"RPC_E_INVALID_OBJREF"},
{RPC_E_NO_CONTEXT,"RPC_E_NO_CONTEXT"},
{RPC_E_TIMEOUT,"RPC_E_TIMEOUT"},
{RPC_E_NO_SYNC,"RPC_E_NO_SYNC"},
{RPC_E_FULLSIC_REQUIRED,"RPC_E_FULLSIC_REQUIRED"},
{RPC_E_INVALID_STD_NAME,"RPC_E_INVALID_STD_NAME"},
{CO_E_FAILEDTOIMPERSONATE,"CO_E_FAILEDTOIMPERSONATE"},
{CO_E_FAILEDTOGETSECCTX,"CO_E_FAILEDTOGETSECCTX"},
{CO_E_FAILEDTOOPENTHREADTOKEN,"CO_E_FAILEDTOOPENTHREADTOKEN"},
{CO_E_FAILEDTOGETTOKENINFO,"CO_E_FAILEDTOGETTOKENINFO"},
{CO_E_TRUSTEEDOESNTMATCHCLIENT,"CO_E_TRUSTEEDOESNTMATCHCLIENT"},
{CO_E_FAILEDTOQUERYCLIENTBLANKET,"CO_E_FAILEDTOQUERYCLIENTBLANKET"},
{CO_E_FAILEDTOSETDACL,"CO_E_FAILEDTOSETDACL"},
{CO_E_ACCESSCHECKFAILED,"CO_E_ACCESSCHECKFAILED"},
{CO_E_NETACCESSAPIFAILED,"CO_E_NETACCESSAPIFAILED"},
{CO_E_WRONGTRUSTEENAMESYNTAX,"CO_E_WRONGTRUSTEENAMESYNTAX"},
{CO_E_INVALIDSID,"CO_E_INVALIDSID"},
{CO_E_CONVERSIONFAILED,"CO_E_CONVERSIONFAILED"},
{CO_E_NOMATCHINGSIDFOUND,"CO_E_NOMATCHINGSIDFOUND"},
{CO_E_LOOKUPACCSIDFAILED,"CO_E_LOOKUPACCSIDFAILED"},
{CO_E_NOMATCHINGNAMEFOUND,"CO_E_NOMATCHINGNAMEFOUND"},
{CO_E_LOOKUPACCNAMEFAILED,"CO_E_LOOKUPACCNAMEFAILED"},
{CO_E_SETSERLHNDLFAILED,"CO_E_SETSERLHNDLFAILED"},
{CO_E_FAILEDTOGETWINDIR,"CO_E_FAILEDTOGETWINDIR"},
{CO_E_PATHTOOLONG,"CO_E_PATHTOOLONG"},
{CO_E_FAILEDTOGENUUID,"CO_E_FAILEDTOGENUUID"},
{CO_E_FAILEDTOCREATEFILE,"CO_E_FAILEDTOCREATEFILE"},
{CO_E_FAILEDTOCLOSEHANDLE,"CO_E_FAILEDTOCLOSEHANDLE"},
{CO_E_EXCEEDSYSACLLIMIT,"CO_E_EXCEEDSYSACLLIMIT"},
{CO_E_ACESINWRONGORDER,"CO_E_ACESINWRONGORDER"},
{CO_E_INCOMPATIBLESTREAMVERSION,"CO_E_INCOMPATIBLESTREAMVERSION"},
{CO_E_FAILEDTOOPENPROCESSTOKEN,"CO_E_FAILEDTOOPENPROCESSTOKEN"},
{CO_E_DECODEFAILED,"CO_E_DECODEFAILED"},
{CO_E_ACNOTINITIALIZED,"CO_E_ACNOTINITIALIZED"},
{CO_E_CANCEL_DISABLED,"CO_E_CANCEL_DISABLED"},
{RPC_E_UNEXPECTED,"RPC_E_UNEXPECTED"},
{ERROR_AUDITING_DISABLED,"ERROR_AUDITING_DISABLED"},
{ERROR_ALL_SIDS_FILTERED,"ERROR_ALL_SIDS_FILTERED"},
{NTE_BAD_UID,"NTE_BAD_UID"},
{NTE_BAD_HASH,"NTE_BAD_HASH"},
{NTE_BAD_KEY,"NTE_BAD_KEY"},
{NTE_BAD_LEN,"NTE_BAD_LEN"},
{NTE_BAD_DATA,"NTE_BAD_DATA"},
{NTE_BAD_SIGNATURE,"NTE_BAD_SIGNATURE"},
{NTE_BAD_VER,"NTE_BAD_VER"},
{NTE_BAD_ALGID,"NTE_BAD_ALGID"},
{NTE_BAD_FLAGS,"NTE_BAD_FLAGS"},
{NTE_BAD_TYPE,"NTE_BAD_TYPE"},
{NTE_BAD_KEY_STATE,"NTE_BAD_KEY_STATE"},
{NTE_BAD_HASH_STATE,"NTE_BAD_HASH_STATE"},
{NTE_NO_KEY,"NTE_NO_KEY"},
{NTE_NO_MEMORY,"NTE_NO_MEMORY"},
{NTE_EXISTS,"NTE_EXISTS"},
{NTE_PERM,"NTE_PERM"},
{NTE_NOT_FOUND,"NTE_NOT_FOUND"},
{NTE_DOUBLE_ENCRYPT,"NTE_DOUBLE_ENCRYPT"},
{NTE_BAD_PROVIDER,"NTE_BAD_PROVIDER"},
{NTE_BAD_PROV_TYPE,"NTE_BAD_PROV_TYPE"},
{NTE_BAD_PUBLIC_KEY,"NTE_BAD_PUBLIC_KEY"},
{NTE_BAD_KEYSET,"NTE_BAD_KEYSET"},
{NTE_PROV_TYPE_NOT_DEF,"NTE_PROV_TYPE_NOT_DEF"},
{NTE_PROV_TYPE_ENTRY_BAD,"NTE_PROV_TYPE_ENTRY_BAD"},
{NTE_KEYSET_NOT_DEF,"NTE_KEYSET_NOT_DEF"},
{NTE_KEYSET_ENTRY_BAD,"NTE_KEYSET_ENTRY_BAD"},
{NTE_PROV_TYPE_NO_MATCH,"NTE_PROV_TYPE_NO_MATCH"},
{NTE_SIGNATURE_FILE_BAD,"NTE_SIGNATURE_FILE_BAD"},
{NTE_PROVIDER_DLL_FAIL,"NTE_PROVIDER_DLL_FAIL"},
{NTE_PROV_DLL_NOT_FOUND,"NTE_PROV_DLL_NOT_FOUND"},
{NTE_BAD_KEYSET_PARAM,"NTE_BAD_KEYSET_PARAM"},
{NTE_FAIL,"NTE_FAIL"},
{NTE_SYS_ERR,"NTE_SYS_ERR"},
{NTE_SILENT_CONTEXT,"NTE_SILENT_CONTEXT"},
{NTE_TOKEN_KEYSET_STORAGE_FULL,"NTE_TOKEN_KEYSET_STORAGE_FULL"},
{NTE_TEMPORARY_PROFILE,"NTE_TEMPORARY_PROFILE"},
{NTE_FIXEDPARAMETER,"NTE_FIXEDPARAMETER"},
{SEC_E_INSUFFICIENT_MEMORY,"SEC_E_INSUFFICIENT_MEMORY"},
{SEC_E_INVALID_HANDLE,"SEC_E_INVALID_HANDLE"},
{SEC_E_UNSUPPORTED_FUNCTION,"SEC_E_UNSUPPORTED_FUNCTION"},
{SEC_E_TARGET_UNKNOWN,"SEC_E_TARGET_UNKNOWN"},
{SEC_E_INTERNAL_ERROR,"SEC_E_INTERNAL_ERROR"},
{SEC_E_SECPKG_NOT_FOUND,"SEC_E_SECPKG_NOT_FOUND"},
{SEC_E_NOT_OWNER,"SEC_E_NOT_OWNER"},
{SEC_E_CANNOT_INSTALL,"SEC_E_CANNOT_INSTALL"},
{SEC_E_INVALID_TOKEN,"SEC_E_INVALID_TOKEN"},
{SEC_E_CANNOT_PACK,"SEC_E_CANNOT_PACK"},
{SEC_E_QOP_NOT_SUPPORTED,"SEC_E_QOP_NOT_SUPPORTED"},
{SEC_E_NO_IMPERSONATION,"SEC_E_NO_IMPERSONATION"},
{SEC_E_LOGON_DENIED,"SEC_E_LOGON_DENIED"},
{SEC_E_UNKNOWN_CREDENTIALS,"SEC_E_UNKNOWN_CREDENTIALS"},
{SEC_E_NO_CREDENTIALS,"SEC_E_NO_CREDENTIALS"},
{SEC_E_MESSAGE_ALTERED,"SEC_E_MESSAGE_ALTERED"},
{SEC_E_OUT_OF_SEQUENCE,"SEC_E_OUT_OF_SEQUENCE"},
{SEC_E_NO_AUTHENTICATING_AUTHORITY,"SEC_E_NO_AUTHENTICATING_AUTHORITY"},
{SEC_I_CONTINUE_NEEDED,"SEC_I_CONTINUE_NEEDED"},
{SEC_I_COMPLETE_NEEDED,"SEC_I_COMPLETE_NEEDED"},
{SEC_I_COMPLETE_AND_CONTINUE,"SEC_I_COMPLETE_AND_CONTINUE"},
{SEC_I_LOCAL_LOGON,"SEC_I_LOCAL_LOGON"},
{SEC_E_BAD_PKGID,"SEC_E_BAD_PKGID"},
{SEC_E_CONTEXT_EXPIRED,"SEC_E_CONTEXT_EXPIRED"},
{SEC_I_CONTEXT_EXPIRED,"SEC_I_CONTEXT_EXPIRED"},
{SEC_E_INCOMPLETE_MESSAGE,"SEC_E_INCOMPLETE_MESSAGE"},
{SEC_E_INCOMPLETE_CREDENTIALS,"SEC_E_INCOMPLETE_CREDENTIALS"},
{SEC_E_BUFFER_TOO_SMALL,"SEC_E_BUFFER_TOO_SMALL"},
{SEC_I_INCOMPLETE_CREDENTIALS,"SEC_I_INCOMPLETE_CREDENTIALS"},
{SEC_I_RENEGOTIATE,"SEC_I_RENEGOTIATE"},
{SEC_E_WRONG_PRINCIPAL,"SEC_E_WRONG_PRINCIPAL"},
{SEC_I_NO_LSA_CONTEXT,"SEC_I_NO_LSA_CONTEXT"},
{SEC_E_TIME_SKEW,"SEC_E_TIME_SKEW"},
{SEC_E_UNTRUSTED_ROOT,"SEC_E_UNTRUSTED_ROOT"},
{SEC_E_ILLEGAL_MESSAGE,"SEC_E_ILLEGAL_MESSAGE"},
{SEC_E_CERT_UNKNOWN,"SEC_E_CERT_UNKNOWN"},
{SEC_E_CERT_EXPIRED,"SEC_E_CERT_EXPIRED"},
{SEC_E_ENCRYPT_FAILURE,"SEC_E_ENCRYPT_FAILURE"},
{SEC_E_DECRYPT_FAILURE,"SEC_E_DECRYPT_FAILURE"},
{SEC_E_ALGORITHM_MISMATCH,"SEC_E_ALGORITHM_MISMATCH"},
{SEC_E_SECURITY_QOS_FAILED,"SEC_E_SECURITY_QOS_FAILED"},
{SEC_E_UNFINISHED_CONTEXT_DELETED,"SEC_E_UNFINISHED_CONTEXT_DELETED"},
{SEC_E_NO_TGT_REPLY,"SEC_E_NO_TGT_REPLY"},
{SEC_E_NO_IP_ADDRESSES,"SEC_E_NO_IP_ADDRESSES"},
{SEC_E_WRONG_CREDENTIAL_HANDLE,"SEC_E_WRONG_CREDENTIAL_HANDLE"},
{SEC_E_CRYPTO_SYSTEM_INVALID,"SEC_E_CRYPTO_SYSTEM_INVALID"},
{SEC_E_MAX_REFERRALS_EXCEEDED,"SEC_E_MAX_REFERRALS_EXCEEDED"},
{SEC_E_MUST_BE_KDC,"SEC_E_MUST_BE_KDC"},
{SEC_E_STRONG_CRYPTO_NOT_SUPPORTED,"SEC_E_STRONG_CRYPTO_NOT_SUPPORTED"},
{SEC_E_TOO_MANY_PRINCIPALS,"SEC_E_TOO_MANY_PRINCIPALS"},
{SEC_E_NO_PA_DATA,"SEC_E_NO_PA_DATA"},
{SEC_E_PKINIT_NAME_MISMATCH,"SEC_E_PKINIT_NAME_MISMATCH"},
{SEC_E_SMARTCARD_LOGON_REQUIRED,"SEC_E_SMARTCARD_LOGON_REQUIRED"},
{SEC_E_SHUTDOWN_IN_PROGRESS,"SEC_E_SHUTDOWN_IN_PROGRESS"},
{SEC_E_KDC_INVALID_REQUEST,"SEC_E_KDC_INVALID_REQUEST"},
{SEC_E_KDC_UNABLE_TO_REFER,"SEC_E_KDC_UNABLE_TO_REFER"},
{SEC_E_KDC_UNKNOWN_ETYPE,"SEC_E_KDC_UNKNOWN_ETYPE"},
{SEC_E_UNSUPPORTED_PREAUTH,"SEC_E_UNSUPPORTED_PREAUTH"},
{SEC_E_DELEGATION_REQUIRED,"SEC_E_DELEGATION_REQUIRED"},
{SEC_E_BAD_BINDINGS,"SEC_E_BAD_BINDINGS"},
{SEC_E_MULTIPLE_ACCOUNTS,"SEC_E_MULTIPLE_ACCOUNTS"},
{SEC_E_NO_KERB_KEY,"SEC_E_NO_KERB_KEY"},
{SEC_E_NO_SPM,"SEC_E_NO_SPM"},
{SEC_E_NOT_SUPPORTED,"SEC_E_NOT_SUPPORTED"},
{CRYPT_E_MSG_ERROR,"CRYPT_E_MSG_ERROR"},
{CRYPT_E_UNKNOWN_ALGO,"CRYPT_E_UNKNOWN_ALGO"},
{CRYPT_E_OID_FORMAT,"CRYPT_E_OID_FORMAT"},
{CRYPT_E_INVALID_MSG_TYPE,"CRYPT_E_INVALID_MSG_TYPE"},
{CRYPT_E_UNEXPECTED_ENCODING,"CRYPT_E_UNEXPECTED_ENCODING"},
{CRYPT_E_AUTH_ATTR_MISSING,"CRYPT_E_AUTH_ATTR_MISSING"},
{CRYPT_E_HASH_VALUE,"CRYPT_E_HASH_VALUE"},
{CRYPT_E_INVALID_INDEX,"CRYPT_E_INVALID_INDEX"},
{CRYPT_E_ALREADY_DECRYPTED,"CRYPT_E_ALREADY_DECRYPTED"},
{CRYPT_E_NOT_DECRYPTED,"CRYPT_E_NOT_DECRYPTED"},
{CRYPT_E_RECIPIENT_NOT_FOUND,"CRYPT_E_RECIPIENT_NOT_FOUND"},
{CRYPT_E_CONTROL_TYPE,"CRYPT_E_CONTROL_TYPE"},
{CRYPT_E_ISSUER_SERIALNUMBER,"CRYPT_E_ISSUER_SERIALNUMBER"},
{CRYPT_E_SIGNER_NOT_FOUND,"CRYPT_E_SIGNER_NOT_FOUND"},
{CRYPT_E_ATTRIBUTES_MISSING,"CRYPT_E_ATTRIBUTES_MISSING"},
{CRYPT_E_STREAM_MSG_NOT_READY,"CRYPT_E_STREAM_MSG_NOT_READY"},
{CRYPT_E_STREAM_INSUFFICIENT_DATA,"CRYPT_E_STREAM_INSUFFICIENT_DATA"},
{CRYPT_I_NEW_PROTECTION_REQUIRED,"CRYPT_I_NEW_PROTECTION_REQUIRED"},
{CRYPT_E_BAD_LEN,"CRYPT_E_BAD_LEN"},
{CRYPT_E_BAD_ENCODE,"CRYPT_E_BAD_ENCODE"},
{CRYPT_E_FILE_ERROR,"CRYPT_E_FILE_ERROR"},
{CRYPT_E_NOT_FOUND,"CRYPT_E_NOT_FOUND"},
{CRYPT_E_EXISTS,"CRYPT_E_EXISTS"},
{CRYPT_E_NO_PROVIDER,"CRYPT_E_NO_PROVIDER"},
{CRYPT_E_SELF_SIGNED,"CRYPT_E_SELF_SIGNED"},
{CRYPT_E_DELETED_PREV,"CRYPT_E_DELETED_PREV"},
{CRYPT_E_NO_MATCH,"CRYPT_E_NO_MATCH"},
{CRYPT_E_UNEXPECTED_MSG_TYPE,"CRYPT_E_UNEXPECTED_MSG_TYPE"},
{CRYPT_E_NO_KEY_PROPERTY,"CRYPT_E_NO_KEY_PROPERTY"},
{CRYPT_E_NO_DECRYPT_CERT,"CRYPT_E_NO_DECRYPT_CERT"},
{CRYPT_E_BAD_MSG,"CRYPT_E_BAD_MSG"},
{CRYPT_E_NO_SIGNER,"CRYPT_E_NO_SIGNER"},
{CRYPT_E_PENDING_CLOSE,"CRYPT_E_PENDING_CLOSE"},
{CRYPT_E_REVOKED,"CRYPT_E_REVOKED"},
{CRYPT_E_NO_REVOCATION_DLL,"CRYPT_E_NO_REVOCATION_DLL"},
{CRYPT_E_NO_REVOCATION_CHECK,"CRYPT_E_NO_REVOCATION_CHECK"},
{CRYPT_E_REVOCATION_OFFLINE,"CRYPT_E_REVOCATION_OFFLINE"},
{CRYPT_E_NOT_IN_REVOCATION_DATABASE,"CRYPT_E_NOT_IN_REVOCATION_DATABASE"},
{CRYPT_E_INVALID_NUMERIC_STRING,"CRYPT_E_INVALID_NUMERIC_STRING"},
{CRYPT_E_INVALID_PRINTABLE_STRING,"CRYPT_E_INVALID_PRINTABLE_STRING"},
{CRYPT_E_INVALID_IA5_STRING,"CRYPT_E_INVALID_IA5_STRING"},
{CRYPT_E_INVALID_X500_STRING,"CRYPT_E_INVALID_X500_STRING"},
{CRYPT_E_NOT_CHAR_STRING,"CRYPT_E_NOT_CHAR_STRING"},
{CRYPT_E_FILERESIZED,"CRYPT_E_FILERESIZED"},
{CRYPT_E_SECURITY_SETTINGS,"CRYPT_E_SECURITY_SETTINGS"},
{CRYPT_E_NO_VERIFY_USAGE_DLL,"CRYPT_E_NO_VERIFY_USAGE_DLL"},
{CRYPT_E_NO_VERIFY_USAGE_CHECK,"CRYPT_E_NO_VERIFY_USAGE_CHECK"},
{CRYPT_E_VERIFY_USAGE_OFFLINE,"CRYPT_E_VERIFY_USAGE_OFFLINE"},
{CRYPT_E_NOT_IN_CTL,"CRYPT_E_NOT_IN_CTL"},
{CRYPT_E_NO_TRUSTED_SIGNER,"CRYPT_E_NO_TRUSTED_SIGNER"},
{CRYPT_E_MISSING_PUBKEY_PARA,"CRYPT_E_MISSING_PUBKEY_PARA"},
{CRYPT_E_OSS_ERROR,"CRYPT_E_OSS_ERROR"},
{OSS_MORE_BUF,"OSS_MORE_BUF"},
{OSS_NEGATIVE_UINTEGER,"OSS_NEGATIVE_UINTEGER"},
{OSS_PDU_RANGE,"OSS_PDU_RANGE"},
{OSS_MORE_INPUT,"OSS_MORE_INPUT"},
{OSS_DATA_ERROR,"OSS_DATA_ERROR"},
{OSS_BAD_ARG,"OSS_BAD_ARG"},
{OSS_BAD_VERSION,"OSS_BAD_VERSION"},
{OSS_OUT_MEMORY,"OSS_OUT_MEMORY"},
{OSS_PDU_MISMATCH,"OSS_PDU_MISMATCH"},
{OSS_LIMITED,"OSS_LIMITED"},
{OSS_BAD_PTR,"OSS_BAD_PTR"},
{OSS_BAD_TIME,"OSS_BAD_TIME"},
{OSS_INDEFINITE_NOT_SUPPORTED,"OSS_INDEFINITE_NOT_SUPPORTED"},
{OSS_MEM_ERROR,"OSS_MEM_ERROR"},
{OSS_BAD_TABLE,"OSS_BAD_TABLE"},
{OSS_TOO_LONG,"OSS_TOO_LONG"},
{OSS_CONSTRAINT_VIOLATED,"OSS_CONSTRAINT_VIOLATED"},
{OSS_FATAL_ERROR,"OSS_FATAL_ERROR"},
{OSS_ACCESS_SERIALIZATION_ERROR,"OSS_ACCESS_SERIALIZATION_ERROR"},
{OSS_NULL_TBL,"OSS_NULL_TBL"},
{OSS_NULL_FCN,"OSS_NULL_FCN"},
{OSS_BAD_ENCRULES,"OSS_BAD_ENCRULES"},
{OSS_UNAVAIL_ENCRULES,"OSS_UNAVAIL_ENCRULES"},
{OSS_CANT_OPEN_TRACE_WINDOW,"OSS_CANT_OPEN_TRACE_WINDOW"},
{OSS_UNIMPLEMENTED,"OSS_UNIMPLEMENTED"},
{OSS_OID_DLL_NOT_LINKED,"OSS_OID_DLL_NOT_LINKED"},
{OSS_CANT_OPEN_TRACE_FILE,"OSS_CANT_OPEN_TRACE_FILE"},
{OSS_TRACE_FILE_ALREADY_OPEN,"OSS_TRACE_FILE_ALREADY_OPEN"},
{OSS_TABLE_MISMATCH,"OSS_TABLE_MISMATCH"},
{OSS_TYPE_NOT_SUPPORTED,"OSS_TYPE_NOT_SUPPORTED"},
{OSS_REAL_DLL_NOT_LINKED,"OSS_REAL_DLL_NOT_LINKED"},
{OSS_REAL_CODE_NOT_LINKED,"OSS_REAL_CODE_NOT_LINKED"},
{OSS_OUT_OF_RANGE,"OSS_OUT_OF_RANGE"},
{OSS_COPIER_DLL_NOT_LINKED,"OSS_COPIER_DLL_NOT_LINKED"},
{OSS_CONSTRAINT_DLL_NOT_LINKED,"OSS_CONSTRAINT_DLL_NOT_LINKED"},
{OSS_COMPARATOR_DLL_NOT_LINKED,"OSS_COMPARATOR_DLL_NOT_LINKED"},
{OSS_COMPARATOR_CODE_NOT_LINKED,"OSS_COMPARATOR_CODE_NOT_LINKED"},
{OSS_MEM_MGR_DLL_NOT_LINKED,"OSS_MEM_MGR_DLL_NOT_LINKED"},
{OSS_PDV_DLL_NOT_LINKED,"OSS_PDV_DLL_NOT_LINKED"},
{OSS_PDV_CODE_NOT_LINKED,"OSS_PDV_CODE_NOT_LINKED"},
{OSS_API_DLL_NOT_LINKED,"OSS_API_DLL_NOT_LINKED"},
{OSS_BERDER_DLL_NOT_LINKED,"OSS_BERDER_DLL_NOT_LINKED"},
{OSS_PER_DLL_NOT_LINKED,"OSS_PER_DLL_NOT_LINKED"},
{OSS_OPEN_TYPE_ERROR,"OSS_OPEN_TYPE_ERROR"},
{OSS_MUTEX_NOT_CREATED,"OSS_MUTEX_NOT_CREATED"},
{OSS_CANT_CLOSE_TRACE_FILE,"OSS_CANT_CLOSE_TRACE_FILE"},
{CRYPT_E_ASN1_ERROR,"CRYPT_E_ASN1_ERROR"},
{CRYPT_E_ASN1_INTERNAL,"CRYPT_E_ASN1_INTERNAL"},
{CRYPT_E_ASN1_EOD,"CRYPT_E_ASN1_EOD"},
{CRYPT_E_ASN1_CORRUPT,"CRYPT_E_ASN1_CORRUPT"},
{CRYPT_E_ASN1_LARGE,"CRYPT_E_ASN1_LARGE"},
{CRYPT_E_ASN1_CONSTRAINT,"CRYPT_E_ASN1_CONSTRAINT"},
{CRYPT_E_ASN1_MEMORY,"CRYPT_E_ASN1_MEMORY"},
{CRYPT_E_ASN1_OVERFLOW,"CRYPT_E_ASN1_OVERFLOW"},
{CRYPT_E_ASN1_BADPDU,"CRYPT_E_ASN1_BADPDU"},
{CRYPT_E_ASN1_BADARGS,"CRYPT_E_ASN1_BADARGS"},
{CRYPT_E_ASN1_BADREAL,"CRYPT_E_ASN1_BADREAL"},
{CRYPT_E_ASN1_BADTAG,"CRYPT_E_ASN1_BADTAG"},
{CRYPT_E_ASN1_CHOICE,"CRYPT_E_ASN1_CHOICE"},
{CRYPT_E_ASN1_RULE,"CRYPT_E_ASN1_RULE"},
{CRYPT_E_ASN1_UTF8,"CRYPT_E_ASN1_UTF8"},
{CRYPT_E_ASN1_PDU_TYPE,"CRYPT_E_ASN1_PDU_TYPE"},
{CRYPT_E_ASN1_NYI,"CRYPT_E_ASN1_NYI"},
{CRYPT_E_ASN1_EXTENDED,"CRYPT_E_ASN1_EXTENDED"},
{CRYPT_E_ASN1_NOEOD,"CRYPT_E_ASN1_NOEOD"},
{CERTSRV_E_BAD_REQUESTSUBJECT,"CERTSRV_E_BAD_REQUESTSUBJECT"},
{CERTSRV_E_NO_REQUEST,"CERTSRV_E_NO_REQUEST"},
{CERTSRV_E_BAD_REQUESTSTATUS,"CERTSRV_E_BAD_REQUESTSTATUS"},
{CERTSRV_E_PROPERTY_EMPTY,"CERTSRV_E_PROPERTY_EMPTY"},
{CERTSRV_E_INVALID_CA_CERTIFICATE,"CERTSRV_E_INVALID_CA_CERTIFICATE"},
{CERTSRV_E_SERVER_SUSPENDED,"CERTSRV_E_SERVER_SUSPENDED"},
{CERTSRV_E_ENCODING_LENGTH,"CERTSRV_E_ENCODING_LENGTH"},
{CERTSRV_E_ROLECONFLICT,"CERTSRV_E_ROLECONFLICT"},
{CERTSRV_E_RESTRICTEDOFFICER,"CERTSRV_E_RESTRICTEDOFFICER"},
{CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED,"CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED"},
{CERTSRV_E_NO_VALID_KRA,"CERTSRV_E_NO_VALID_KRA"},
{CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL,"CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL"},
{CERTSRV_E_NO_CAADMIN_DEFINED,"CERTSRV_E_NO_CAADMIN_DEFINED"},
{CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE,"CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE"},
{CERTSRV_E_NO_DB_SESSIONS,"CERTSRV_E_NO_DB_SESSIONS"},
{CERTSRV_E_ALIGNMENT_FAULT,"CERTSRV_E_ALIGNMENT_FAULT"},
{CERTSRV_E_ENROLL_DENIED,"CERTSRV_E_ENROLL_DENIED"},
{CERTSRV_E_TEMPLATE_DENIED,"CERTSRV_E_TEMPLATE_DENIED"},
{CERTSRV_E_UNSUPPORTED_CERT_TYPE,"CERTSRV_E_UNSUPPORTED_CERT_TYPE"},
{CERTSRV_E_NO_CERT_TYPE,"CERTSRV_E_NO_CERT_TYPE"},
{CERTSRV_E_TEMPLATE_CONFLICT,"CERTSRV_E_TEMPLATE_CONFLICT"},
{CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED,"CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED"},
{CERTSRV_E_ARCHIVED_KEY_REQUIRED,"CERTSRV_E_ARCHIVED_KEY_REQUIRED"},
{CERTSRV_E_SMIME_REQUIRED,"CERTSRV_E_SMIME_REQUIRED"},
{CERTSRV_E_BAD_RENEWAL_SUBJECT,"CERTSRV_E_BAD_RENEWAL_SUBJECT"},
{CERTSRV_E_BAD_TEMPLATE_VERSION,"CERTSRV_E_BAD_TEMPLATE_VERSION"},
{CERTSRV_E_TEMPLATE_POLICY_REQUIRED,"CERTSRV_E_TEMPLATE_POLICY_REQUIRED"},
{CERTSRV_E_SIGNATURE_POLICY_REQUIRED,"CERTSRV_E_SIGNATURE_POLICY_REQUIRED"},
{CERTSRV_E_SIGNATURE_COUNT,"CERTSRV_E_SIGNATURE_COUNT"},
{CERTSRV_E_SIGNATURE_REJECTED,"CERTSRV_E_SIGNATURE_REJECTED"},
{CERTSRV_E_ISSUANCE_POLICY_REQUIRED,"CERTSRV_E_ISSUANCE_POLICY_REQUIRED"},
{CERTSRV_E_SUBJECT_UPN_REQUIRED,"CERTSRV_E_SUBJECT_UPN_REQUIRED"},
{CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED,"CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED"},
{CERTSRV_E_SUBJECT_DNS_REQUIRED,"CERTSRV_E_SUBJECT_DNS_REQUIRED"},
{CERTSRV_E_ARCHIVED_KEY_UNEXPECTED,"CERTSRV_E_ARCHIVED_KEY_UNEXPECTED"},
{CERTSRV_E_KEY_LENGTH,"CERTSRV_E_KEY_LENGTH"},
{XENROLL_E_KEY_NOT_EXPORTABLE,"XENROLL_E_KEY_NOT_EXPORTABLE"},
{XENROLL_E_CANNOT_ADD_ROOT_CERT,"XENROLL_E_CANNOT_ADD_ROOT_CERT"},
{XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND,"XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND"},
{XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH,"XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH"},
{XENROLL_E_RESPONSE_KA_HASH_MISMATCH,"XENROLL_E_RESPONSE_KA_HASH_MISMATCH"},
{XENROLL_E_KEYSPEC_SMIME_MISMATCH,"XENROLL_E_KEYSPEC_SMIME_MISMATCH"},
{TRUST_E_SYSTEM_ERROR,"TRUST_E_SYSTEM_ERROR"},
{TRUST_E_NO_SIGNER_CERT,"TRUST_E_NO_SIGNER_CERT"},
{TRUST_E_COUNTER_SIGNER,"TRUST_E_COUNTER_SIGNER"},
{TRUST_E_CERT_SIGNATURE,"TRUST_E_CERT_SIGNATURE"},
{TRUST_E_TIME_STAMP,"TRUST_E_TIME_STAMP"},
{TRUST_E_BAD_DIGEST,"TRUST_E_BAD_DIGEST"},
{TRUST_E_BASIC_CONSTRAINTS,"TRUST_E_BASIC_CONSTRAINTS"},
{TRUST_E_FINANCIAL_CRITERIA,"TRUST_E_FINANCIAL_CRITERIA"},
{MSSIPOTF_E_OUTOFMEMRANGE,"MSSIPOTF_E_OUTOFMEMRANGE"},
{MSSIPOTF_E_CANTGETOBJECT,"MSSIPOTF_E_CANTGETOBJECT"},
{MSSIPOTF_E_NOHEADTABLE,"MSSIPOTF_E_NOHEADTABLE"},
{MSSIPOTF_E_BAD_MAGICNUMBER,"MSSIPOTF_E_BAD_MAGICNUMBER"},
{MSSIPOTF_E_BAD_OFFSET_TABLE,"MSSIPOTF_E_BAD_OFFSET_TABLE"},
{MSSIPOTF_E_TABLE_TAGORDER,"MSSIPOTF_E_TABLE_TAGORDER"},
{MSSIPOTF_E_TABLE_LONGWORD,"MSSIPOTF_E_TABLE_LONGWORD"},
{MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT,"MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT"},
{MSSIPOTF_E_TABLES_OVERLAP,"MSSIPOTF_E_TABLES_OVERLAP"},
{MSSIPOTF_E_TABLE_PADBYTES,"MSSIPOTF_E_TABLE_PADBYTES"},
{MSSIPOTF_E_FILETOOSMALL,"MSSIPOTF_E_FILETOOSMALL"},
{MSSIPOTF_E_TABLE_CHECKSUM,"MSSIPOTF_E_TABLE_CHECKSUM"},
{MSSIPOTF_E_FILE_CHECKSUM,"MSSIPOTF_E_FILE_CHECKSUM"},
{MSSIPOTF_E_FAILED_POLICY,"MSSIPOTF_E_FAILED_POLICY"},
{MSSIPOTF_E_FAILED_HINTS_CHECK,"MSSIPOTF_E_FAILED_HINTS_CHECK"},
{MSSIPOTF_E_NOT_OPENTYPE,"MSSIPOTF_E_NOT_OPENTYPE"},
{MSSIPOTF_E_FILE,"MSSIPOTF_E_FILE"},
{MSSIPOTF_E_CRYPT,"MSSIPOTF_E_CRYPT"},
{MSSIPOTF_E_BADVERSION,"MSSIPOTF_E_BADVERSION"},
{MSSIPOTF_E_DSIG_STRUCTURE,"MSSIPOTF_E_DSIG_STRUCTURE"},
{MSSIPOTF_E_PCONST_CHECK,"MSSIPOTF_E_PCONST_CHECK"},
{MSSIPOTF_E_STRUCTURE,"MSSIPOTF_E_STRUCTURE"},
{NTE_OP_OK,"NTE_OP_OK"},
{TRUST_E_PROVIDER_UNKNOWN,"TRUST_E_PROVIDER_UNKNOWN"},
{TRUST_E_ACTION_UNKNOWN,"TRUST_E_ACTION_UNKNOWN"},
{TRUST_E_SUBJECT_FORM_UNKNOWN,"TRUST_E_SUBJECT_FORM_UNKNOWN"},
{TRUST_E_SUBJECT_NOT_TRUSTED,"TRUST_E_SUBJECT_NOT_TRUSTED"},
{DIGSIG_E_ENCODE,"DIGSIG_E_ENCODE"},
{DIGSIG_E_DECODE,"DIGSIG_E_DECODE"},
{DIGSIG_E_EXTENSIBILITY,"DIGSIG_E_EXTENSIBILITY"},
{DIGSIG_E_CRYPTO,"DIGSIG_E_CRYPTO"},
{PERSIST_E_SIZEDEFINITE,"PERSIST_E_SIZEDEFINITE"},
{PERSIST_E_SIZEINDEFINITE,"PERSIST_E_SIZEINDEFINITE"},
{PERSIST_E_NOTSELFSIZING,"PERSIST_E_NOTSELFSIZING"},
{TRUST_E_NOSIGNATURE,"TRUST_E_NOSIGNATURE"},
{CERT_E_EXPIRED,"CERT_E_EXPIRED"},
{CERT_E_VALIDITYPERIODNESTING,"CERT_E_VALIDITYPERIODNESTING"},
{CERT_E_ROLE,"CERT_E_ROLE"},
{CERT_E_PATHLENCONST,"CERT_E_PATHLENCONST"},
{CERT_E_CRITICAL,"CERT_E_CRITICAL"},
{CERT_E_PURPOSE,"CERT_E_PURPOSE"},
{CERT_E_ISSUERCHAINING,"CERT_E_ISSUERCHAINING"},
{CERT_E_MALFORMED,"CERT_E_MALFORMED"},
{CERT_E_UNTRUSTEDROOT,"CERT_E_UNTRUSTEDROOT"},
{CERT_E_CHAINING,"CERT_E_CHAINING"},
{TRUST_E_FAIL,"TRUST_E_FAIL"},
{CERT_E_REVOKED,"CERT_E_REVOKED"},
{CERT_E_UNTRUSTEDTESTROOT,"CERT_E_UNTRUSTEDTESTROOT"},
{CERT_E_REVOCATION_FAILURE,"CERT_E_REVOCATION_FAILURE"},
{CERT_E_CN_NO_MATCH,"CERT_E_CN_NO_MATCH"},
{CERT_E_WRONG_USAGE,"CERT_E_WRONG_USAGE"},
{TRUST_E_EXPLICIT_DISTRUST,"TRUST_E_EXPLICIT_DISTRUST"},
{CERT_E_UNTRUSTEDCA,"CERT_E_UNTRUSTEDCA"},
{CERT_E_INVALID_POLICY,"CERT_E_INVALID_POLICY"},
{CERT_E_INVALID_NAME,"CERT_E_INVALID_NAME"},
{SPAPI_E_EXPECTED_SECTION_NAME,"SPAPI_E_EXPECTED_SECTION_NAME"},
{SPAPI_E_BAD_SECTION_NAME_LINE,"SPAPI_E_BAD_SECTION_NAME_LINE"},
{SPAPI_E_SECTION_NAME_TOO_LONG,"SPAPI_E_SECTION_NAME_TOO_LONG"},
{SPAPI_E_GENERAL_SYNTAX,"SPAPI_E_GENERAL_SYNTAX"},
{SPAPI_E_WRONG_INF_STYLE,"SPAPI_E_WRONG_INF_STYLE"},
{SPAPI_E_SECTION_NOT_FOUND,"SPAPI_E_SECTION_NOT_FOUND"},
{SPAPI_E_LINE_NOT_FOUND,"SPAPI_E_LINE_NOT_FOUND"},
{SPAPI_E_NO_BACKUP,"SPAPI_E_NO_BACKUP"},
{SPAPI_E_NO_ASSOCIATED_CLASS,"SPAPI_E_NO_ASSOCIATED_CLASS"},
{SPAPI_E_CLASS_MISMATCH,"SPAPI_E_CLASS_MISMATCH"},
{SPAPI_E_DUPLICATE_FOUND,"SPAPI_E_DUPLICATE_FOUND"},
{SPAPI_E_NO_DRIVER_SELECTED,"SPAPI_E_NO_DRIVER_SELECTED"},
{SPAPI_E_KEY_DOES_NOT_EXIST,"SPAPI_E_KEY_DOES_NOT_EXIST"},
{SPAPI_E_INVALID_DEVINST_NAME,"SPAPI_E_INVALID_DEVINST_NAME"},
{SPAPI_E_INVALID_CLASS,"SPAPI_E_INVALID_CLASS"},
{SPAPI_E_DEVINST_ALREADY_EXISTS,"SPAPI_E_DEVINST_ALREADY_EXISTS"},
{SPAPI_E_DEVINFO_NOT_REGISTERED,"SPAPI_E_DEVINFO_NOT_REGISTERED"},
{SPAPI_E_INVALID_REG_PROPERTY,"SPAPI_E_INVALID_REG_PROPERTY"},
{SPAPI_E_NO_INF,"SPAPI_E_NO_INF"},
{SPAPI_E_NO_SUCH_DEVINST,"SPAPI_E_NO_SUCH_DEVINST"},
{SPAPI_E_CANT_LOAD_CLASS_ICON,"SPAPI_E_CANT_LOAD_CLASS_ICON"},
{SPAPI_E_INVALID_CLASS_INSTALLER,"SPAPI_E_INVALID_CLASS_INSTALLER"},
{SPAPI_E_DI_DO_DEFAULT,"SPAPI_E_DI_DO_DEFAULT"},
{SPAPI_E_DI_NOFILECOPY,"SPAPI_E_DI_NOFILECOPY"},
{SPAPI_E_INVALID_HWPROFILE,"SPAPI_E_INVALID_HWPROFILE"},
{SPAPI_E_NO_DEVICE_SELECTED,"SPAPI_E_NO_DEVICE_SELECTED"},
{SPAPI_E_DEVINFO_LIST_LOCKED,"SPAPI_E_DEVINFO_LIST_LOCKED"},
{SPAPI_E_DEVINFO_DATA_LOCKED,"SPAPI_E_DEVINFO_DATA_LOCKED"},
{SPAPI_E_DI_BAD_PATH,"SPAPI_E_DI_BAD_PATH"},
{SPAPI_E_NO_CLASSINSTALL_PARAMS,"SPAPI_E_NO_CLASSINSTALL_PARAMS"},
{SPAPI_E_FILEQUEUE_LOCKED,"SPAPI_E_FILEQUEUE_LOCKED"},
{SPAPI_E_BAD_SERVICE_INSTALLSECT,"SPAPI_E_BAD_SERVICE_INSTALLSECT"},
{SPAPI_E_NO_CLASS_DRIVER_LIST,"SPAPI_E_NO_CLASS_DRIVER_LIST"},
{SPAPI_E_NO_ASSOCIATED_SERVICE,"SPAPI_E_NO_ASSOCIATED_SERVICE"},
{SPAPI_E_NO_DEFAULT_DEVICE_INTERFACE,"SPAPI_E_NO_DEFAULT_DEVICE_INTERFACE"},
{SPAPI_E_DEVICE_INTERFACE_ACTIVE,"SPAPI_E_DEVICE_INTERFACE_ACTIVE"},
{SPAPI_E_DEVICE_INTERFACE_REMOVED,"SPAPI_E_DEVICE_INTERFACE_REMOVED"},
{SPAPI_E_BAD_INTERFACE_INSTALLSECT,"SPAPI_E_BAD_INTERFACE_INSTALLSECT"},
{SPAPI_E_NO_SUCH_INTERFACE_CLASS,"SPAPI_E_NO_SUCH_INTERFACE_CLASS"},
{SPAPI_E_INVALID_REFERENCE_STRING,"SPAPI_E_INVALID_REFERENCE_STRING"},
{SPAPI_E_INVALID_MACHINENAME,"SPAPI_E_INVALID_MACHINENAME"},
{SPAPI_E_REMOTE_COMM_FAILURE,"SPAPI_E_REMOTE_COMM_FAILURE"},
{SPAPI_E_MACHINE_UNAVAILABLE,"SPAPI_E_MACHINE_UNAVAILABLE"},
{SPAPI_E_NO_CONFIGMGR_SERVICES,"SPAPI_E_NO_CONFIGMGR_SERVICES"},
{SPAPI_E_INVALID_PROPPAGE_PROVIDER,"SPAPI_E_INVALID_PROPPAGE_PROVIDER"},
{SPAPI_E_NO_SUCH_DEVICE_INTERFACE,"SPAPI_E_NO_SUCH_DEVICE_INTERFACE"},
{SPAPI_E_DI_POSTPROCESSING_REQUIRED,"SPAPI_E_DI_POSTPROCESSING_REQUIRED"},
{SPAPI_E_INVALID_COINSTALLER,"SPAPI_E_INVALID_COINSTALLER"},
{SPAPI_E_NO_COMPAT_DRIVERS,"SPAPI_E_NO_COMPAT_DRIVERS"},
{SPAPI_E_NO_DEVICE_ICON,"SPAPI_E_NO_DEVICE_ICON"},
{SPAPI_E_INVALID_INF_LOGCONFIG,"SPAPI_E_INVALID_INF_LOGCONFIG"},
{SPAPI_E_DI_DONT_INSTALL,"SPAPI_E_DI_DONT_INSTALL"},
{SPAPI_E_INVALID_FILTER_DRIVER,"SPAPI_E_INVALID_FILTER_DRIVER"},
{SPAPI_E_NON_WINDOWS_NT_DRIVER,"SPAPI_E_NON_WINDOWS_NT_DRIVER"},
{SPAPI_E_NON_WINDOWS_DRIVER,"SPAPI_E_NON_WINDOWS_DRIVER"},
{SPAPI_E_NO_CATALOG_FOR_OEM_INF,"SPAPI_E_NO_CATALOG_FOR_OEM_INF"},
{SPAPI_E_DEVINSTALL_QUEUE_NONNATIVE,"SPAPI_E_DEVINSTALL_QUEUE_NONNATIVE"},
{SPAPI_E_NOT_DISABLEABLE,"SPAPI_E_NOT_DISABLEABLE"},
{SPAPI_E_CANT_REMOVE_DEVINST,"SPAPI_E_CANT_REMOVE_DEVINST"},
{SPAPI_E_INVALID_TARGET,"SPAPI_E_INVALID_TARGET"},
{SPAPI_E_DRIVER_NONNATIVE,"SPAPI_E_DRIVER_NONNATIVE"},
{SPAPI_E_IN_WOW64,"SPAPI_E_IN_WOW64"},
{SPAPI_E_SET_SYSTEM_RESTORE_POINT,"SPAPI_E_SET_SYSTEM_RESTORE_POINT"},
{SPAPI_E_INCORRECTLY_COPIED_INF,"SPAPI_E_INCORRECTLY_COPIED_INF"},
{SPAPI_E_SCE_DISABLED,"SPAPI_E_SCE_DISABLED"},
{SPAPI_E_ERROR_NOT_INSTALLED,"SPAPI_E_ERROR_NOT_INSTALLED"},
{SCARD_S_SUCCESS,"SCARD_S_SUCCESS"},
{SCARD_F_INTERNAL_ERROR,"SCARD_F_INTERNAL_ERROR"},
{SCARD_E_CANCELLED,"SCARD_E_CANCELLED"},
{SCARD_E_INVALID_HANDLE,"SCARD_E_INVALID_HANDLE"},
{SCARD_E_INVALID_PARAMETER,"SCARD_E_INVALID_PARAMETER"},
{SCARD_E_INVALID_TARGET,"SCARD_E_INVALID_TARGET"},
{SCARD_E_NO_MEMORY,"SCARD_E_NO_MEMORY"},
{SCARD_F_WAITED_TOO_LONG,"SCARD_F_WAITED_TOO_LONG"},
{SCARD_E_INSUFFICIENT_BUFFER,"SCARD_E_INSUFFICIENT_BUFFER"},
{SCARD_E_UNKNOWN_READER,"SCARD_E_UNKNOWN_READER"},
{SCARD_E_TIMEOUT,"SCARD_E_TIMEOUT"},
{SCARD_E_SHARING_VIOLATION,"SCARD_E_SHARING_VIOLATION"},
{SCARD_E_NO_SMARTCARD,"SCARD_E_NO_SMARTCARD"},
{SCARD_E_UNKNOWN_CARD,"SCARD_E_UNKNOWN_CARD"},
{SCARD_E_CANT_DISPOSE,"SCARD_E_CANT_DISPOSE"},
{SCARD_E_PROTO_MISMATCH,"SCARD_E_PROTO_MISMATCH"},
{SCARD_E_NOT_READY,"SCARD_E_NOT_READY"},
{SCARD_E_INVALID_VALUE,"SCARD_E_INVALID_VALUE"},
{SCARD_E_SYSTEM_CANCELLED,"SCARD_E_SYSTEM_CANCELLED"},
{SCARD_F_COMM_ERROR,"SCARD_F_COMM_ERROR"},
{SCARD_F_UNKNOWN_ERROR,"SCARD_F_UNKNOWN_ERROR"},
{SCARD_E_INVALID_ATR,"SCARD_E_INVALID_ATR"},
{SCARD_E_NOT_TRANSACTED,"SCARD_E_NOT_TRANSACTED"},
{SCARD_E_READER_UNAVAILABLE,"SCARD_E_READER_UNAVAILABLE"},
{SCARD_P_SHUTDOWN,"SCARD_P_SHUTDOWN"},
{SCARD_E_PCI_TOO_SMALL,"SCARD_E_PCI_TOO_SMALL"},
{SCARD_E_READER_UNSUPPORTED,"SCARD_E_READER_UNSUPPORTED"},
{SCARD_E_DUPLICATE_READER,"SCARD_E_DUPLICATE_READER"},
{SCARD_E_CARD_UNSUPPORTED,"SCARD_E_CARD_UNSUPPORTED"},
{SCARD_E_NO_SERVICE,"SCARD_E_NO_SERVICE"},
{SCARD_E_SERVICE_STOPPED,"SCARD_E_SERVICE_STOPPED"},
{SCARD_E_UNEXPECTED,"SCARD_E_UNEXPECTED"},
{SCARD_E_ICC_INSTALLATION,"SCARD_E_ICC_INSTALLATION"},
{SCARD_E_ICC_CREATEORDER,"SCARD_E_ICC_CREATEORDER"},
{SCARD_E_UNSUPPORTED_FEATURE,"SCARD_E_UNSUPPORTED_FEATURE"},
{SCARD_E_DIR_NOT_FOUND,"SCARD_E_DIR_NOT_FOUND"},
{SCARD_E_FILE_NOT_FOUND,"SCARD_E_FILE_NOT_FOUND"},
{SCARD_E_NO_DIR,"SCARD_E_NO_DIR"},
{SCARD_E_NO_FILE,"SCARD_E_NO_FILE"},
{SCARD_E_NO_ACCESS,"SCARD_E_NO_ACCESS"},
{SCARD_E_WRITE_TOO_MANY,"SCARD_E_WRITE_TOO_MANY"},
{SCARD_E_BAD_SEEK,"SCARD_E_BAD_SEEK"},
{SCARD_E_INVALID_CHV,"SCARD_E_INVALID_CHV"},
{SCARD_E_UNKNOWN_RES_MNG,"SCARD_E_UNKNOWN_RES_MNG"},
{SCARD_E_NO_SUCH_CERTIFICATE,"SCARD_E_NO_SUCH_CERTIFICATE"},
{SCARD_E_CERTIFICATE_UNAVAILABLE,"SCARD_E_CERTIFICATE_UNAVAILABLE"},
{SCARD_E_NO_READERS_AVAILABLE,"SCARD_E_NO_READERS_AVAILABLE"},
{SCARD_E_COMM_DATA_LOST,"SCARD_E_COMM_DATA_LOST"},
{SCARD_E_NO_KEY_CONTAINER,"SCARD_E_NO_KEY_CONTAINER"},
{SCARD_W_UNSUPPORTED_CARD,"SCARD_W_UNSUPPORTED_CARD"},
{SCARD_W_UNRESPONSIVE_CARD,"SCARD_W_UNRESPONSIVE_CARD"},
{SCARD_W_UNPOWERED_CARD,"SCARD_W_UNPOWERED_CARD"},
{SCARD_W_RESET_CARD,"SCARD_W_RESET_CARD"},
{SCARD_W_REMOVED_CARD,"SCARD_W_REMOVED_CARD"},
{SCARD_W_SECURITY_VIOLATION,"SCARD_W_SECURITY_VIOLATION"},
{SCARD_W_WRONG_CHV,"SCARD_W_WRONG_CHV"},
{SCARD_W_CHV_BLOCKED,"SCARD_W_CHV_BLOCKED"},
{SCARD_W_EOF,"SCARD_W_EOF"},
{SCARD_W_CANCELLED_BY_USER,"SCARD_W_CANCELLED_BY_USER"},
{SCARD_W_CARD_NOT_AUTHENTICATED,"SCARD_W_CARD_NOT_AUTHENTICATED"},
{COMADMIN_E_OBJECTERRORS,"COMADMIN_E_OBJECTERRORS"},
{COMADMIN_E_OBJECTINVALID,"COMADMIN_E_OBJECTINVALID"},
{COMADMIN_E_KEYMISSING,"COMADMIN_E_KEYMISSING"},
{COMADMIN_E_ALREADYINSTALLED,"COMADMIN_E_ALREADYINSTALLED"},
{COMADMIN_E_APP_FILE_WRITEFAIL,"COMADMIN_E_APP_FILE_WRITEFAIL"},
{COMADMIN_E_APP_FILE_READFAIL,"COMADMIN_E_APP_FILE_READFAIL"},
{COMADMIN_E_APP_FILE_VERSION,"COMADMIN_E_APP_FILE_VERSION"},
{COMADMIN_E_BADPATH,"COMADMIN_E_BADPATH"},
{COMADMIN_E_APPLICATIONEXISTS,"COMADMIN_E_APPLICATIONEXISTS"},
{COMADMIN_E_ROLEEXISTS,"COMADMIN_E_ROLEEXISTS"},
{COMADMIN_E_CANTCOPYFILE,"COMADMIN_E_CANTCOPYFILE"},
{COMADMIN_E_NOUSER,"COMADMIN_E_NOUSER"},
{COMADMIN_E_INVALIDUSERIDS,"COMADMIN_E_INVALIDUSERIDS"},
{COMADMIN_E_NOREGISTRYCLSID,"COMADMIN_E_NOREGISTRYCLSID"},
{COMADMIN_E_BADREGISTRYPROGID,"COMADMIN_E_BADREGISTRYPROGID"},
{COMADMIN_E_AUTHENTICATIONLEVEL,"COMADMIN_E_AUTHENTICATIONLEVEL"},
{COMADMIN_E_USERPASSWDNOTVALID,"COMADMIN_E_USERPASSWDNOTVALID"},
{COMADMIN_E_CLSIDORIIDMISMATCH,"COMADMIN_E_CLSIDORIIDMISMATCH"},
{COMADMIN_E_REMOTEINTERFACE,"COMADMIN_E_REMOTEINTERFACE"},
{COMADMIN_E_DLLREGISTERSERVER,"COMADMIN_E_DLLREGISTERSERVER"},
{COMADMIN_E_NOSERVERSHARE,"COMADMIN_E_NOSERVERSHARE"},
{COMADMIN_E_DLLLOADFAILED,"COMADMIN_E_DLLLOADFAILED"},
{COMADMIN_E_BADREGISTRYLIBID,"COMADMIN_E_BADREGISTRYLIBID"},
{COMADMIN_E_APPDIRNOTFOUND,"COMADMIN_E_APPDIRNOTFOUND"},
{COMADMIN_E_REGISTRARFAILED,"COMADMIN_E_REGISTRARFAILED"},
{COMADMIN_E_COMPFILE_DOESNOTEXIST,"COMADMIN_E_COMPFILE_DOESNOTEXIST"},
{COMADMIN_E_COMPFILE_LOADDLLFAIL,"COMADMIN_E_COMPFILE_LOADDLLFAIL"},
{COMADMIN_E_COMPFILE_GETCLASSOBJ,"COMADMIN_E_COMPFILE_GETCLASSOBJ"},
{COMADMIN_E_COMPFILE_CLASSNOTAVAIL,"COMADMIN_E_COMPFILE_CLASSNOTAVAIL"},
{COMADMIN_E_COMPFILE_BADTLB,"COMADMIN_E_COMPFILE_BADTLB"},
{COMADMIN_E_COMPFILE_NOTINSTALLABLE,"COMADMIN_E_COMPFILE_NOTINSTALLABLE"},
{COMADMIN_E_NOTCHANGEABLE,"COMADMIN_E_NOTCHANGEABLE"},
{COMADMIN_E_NOTDELETEABLE,"COMADMIN_E_NOTDELETEABLE"},
{COMADMIN_E_SESSION,"COMADMIN_E_SESSION"},
{COMADMIN_E_COMP_MOVE_LOCKED,"COMADMIN_E_COMP_MOVE_LOCKED"},
{COMADMIN_E_COMP_MOVE_BAD_DEST,"COMADMIN_E_COMP_MOVE_BAD_DEST"},
{COMADMIN_E_REGISTERTLB,"COMADMIN_E_REGISTERTLB"},
{COMADMIN_E_SYSTEMAPP,"COMADMIN_E_SYSTEMAPP"},
{COMADMIN_E_COMPFILE_NOREGISTRAR,"COMADMIN_E_COMPFILE_NOREGISTRAR"},
{COMADMIN_E_COREQCOMPINSTALLED,"COMADMIN_E_COREQCOMPINSTALLED"},
{COMADMIN_E_SERVICENOTINSTALLED,"COMADMIN_E_SERVICENOTINSTALLED"},
{COMADMIN_E_PROPERTYSAVEFAILED,"COMADMIN_E_PROPERTYSAVEFAILED"},
{COMADMIN_E_OBJECTEXISTS,"COMADMIN_E_OBJECTEXISTS"},
{COMADMIN_E_COMPONENTEXISTS,"COMADMIN_E_COMPONENTEXISTS"},
{COMADMIN_E_REGFILE_CORRUPT,"COMADMIN_E_REGFILE_CORRUPT"},
{COMADMIN_E_PROPERTY_OVERFLOW,"COMADMIN_E_PROPERTY_OVERFLOW"},
{COMADMIN_E_NOTINREGISTRY,"COMADMIN_E_NOTINREGISTRY"},
{COMADMIN_E_OBJECTNOTPOOLABLE,"COMADMIN_E_OBJECTNOTPOOLABLE"},
{COMADMIN_E_APPLID_MATCHES_CLSID,"COMADMIN_E_APPLID_MATCHES_CLSID"},
{COMADMIN_E_ROLE_DOES_NOT_EXIST,"COMADMIN_E_ROLE_DOES_NOT_EXIST"},
{COMADMIN_E_START_APP_NEEDS_COMPONENTS,"COMADMIN_E_START_APP_NEEDS_COMPONENTS"},
{COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM,"COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM"},
{COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY,"COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY"},
{COMADMIN_E_CAN_NOT_START_APP,"COMADMIN_E_CAN_NOT_START_APP"},
{COMADMIN_E_CAN_NOT_EXPORT_SYS_APP,"COMADMIN_E_CAN_NOT_EXPORT_SYS_APP"},
{COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT,"COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT"},
{COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER,"COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER"},
{COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE,"COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE"},
{COMADMIN_E_BASE_PARTITION_ONLY,"COMADMIN_E_BASE_PARTITION_ONLY"},
{COMADMIN_E_START_APP_DISABLED,"COMADMIN_E_START_APP_DISABLED"},
{COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME,"COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME"},
{COMADMIN_E_CAT_INVALID_PARTITION_NAME,"COMADMIN_E_CAT_INVALID_PARTITION_NAME"},
{COMADMIN_E_CAT_PARTITION_IN_USE,"COMADMIN_E_CAT_PARTITION_IN_USE"},
{COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES,"COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES"},
{COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED,"COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED"},
{COMADMIN_E_AMBIGUOUS_APPLICATION_NAME,"COMADMIN_E_AMBIGUOUS_APPLICATION_NAME"},
{COMADMIN_E_AMBIGUOUS_PARTITION_NAME,"COMADMIN_E_AMBIGUOUS_PARTITION_NAME"},
{COMADMIN_E_REGDB_NOTINITIALIZED,"COMADMIN_E_REGDB_NOTINITIALIZED"},
{COMADMIN_E_REGDB_NOTOPEN,"COMADMIN_E_REGDB_NOTOPEN"},
{COMADMIN_E_REGDB_SYSTEMERR,"COMADMIN_E_REGDB_SYSTEMERR"},
{COMADMIN_E_REGDB_ALREADYRUNNING,"COMADMIN_E_REGDB_ALREADYRUNNING"},
{COMADMIN_E_MIG_VERSIONNOTSUPPORTED,"COMADMIN_E_MIG_VERSIONNOTSUPPORTED"},
{COMADMIN_E_MIG_SCHEMANOTFOUND,"COMADMIN_E_MIG_SCHEMANOTFOUND"},
{COMADMIN_E_CAT_BITNESSMISMATCH,"COMADMIN_E_CAT_BITNESSMISMATCH"},
{COMADMIN_E_CAT_UNACCEPTABLEBITNESS,"COMADMIN_E_CAT_UNACCEPTABLEBITNESS"},
{COMADMIN_E_CAT_WRONGAPPBITNESS,"COMADMIN_E_CAT_WRONGAPPBITNESS"},
{COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED,"COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED"},
{COMADMIN_E_CAT_SERVERFAULT,"COMADMIN_E_CAT_SERVERFAULT"},
{COMQC_E_APPLICATION_NOT_QUEUED,"COMQC_E_APPLICATION_NOT_QUEUED"},
{COMQC_E_NO_QUEUEABLE_INTERFACES,"COMQC_E_NO_QUEUEABLE_INTERFACES"},
{COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE,"COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE"},
{COMQC_E_NO_IPERSISTSTREAM,"COMQC_E_NO_IPERSISTSTREAM"},
{COMQC_E_BAD_MESSAGE,"COMQC_E_BAD_MESSAGE"},
{COMQC_E_UNAUTHENTICATED,"COMQC_E_UNAUTHENTICATED"},
{COMQC_E_UNTRUSTED_ENQUEUER,"COMQC_E_UNTRUSTED_ENQUEUER"},
{MSDTC_E_DUPLICATE_RESOURCE,"MSDTC_E_DUPLICATE_RESOURCE"},
{COMADMIN_E_OBJECT_PARENT_MISSING,"COMADMIN_E_OBJECT_PARENT_MISSING"},
{COMADMIN_E_OBJECT_DOES_NOT_EXIST,"COMADMIN_E_OBJECT_DOES_NOT_EXIST"},
{COMADMIN_E_APP_NOT_RUNNING,"COMADMIN_E_APP_NOT_RUNNING"},
{COMADMIN_E_INVALID_PARTITION,"COMADMIN_E_INVALID_PARTITION"},
{COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE,"COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE"},
{COMADMIN_E_USER_IN_SET,"COMADMIN_E_USER_IN_SET"},
{COMADMIN_E_CANTRECYCLELIBRARYAPPS,"COMADMIN_E_CANTRECYCLELIBRARYAPPS"},
{COMADMIN_E_CANTRECYCLESERVICEAPPS,"COMADMIN_E_CANTRECYCLESERVICEAPPS"},
{COMADMIN_E_PROCESSALREADYRECYCLED,"COMADMIN_E_PROCESSALREADYRECYCLED"},
{COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED,"COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED"},
{COMADMIN_E_CANTMAKEINPROCSERVICE,"COMADMIN_E_CANTMAKEINPROCSERVICE"},
{COMADMIN_E_PROGIDINUSEBYCLSID,"COMADMIN_E_PROGIDINUSEBYCLSID"},
{COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET,"COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET"},
{COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED,"COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED"},
{COMADMIN_E_PARTITION_ACCESSDENIED,"COMADMIN_E_PARTITION_ACCESSDENIED"},
{COMADMIN_E_PARTITION_MSI_ONLY,"COMADMIN_E_PARTITION_MSI_ONLY"},
{COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT,"COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT"},
{COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS,"COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS"},
{COMADMIN_E_COMP_MOVE_SOURCE,"COMADMIN_E_COMP_MOVE_SOURCE"},
{COMADMIN_E_COMP_MOVE_DEST,"COMADMIN_E_COMP_MOVE_DEST"},
{COMADMIN_E_COMP_MOVE_PRIVATE,"COMADMIN_E_COMP_MOVE_PRIVATE"},
{COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET,"COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET"},
{COMADMIN_E_CANNOT_ALIAS_EVENTCLASS,"COMADMIN_E_CANNOT_ALIAS_EVENTCLASS"},
{COMADMIN_E_PRIVATE_ACCESSDENIED,"COMADMIN_E_PRIVATE_ACCESSDENIED"},
{COMADMIN_E_SAFERINVALID,"COMADMIN_E_SAFERINVALID"},
{COMADMIN_E_REGISTRY_ACCESSDENIED,"COMADMIN_E_REGISTRY_ACCESSDENIED"},
{COMADMIN_E_CAT_DUPLICATE_PARTITION_SET_NAME,"COMADMIN_E_CAT_DUPLICATE_PARTITION_SET_NAME"},
{COMADMIN_E_CAT_INVALID_PARTITION_SET_NAME,"COMADMIN_E_CAT_INVALID_PARTITION_SET_NAME"},
{MAPI_E_CALL_FAILED,"MAPI_E_CALL_FAILED"},
{MAPI_E_NOT_ENOUGH_MEMORY,"MAPI_E_NOT_ENOUGH_MEMORY"},
{MAPI_E_INVALID_PARAMETER,"MAPI_E_INVALID_PARAMETER"},
{MAPI_E_INTERFACE_NOT_SUPPORTED,"MAPI_E_INTERFACE_NOT_SUPPORTED"},
{MAPI_E_NO_ACCESS,"MAPI_E_NO_ACCESS"},
{MAPI_E_NO_SUPPORT,"MAPI_E_NO_SUPPORT"},
{MAPI_E_STRING_TOO_LONG,"MAPI_E_STRING_TOO_LONG"},
{MAPI_E_UNKNOWN_FLAGS,"MAPI_E_UNKNOWN_FLAGS"},
{MAPI_E_INVALID_ENTRYID,"MAPI_E_INVALID_ENTRYID"},
{MAPI_E_INVALID_OBJECT,"MAPI_E_INVALID_OBJECT"},
{MAPI_E_OBJECT_CHANGED,"MAPI_E_OBJECT_CHANGED"},
{MAPI_E_OBJECT_DELETED,"MAPI_E_OBJECT_DELETED"},
{MAPI_E_BUSY,"MAPI_E_BUSY"},
{MAPI_E_NOT_ENOUGH_DISK,"MAPI_E_NOT_ENOUGH_DISK"},
{MAPI_E_NOT_ENOUGH_RESOURCES,"MAPI_E_NOT_ENOUGH_RESOURCES"},
{MAPI_E_NOT_FOUND,"MAPI_E_NOT_FOUND"},
{MAPI_E_VERSION,"MAPI_E_VERSION"},
{MAPI_E_LOGON_FAILED,"MAPI_E_LOGON_FAILED"},
{MAPI_E_SESSION_LIMIT,"MAPI_E_SESSION_LIMIT"},
{MAPI_E_USER_CANCEL,"MAPI_E_USER_CANCEL"},
{MAPI_E_UNABLE_TO_ABORT,"MAPI_E_UNABLE_TO_ABORT"},
{MAPI_E_NETWORK_ERROR,"MAPI_E_NETWORK_ERROR"},
{MAPI_E_DISK_ERROR,"MAPI_E_DISK_ERROR"},
{MAPI_E_TOO_COMPLEX,"MAPI_E_TOO_COMPLEX"},
{MAPI_E_BAD_COLUMN,"MAPI_E_BAD_COLUMN"},
{MAPI_E_EXTENDED_ERROR,"MAPI_E_EXTENDED_ERROR"},
{MAPI_E_COMPUTED,"MAPI_E_COMPUTED"},
{MAPI_E_CORRUPT_DATA,"MAPI_E_CORRUPT_DATA"},
{MAPI_E_UNCONFIGURED,"MAPI_E_UNCONFIGURED"},
{MAPI_E_FAILONEPROVIDER,"MAPI_E_FAILONEPROVIDER"},
{MAPI_E_UNKNOWN_CPID,"MAPI_E_UNKNOWN_CPID"},
{MAPI_E_UNKNOWN_LCID,"MAPI_E_UNKNOWN_LCID"},
{MAPI_E_PASSWORD_CHANGE_REQUIRED,"MAPI_E_PASSWORD_CHANGE_REQUIRED"},
{MAPI_E_PASSWORD_EXPIRED,"MAPI_E_PASSWORD_EXPIRED"},
{MAPI_E_INVALID_WORKSTATION_ACCOUNT,"MAPI_E_INVALID_WORKSTATION_ACCOUNT"},
{MAPI_E_INVALID_ACCESS_TIME,"MAPI_E_INVALID_ACCESS_TIME"},
{MAPI_E_ACCOUNT_DISABLED,"MAPI_E_ACCOUNT_DISABLED"},
{MAPI_E_END_OF_SESSION,"MAPI_E_END_OF_SESSION"},
{MAPI_E_UNKNOWN_ENTRYID,"MAPI_E_UNKNOWN_ENTRYID"},
{MAPI_E_MISSING_REQUIRED_COLUMN,"MAPI_E_MISSING_REQUIRED_COLUMN"},
{MAPI_E_BAD_VALUE,"MAPI_E_BAD_VALUE"},
{MAPI_E_INVALID_TYPE,"MAPI_E_INVALID_TYPE"},
{MAPI_E_TYPE_NO_SUPPORT,"MAPI_E_TYPE_NO_SUPPORT"},
{MAPI_E_UNEXPECTED_TYPE,"MAPI_E_UNEXPECTED_TYPE"},
{MAPI_E_TOO_BIG,"MAPI_E_TOO_BIG"},
{MAPI_E_DECLINE_COPY,"MAPI_E_DECLINE_COPY"},
{MAPI_E_UNEXPECTED_ID,"MAPI_E_UNEXPECTED_ID"},
{MAPI_E_UNABLE_TO_COMPLETE,"MAPI_E_UNABLE_TO_COMPLETE"},
{MAPI_E_TIMEOUT,"MAPI_E_TIMEOUT"},
{MAPI_E_TABLE_EMPTY,"MAPI_E_TABLE_EMPTY"},
{MAPI_E_TABLE_TOO_BIG,"MAPI_E_TABLE_TOO_BIG"},
{MAPI_E_INVALID_BOOKMARK,"MAPI_E_INVALID_BOOKMARK"},
{MAPI_E_WAIT,"MAPI_E_WAIT"},
{MAPI_E_CANCEL,"MAPI_E_CANCEL"},
{MAPI_E_NOT_ME,"MAPI_E_NOT_ME"},
{MAPI_E_CORRUPT_STORE,"MAPI_E_CORRUPT_STORE"},
{MAPI_E_NOT_IN_QUEUE,"MAPI_E_NOT_IN_QUEUE"},
{MAPI_E_NO_SUPPRESS,"MAPI_E_NO_SUPPRESS"},
{MAPI_E_COLLISION,"MAPI_E_COLLISION"},
{MAPI_E_NOT_INITIALIZED,"MAPI_E_NOT_INITIALIZED"},
{MAPI_E_NON_STANDARD,"MAPI_E_NON_STANDARD"},
{MAPI_E_NO_RECIPIENTS,"MAPI_E_NO_RECIPIENTS"},
{MAPI_E_SUBMITTED,"MAPI_E_SUBMITTED"},
{MAPI_E_HAS_FOLDERS,"MAPI_E_HAS_FOLDERS"},
{MAPI_E_HAS_MESSAGES,"MAPI_E_HAS_MESSAGES"},
{MAPI_E_FOLDER_CYCLE,"MAPI_E_FOLDER_CYCLE"},
{MAPI_E_AMBIGUOUS_RECIP,"MAPI_E_AMBIGUOUS_RECIP"},
{0,0}};

EXTERN_C LPCSTR WINAPI LookupHresultA(HRESULT hResult)
{
    for( ULONG ulIndex = 0; allcodes[ulIndex].lpszDescription; ulIndex++ )
    {
        if( allcodes[ulIndex].hResult == hResult )
        {
            return allcodes[ulIndex].lpszDescription;
        }
    }

    static CHAR szMessageBuffer[256];
    FormatMessage( 
        FORMAT_MESSAGE_FROM_SYSTEM | 
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        hResult,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
        szMessageBuffer,
        sizeof(szMessageBuffer),
        NULL 
    );
    return szMessageBuffer;
}

